/***************************************************************************
 *                                                                         *
 *  Copyright 2003-2015 Avago Technologies.  All rights reserved.          *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiConfig.h
 *          Title:  SAS Firmware IOP Interface Config Structures and Definitions
 *     Programmer:  Steve Johnson
 *  Creation Date:  June 11, 2003
 *
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface config structures and
 * definitions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPICONFIG_H_INCLUDED
#define IOPICONFIG_H_INCLUDED

 /******************************************************************************
 *
 * CONFIG PAGE ALLOCATIONS
 *
 *    To prevent overlap in defining Config Pages between various projects, the
 *    following ranges have been defined.  NOTE: This won't help prevent
 *    overlap between different generations using the same range. That will
 *    still require coordination between the two teams. It's possible that a
 *    project will define a Config Page that is later needed by the general
 *    code or a different project.  There isn't much we can do about that.
 *    The main idea is to prevent the same Config Page from having multiple
 *    definitions which could cause confusion, break tools, etc.
 *
 *  Manufacturing Pages
 *    MPI       00 -  07
 *    SCS       08 -  31
 *
 *  Extended Manufacturing
 *    N/A        0 -  39 (Reserved)
 *    SCS       40 -  60
 *    SCS       61 - 219 (Reserved)
 *    ASD      220 - 229
 *    SBA      230 - 239
 *    OEM      240 - 255
 *
 ******************************************************************************/

/******************************************************************************
*
*      I O P I   C O N F I G   S T R U C T S   A N D   D E F I N I T I O N S
*
******************************************************************************/

/* Steve Schremmer said to put product specific page versions in this file. */
#define MPI2_EXTENDED_MANUFACTURING40_PAGEVERSION     (0x00)
#define MPI2_EXTENDED_MANUFACTURING41_PAGEVERSION     (0x00)
#define MPI2_EXTENDED_MANUFACTURING42_PAGEVERSION     (0x00)
#define MPI2_EXTENDED_MANUFACTURING43_PAGEVERSION     (0x00)
#define MPI2_EXTENDED_MANUFACTURING44_PAGEVERSION     (0x00)

/* GEN2 manufacturing page 3 is inserted here for customers who need to support both */
/* GEN2 and GEN3 with 1 header file.                                                */
/*****************************************************************************
*
*  Manufacturing Page 3 Config Page
*
*****************************************************************************/

#define IOC_MFG_PAGE_3_FLAG_FAST_SEEPROM (0x01)


#if  !defined(__LANGUAGE_ASM__)
typedef struct _IOC_PHY_GROUP
{
    U32                                 Misc;           /* 0x00 */
    U32                                 Sas1G1Low;      /* 0x04 */
    U32                                 Sas1G1High;     /* 0x08 */
    U32                                 Sas1G2Low;      /* 0x0C */
    U32                                 Sas1G2High;     /* 0x10 */
    U32                                 SasOobLow;      /* 0x14 */
    U32                                 SasOobHigh;     /* 0x18 */
    U32                                 Sas2G1Low;      /* 0x1C */
    U32                                 Sas2G1High;     /* 0x20 */
    U32                                 Sas2G2Low;      /* 0x24 */
    U32                                 Sas2G2High;     /* 0x28 */
    U32                                 Sas2G3Low;      /* 0x2C */
    U32                                 Sas2G3High;     /* 0x30 */
    U32                                 SataG1Low;      /* 0x34 */
    U32                                 SataG1High;     /* 0x38 */
    U32                                 SataG2Low;      /* 0x3C */
    U32                                 SataG2High;     /* 0x40 */
    U32                                 SataG3Low;      /* 0x44 */
    U32                                 SataG3High;     /* 0x48 */
    U32                                 SataOobLow;     /* 0x4C */
    U32                                 SataOobHigh;    /* 0x50 */
} IOC_PHY_GROUP, *PTR_IOC_PHY_GROUP;                    /* 0x54 bytes in size */

#define IOC_PHY_NUM_PHY_GROUPS                  4

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_3
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                         /* 00h */
    MPI2_CHIP_REVISION_ID               ChipId;                                         /* 04h */
    U32                                 Reserved1;                                      /* 08h */
    U32                                 Reserved2;                                      /* 08h */
    U32                                 Reserved3;                                      /* 08h */
    U32                                 Reserved4;                                      /* 08h */
    IOC_PHY_GROUP                       PhyGroup[IOC_PHY_NUM_PHY_GROUPS];               /* 18h (0x54*4=150 bytes) */
    U8                                  NumPhys;                                        /* 168h */
    U8                                  Flags;                                          /* 169h */
    U8                                  Reserved6;                                      /* 16Ah */
    U8                                  Reserved7;                                      /* 16Bh */
    U32                                 Phy[IOC_NUM_PHYS];                              /* 16Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_3, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_3;                          /* Size = 0x16C + 4*IOC_NUM_PHYS */

/* definition of phy group in Phy*/
#define IOC_PHY_PHY_GROUP_MASK                  (0x00000003)    /* Specify phy group to use this phy*/
#define IOC_PHY_PHY_GROUP_0                     (0x00000000)
#define IOC_PHY_PHY_GROUP_1                     (0x00000001)
#define IOC_PHY_PHY_GROUP_2                     (0x00000002)
#define IOC_PHY_PHY_GROUP_3                     (0x00000003)

/*Definition of InvRxData, InvTxData in Phy */
#define IOC_PHY_INV_RX_DATA                     (0x00000004)  /* GEN2 ONLY */
#define IOC_PHY_INV_TX_DATA                     (0x00000008)  /* GEN2 ONLY */

#define IOC_PHY_SAS_ONLY                        (0x00000010)
#define IOC_PHY_EN_SATA_PARTIAL_POW_MGMT        (0x00000020)  /* partial power management */
#define IOC_PHY_EN_SATA_SLUMBER_POW_MGMT        (0x00000040)  /* slumber power management */

/* definition of SAS SNW participation in Phy */
#define IOC_PHY_SAS_SNW1_PARTICIPATION          (0x00000100)
#define IOC_PHY_SAS_SNW2_PARTICIPATION          (0x00000200)
#define IOC_PHY_SAS_SNW3_PARTICIPATION          (0x00000400)

/* definition of SAS/SATA G in Phy */
#define IOC_PHY_G3_60_WITH_SSC                  (0x00040000)  /* 6.0 Gps */
#define IOC_PHY_G3_60_WITHOUT_SSC               (0x00080000)
#define IOC_PHY_G2_30_WITH_SSC                  (0x00100000)  /* 3.0 Gps */
#define IOC_PHY_G2_30_WITHOUT_SSC               (0x00200000)
#define IOC_PHY_G1_15_WITH_SSC                  (0x00400000)  /* 1.5 Gps */
#define IOC_PHY_G1_15_WITHOUT_SSC               (0x00800000)
#define IOC_PHY_SATA_SSC_EN                     (0x01000000)  /* Spread Spectrum Clocking */


/* Definition of Misc in PHY_GROUP */
#define IOC_PHY_GRP_MISC_RG_MASK                (0x0000c000)    /* Receiver PLL gain adjustment */
#define IOC_PHY_GRP_MISC_RG_SHIFT14             (14)
#define IOC_PHY_GRP_MISC_ALWAYS_WSYNC_EN        (0x00000001)    /* Always word synchronize */
#define IOC_PHY_GRP_MISC_GAIN_HIGH              (0xff000000)
#define IOC_PHY_GRP_MISC_GAIN_HIGH_SHIFT        (24)
#define IOC_PHY_GRP_MISC_GAIN_LOW               (0x00ff0000)
#define IOC_PHY_GRP_MISC_GAIN_LOW_SHIFT         (16)

/* Definition of post emphasis control for Sas/Sata (lower dword) in PHY_GROUP */
#define IOC_PHY_GRP_HIGH_EMPH_POST_MASK         (0x00000007)    /* output signal emphasis control - post cursor tap*/
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL0        (0x00000000)
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL1        (0x00000001)
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL2        (0x00000002)
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL3        (0x00000003)
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL4        (0x00000004)
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL5        (0x00000005)
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL6        (0x00000006)
#define IOC_PHY_GRP_HIGH_EMPH_POST_CTRL7        (0x00000007)

/* Definition of pre emphasis control for Sas/Sata (lower dword) in PHY_GROUP */
#define IOC_PHY_GRP_HIGH_EMPH_PRE_MASK          (0x00000018)    /* output signal emphasis control - pre cursor tap*/
#define IOC_PHY_GRP_HIGH_EMPH_PRE_SHIFT3        (3)
#define IOC_PHY_GRP_HIGH_EMPH_PRE_CTRL0         (0x00000000)
#define IOC_PHY_GRP_HIGH_EMPH_PRE_CTRL1         (0x00000008)
#define IOC_PHY_GRP_HIGH_EMPH_PRE_CTRL2         (0x00000010)

/* Definition of Fall Rise in Sas/Sata (lower dword) in PHY_GROUP */
#define IOC_PHY_GRP_HIGH_RISE_FALL_MASK         (0x000000e0)    /* Slew rate control for each lane, 000: min rise time*/
#define IOC_PHY_GRP_HIGH_RISE_FALL_MIN          (0x00000000)
#define IOC_PHY_GRP_HIGH_RISE_FALL_MAX          (0x000000e0)
#define IOC_PHY_GRP_HIGH_RISE_FALL_SHIFT5       (5)

/* Definition of VtxLo in Sas/Sata (lower dword) in PHY_GROUP */
#define IOC_PHY_GRP_HIGH_VTXLO_MASK             (0x0000f000)    /* output Voltage Swing control */
#define IOC_PHY_GRP_HIGH_VTXLO_SHIFT12          (12)

/* Definition of RxEq in Sas/Sata (higher dword) in PHY_GROUP */
#define IOC_PHY_GRP_LOW_RXEQ_MASK               (0x000003ff)    /* Receive Adaptive Equalization */

/* Definition of LosAdj in Sas/Sata (higher dword) in PHY_GROUP */
#define IOC_PHY_GRP_LOW_LOSADJ_MASK             (0x00070000)    /* Receive Loss of signal threshold adjust*/
#define IOC_PHY_GRP_LOW_LOSADJ_SHIFT16          (16)

/* Definition of RxZeroDat in Sas/Sata (higher dword) in PHY_GROUP */
#define IOC_PHY_GRP_LOW_RXZERODAT_MASK          (0x00180000)    /* Receive PLL Zero in Lock to Data */
#define IOC_PHY_GRP_LOW_RXZERODAT_SHIFT19       (19)

/* Definition of RxZeroRef in Sas/Sata (higher dword) in PHY_GROUP */
#define IOC_PHY_GRP_LOW_RXZEROREF_MASK          (0x00e00000)   /* Receve PLL Zero in Lock to Ref */
#define IOC_PHY_GRP_LOW_RXZEROREF_SHIFT21       (21)

/* Definition of PHY Flags */
#define IOC_PHY_FLAGS_OPTICAL_SUPPORT_ENABLE    (0x01)
#define IOC_PHY_FLAGS_TX_PHY_ADJUSTMENT         (0x02)

/*****************************************************************************
*
*  Manufacturing Page 6 Config Page
*
*****************************************************************************/

/*
 * The number of entries for this page needs to be the MAXIMUM number of GPIO's supported by the build.
 *
 *  Intruder = 84
 *  Ventura = 80
 *  Meteor = 56
 *  Falcon = 36
 *  Liberator = 32
 *
 * This value needs to be synchronized with the value used to
 * override MPI2_IO_UNIT_PAGE_3_GPIO_VAL_MAX.
 */
#ifndef IOC_MFG_PAGE6_GPIO_DEFS
#define IOC_MFG_PAGE6_GPIO_DEFS                     (MPI2_IO_UNIT_PAGE_3_GPIO_VAL_MAX)
#endif

/* GPIODefinition */
typedef struct _IOC_CFG_MFG_6_GPIO_DEF
{
    U8                                  FunctionCode;                                      /* 00h */
    U8                                  Flags;                                             /* 01h */
    U8                                  Param1;                                            /* 02h */
    U8                                  Param2;                                            /* 03h */
    U32                                 Param3;                                            /* 04h */
} IOC_CFG_MFG_6_GPIO_DEF, *PTR_IOC_CFG_MFG_6_GPIO_DEF;

/* All GPIO Definition Flags definitions */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_MASK                     (0x03)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_INPUT                    (0x00)    /* Pad may be used as an input */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_OUTPUT_OPEN_DRAIN        (0x01)    /* 1: High impedance, 0: Enabled, pull up required*/
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_OUTPUT_OPEN_SOURCE       (0x02)    /* 1: Enabled 0: High Impedance, pull down required */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_OUTPUT_PUSH_PULL         (0x03)    /* Always driving */

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_MASK                  (0x04)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_LOW                   (0x00)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_HIGH                  (IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_MASK)

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_BIDIR_ENABLE                 (0x08)

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_MASK                  (0x30)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIO                  (0x00)    /* Same for GPIO bank A & B */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_LED_DATA              (0x10)    /* Maps the full LED data bus. Same for GPIO bank A & B */

#if VENTURA
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIOA_PDB_PORT_RST    (0x20)    /* GPIO bank A - PDB 0 & 1 Port Reset selection */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIOA_LED_DATA0_7     (0x30)    /* GPIO bank A - LED data bits 0_7 selection */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIOB_PDB_CLKREQ_SIO  (0x20)    /* GPIO Bank B - PDB Clock Req & SGPIO sideband selection */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIOB_PDB_RST_ISTWI   (0x30)    /* GPIO Bank B - PDB Port Reset & ISTWI selection */
#else /* INTRUDER */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIOB_SIO             (0x20)    /* GPIO Bank B - SIO sideband selection */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIOB_ISTWI           (0x30)    /* GPIO Bank B - ISTWI selection */
#endif

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_SH                    (4)       /* Alt Data Sel shift */

/* GPIO Drive control (GPIODef Flags bits [7:6]) - Controls the GPIO current low to high. */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DRV_CTRL_MASK                (0xC0)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DRV_CTRL_100_OHM             (0x00)    /* 100 ohm impedance (lowest current) */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DRV_CTRL_66_OHM              (0x40)    /* 66 ohm impedance                   */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DRV_CTRL_50_OHM              (0x80)    /* 55 ohm impedance                   */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DRV_CTRL_33_OHM              (0xC0)    /* 33 ohm impedance (Highest current) */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DRV_CTRL_SH                  (6)       /* Drive control shift */

/* GPIO Function code definitions (8 bits) */
#define IOC_CFG_MFG_6_GPIO_FUNCTION_UNKNOWN                       (0x0)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DEFAULT                       (0x1)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_IR_DRIVE_FAULT                (0x2)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DA_SEP_RESET                  (0x3)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DA_SEP_DETECT                 (0x4)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_SEP_DETECT_AND_RESET          (0x5)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_UNUSED_6                      (0x6)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_UNUSED_7                      (0x7)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_BKPLN_MGMT_TYPE               (0x8)

#define IOC_CFG_MFG_6_GPIO_FUNCTION_DRIVE_ACTIVITY_LED            (0x9)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_PORT_STATUS_GREEN_LED         (0xA)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_PORT_STATUS_YELLOW_LED        (0xB)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_HEARTBEAT_LED                 (0xC)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DA_SEP_CONNECT                (0xD)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_I2C_MUX_RESET                 (0xE)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_MFG3_MUX_BIT                  (0xF)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_INTERRUPT                     (0x10)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_FAULT_INDICATION              (0x11)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_IBUTTON                       (0x12)
                                                                            /* FC 0x14-0x16 are used by SSS HBA */
#define IOC_CFG_MFG_6_GPIO_FUNCTION_SAS_HD_CABLE_MGMT             (0x17)    /* Mini SAS HD Cable Mgmt */

#define IOC_CFG_MFG_6_GPIO_FUNCTION_PCIE_X8LINK_LED               (0x19)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_OVERTEMP_LED                  (0x1A)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_WRITE_PROTECT                 (0x1C)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_BACKEND_PCIE_PERST            (0x1D)

#define IOC_CFG_MFG_6_GPIO_FUNCTION_EXT_DEBUG_TRIGGER             (0x20)

#define IOC_CFG_MFG_6_GPIO_FUNCTION_ASD_RESERVED                  (0xA0)    /* 0xA0 - 0xAF are reserved for ASD */

#define IOC_CFG_MFG_6_GPIO_FUNCTION_VENDOR_SPECIFIC               (0xC0)    /* 0xC0 - 0xFF are Vendor Specific */


/* Param1 definitions for INTERRUPT Function */
#define IOC_CFG_MFG_6_INT_PARAM1_TYPE_MASK                        (0x01)
#define IOC_CFG_MFG_6_INT_PARAM1_EDGE_SENSITIVE                   (0x00)
#define IOC_CFG_MFG_6_INT_PARAM1_LEVEL_SENSITIVE                  (0x01)

#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_MASK                      (0xF0)
#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_GENERIC                   (0x00)  /* For backward compatible */
#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_HD_CABLE_MGMT             (0x10)  /* HD Cable Management interrupt */
#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_ACTIVE_CABLE_OVERCURRENT  (0x20)  /* HD Active Cable over current interrupt */

/* Param1 definitions for PORT STATUS YELLOW LED Function */
#define IOC_CFG_MFG6_YLED_PARAM1_NO_LINK_EN                       (0x01)
#define IOC_CFG_MFG6_YLED_PARAM1_NONOPT_EN                        (0x02)
#define IOC_CFG_MFG6_YLED_PARAM1_OPT_EN                           (0x04)
#define IOC_CFG_MFG6_YLED_PARAM1_PRECEDENCE_MASK                  (0x08)
#define IOC_CFG_MFG6_YLED_PARAM1_PRECEDENCE_NO_LINK               (0x08)
#define IOC_CFG_MFG6_YLED_PARAM1_PRECEDENCE_OPT                   (0x00)

/* Param1 definitions for SAS HD CABLE MGMT Function */
#define IOC_CFG_MFG_6_HD_CABLE_MGMT_PARAM1_MOD_PRESENT            (0x00)  /* SAS HD cable module present signal */
#define IOC_CFG_MFG_6_HD_CABLE_MGMT_PARAM1_ACTIVE_PWR_ENABLE      (0x01)  /* Active Cable power enable signal */
#define IOC_CFG_MFG_6_HD_CABLE_MGMT_PARAM1_MGMT_PWR_ENABLE        (0x02)  /* Cable Mgmt interface power enable signal */

/* Param1 definitions for WRITE PROTECT Function */
#define IOC_CFG_MFG_6_WP_PARAM1_DISABLE_EEPROM_WP_ON_BOOT         (0x01)  /* Disable the EEPROM write protection on boot. */

/* Param1 definitions for BACKEND PCIE PERST Function */
#define IOC_CFG_MFG_6_PCIE_PERST_PARAM1_ALWAYS_MODIFY             (0xFF)

/* Virtual GPIO Definitions */
#define IOC_CFG_MFG_6_GPIO_VIRT_ALTSIO_RES0                       (0x28)    /* Virtual GPIO for AltSIOFunc Res0 (bit 8)*/
#define IOC_CFG_MFG_6_GPIO_VIRT_ALTSIO_RES1                       (0x29)    /* Virtual GPIO for AltSIOFunc Res1 (bit 9)*/


/* GPIO Defines */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_6
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U8                                  NumGPIO;                                           /* 04h */
    U8                                  Reserved1;                                         /* 05h */
    U16                                 Flags;                                             /* 06h */
    U32                                 Reserved2;                                         /* 08h */
    U32                                 Reserved3;                                         /* 0Ch */
    IOC_CFG_MFG_6_GPIO_DEF              GPIODefinition[IOC_MFG_PAGE6_GPIO_DEFS];           /* 10h */
} IOC_CONFIG_PAGE_MFG_PAGE_6, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_6;

#define IOC_CONFIG_PAGE_MFG6_FLAGS_HBLED_DISABLE                  (0x0001)  /* 1: Disable Heartbeat 0: Enable Heartbeat */
#define IOC_CONFIG_PAGE_MFG6_FLAGS_SLEW_RATE_FAST                 (0x0002)  /* 1: Fast slew rate 0: Slow slew rate */
#define IOC_CONFIG_PAGE_MFG6_FLAGS_RESERVED_18_31                 (0xFFFC)
#define IOC_CONFIG_PAGE_MFG6_FLAGS_SLEW_RATE_SH                   (0x0001)  /* Slew rate shift */



/*****************************************************************************
*
*  Manufacturing Page 8 Config Page
*
*****************************************************************************/
/* This page is obsolete. Extended Manufacturing Pages 42/43 are used instead. */


/*****************************************************************************
*
*  Manufacturing Page 9 Config Page
*
*****************************************************************************/

typedef struct
{
    U32 Maximum;        /* Maximum Number Of Resources allowed */
    U32 Decrement;      /* Amount Subtracted until Minimum is reached */
    U32 Minimum;        /* Minimum Number Of Resources allowed */
    U32 Actual;         /* Actual Number of Resources Allocated if Successful (May be zero in NVStore) */
} IOC_CONFIG_MFG9_RESOURCE;

typedef enum
{
    IOC_CFG_MFG9_RESOURCE_NUM_VFS                       = 0,    /* Number of Virtual Functions Supported (may be less than MAX) */
    IOC_CFG_MFG9_RESOURCE_NUM_VPS                       = 1,    /* Number of Virtual Ports Supported */
    IOC_CFG_MFG9_RESOURCE_HOST_CREDITS_PER_VF           = 2,    /* Number of Global Credits per VF */
    IOC_CFG_MFG9_RESOURCE_HI_PRIORITY_Q_DEPTH_PER_VF    = 3,    /* Number of HiPQ Credits per VF */
    IOC_CFG_MFG9_RESOURCE_TARGETS                       = 4,    /* Number of Targets supported */
    IOC_CFG_MFG9_RESOURCE_INITIATORS                    = 5,    /* Number of Initiators supported */
    IOC_CFG_MFG9_RESOURCE_TGT_CMD_BUFFERS_PER_VP        = 6,    /* Number of Target Cmd Buffers per VP*/
    IOC_CFG_MFG9_RESOURCE_EXPANDERS                     = 7,    /* Number of Expanders Supported */
    IOC_CFG_MFG9_RESOURCE_PHYS                          = 8,    /* Max number of phys supported per enclosure */
    IOC_CFG_MFG9_RESOURCE_ENCLOSURES                    = 9,    /* Max Enclosures supported */
    IOC_CFG_MFG9_RESOURCE_RING_BUF_SIZE                 =10,    /* Size of ring buffer in bytes */
    IOC_CFG_MFG9_RESOURCE_IR_BUFFER_SIZE                =11,    /* Size of RAID resync buffer in bytes */
    IOC_CFG_MFG9_RESOURCE_NUM_ROUTE_TABLE_ENTRIES       =12,    /* Number of expander route table entries */
#if COMPILE_OPTION_SAS_PORT_VIRT
    IOC_CFG_MFG9_RESOURCE_NUM_VPHYS_PER_VE              =13,    /* Number of VPhys per VE */
    IOC_CFG_MFG9_RESOURCE_NUM_VES                       =14,    /* Number of VE*/
#endif  /* COMPILE_OPTION_SAS_PORT_VIRT  */
    IOC_CFG_MFG9_RESOURCE_CHAIN_ELEMENTS_PER_VF         =15,    /* Number of Local Chain Buffer Elements per VF */
    IOC_CFG_MFG9_RESOURCE_SES_DIAG_PAGEBUFFER_SIZE      =16,    /* Number of bytes allocated to hold SES page buffers (recommended minimum of 512) */
#if VENTURA
    IOC_CFG_MFG9_RESOURCE_BACKEND_PCIE_DEVICES          =17,    /* Number of Backend PCIe Devices supported */
    IOC_CFG_MFG9_RESOURCE_NUM_IOS_PER_AHCI_DEV          =18,    /* Number of simultaneous IOs per backend AHCI device supported */
#endif
    /* This must always be the last member in this ENUM! */
    IOC_CFG_MFG9_NUMBER_OF_RESOURCES            /* Number of resources supported by this build */
} IOC_CONFIG_MFG9_RESOURCE_ENUM;

/*
 * Other RTConfig values and their sources:
 * HostFrameSize - Size of host frame from IOCInit
 * IOCRequestFrameSize - CONSTANT, Build Option
 * PLIocRequestFrameSize - Variable only in PL builds (PL should not use the CONSTANT
 * NumResourceFrames - 128 for all builds
 * MaxVirtualFunctions - from hardware
 */

/* misc stuffs moved from mfg page 3 */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_9
{
    MPI2_CONFIG_PAGE_HEADER     Header;                                            /* 00h */
    U32                         MaxAttempts;    /* Maximum Interations before FAULT */
    U32                         NumResources;   /* Number of resource described in Resource Array */
    U16                         MessageSize;    /* Size of message frame in bytes (1024 max; integral multiple of 16) */
    U16                         Flags;          /* Misc. configuration values */
    U16                         ChainSize;      /* Size of a Local Chain Buffer Element in 16 byte lines (0 interpreted as 1MB) *//* 10h */
    U8                          MaxChainDepth;  /* Maximum number of chains *//* 12h */
    U8                          Reserved;       /* Reserved */                 /* 13h */
    IOC_CONFIG_MFG9_RESOURCE    ResourceArray[IOC_CFG_MFG9_NUMBER_OF_RESOURCES];   /* 14h */
} IOC_CONFIG_PAGE_MFG_PAGE_9, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_9;

#define IOC_CFG_MFG_PG9_FLAG_NUM_ADDL_SGLS_MASK    (0x03)
#define IOC_CFG_MFG_PG9_FLAG_ENABLE_BIDIR_CONTEXT  (0x04)

#define IOC_CFG_MFG_PG9_FLAG_BACKEND_AHCI_DEVICE_SUPPORT    (0x08)
#define IOC_CFG_MFG_PG9_FLAG_BACKEND_NVME_DEVICE_SUPPORT    (0x10)
#define IOC_CFG_MFG_PG9_FLAG_BACKEND_SOP_DEVICE_SUPPORT     (0x20)

typedef struct _IOC_CONFIG_UART_SETTINGS
{
    U8      Flags;
    U8      Reserved_MoreFlags;
    U16     Reserved_TO;
    U32     BaudRate;
} IOC_CONFIG_UART_SETTINGS;

#define IOC_CONFIG_UART_FLAGS_USE_EXT_CLK       (0x80) /* 0x80: Use ext clock, 0x00: Use internal clock (30Mhz) */
#define IOC_CONFIG_UART_FLAGS_USE_XON_XOFF      (0x40) /* 0x40: Use XON/XOFF, 0x00: No Flow Control */
#define IOC_CONFIG_UART_FLAGS_DISABLED          (0x20) /* 0x20: 0: Init and use this UART, 1: Disable UART usage */

typedef struct _IOC_CPLTQ_INT_CONFIG
{
    U32     SasCoreCompletionIntCount           : 10;  /* Number of completions to collect prior to CpltQ Int*/
    U32     Reserved10_15                       :  6;
    U32     SasCoreCompletionIntTimer           : 10;  /* Number of microseconds to hold compltion prior to CpltQ Int */
    U32     Reserved26_31                       :  6;
} IOC_CPLTQ_INT_CONFIG;

/*****************************************************************************
*
*  Manufacturing Page 10 Config Page
*   This is an IR MFG Page which will be used by firmware to
*   determine policies to apply based on OEM requirements
*
*   GenericFlags will be defined in this document as all OEMs will have access
*   to these flags.
*
*   OEMSpecificFlags will not be available in this document as these flags are
*   specific to each OEM, and the information must only be provided to the
*   specific OEM for which the values apply.
*
*****************************************************************************/

/* OEM Identifiers */
#define IOC_MFG10_OEM_ID_INVALID                   (0x00000000)
#define IOC_MFG10_OEM_ID_DELL                      (0x00000001)
#define IOC_MFG10_OEM_ID_FSC                       (0x00000002)
#define IOC_MFG10_OEM_ID_SUN                       (0x00000003)
#define IOC_MFG10_OEM_ID_IBM                       (0x00000004)
#define IOC_MFG10_OEM_ID_LSI                       (0x000000FF)

/* GENERIC Flags 0*/
#define IOC_MFG10_GF0_OCE_DISABLED                  (0x00000001) /* bit 0 */
#define IOC_MFG10_GF0_R1E_DRIVE_COUNT               (0x00000002) /* bit 1 */
#define IOC_MFG10_GF0_R10_DISPLAY                   (0x00000004) /* bit 2*/
#define IOC_MFG10_GF0_SSD_DATA_SCRUB_DISABLE        (0x00000008) /* bit 3 */
#define IOC_MFG10_GF0_SINGLE_DRIVE_R0               (0x00000010) /* bit 4 */
#define IOC_MFG10_GF0_MULTIPATHING                  (0x00000020) /* bit 5 */
#define IOC_MFG10_GF0_FAST_BGI_ENABLED              (0x00000040) /* bit 6 */
#define IOC_MFG10_GF0_DPICZ_REQUIRED_FOR_PI         (0x00000080) /* bit 7 */
#define IOC_MFG10_GFO_ENABLE_PI                     (0x00000100) /* bit 8 */
#define IOC_MFG10_GFO_ENABLE_PI_MIXING              (0x00000200) /* bit 9 */
#define IOC_MFG10_GFO_PI_TYPE_1_SUPPORTED           (0x00000400) /* bit 10*/
#define IOC_MFG10_GFO_PI_TYPE_2_SUPPORTED           (0x00000800) /* bit 11*/
#define IOC_MFG10_GFO_PI_TYPE_3_SUPPORTED           (0x00001000) /* bit 12*/
#define IOC_MFG10_GFO_4K_SUPPORT_DISABLE            (0x00002000)
                                                                 /* 13 - 15 currently unused */
#define IOC_MFG10_GF0_IR_FP_DISABLED                (0x00010000) /* bit 16*/
#define IOC_MFG10_GF0_CONSISTENCY_CHECKING_DISABLED (0x00020000) /* bit 17*/


/* OEM Specific Flags will come from OEM specific header files */

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_10
{
    MPI2_CONFIG_PAGE_HEADER Header;                                 /* 00h */
    U8                      OEMIdentifier;                          /* 04h */
    U8                      Reserved1;                              /* 05h */
    U16                     Reserved2;                              /* 08h */
    U32                     Reserved3;                              /* 0Ch */
    U32                     GenericFlags0;                          /* 10h */
    U32                     GenericFlags1;                          /* 14h */
    U32                     Reserved4;                              /* 18h */
    U32                     OEMSpecificFlags0;                      /* 1Ch */
    U32                     OEMSpecificFlags1;                      /* 20h */
    U32                     Reserved5[18];                          /* 24h - 60h*/
} IOC_CONFIG_PAGE_MFG_PAGE_10, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_10;

/* Miscellanous settings that do not belong elsewhere */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_11
{
    /* The DoNotUse fields are used by gen2. Don't define new stuff in those fields. */
    MPI2_CONFIG_PAGE_HEADER     Header;                                                     /* 0x00 */

    U8                          FlashTime;                                                  /* 0x04 */
    U8                          NVTime;                                                     /* 0x05 */
    U16                         Flag;                                                       /* 0x06 */

    U8                          RFIoTimeout;    /* Reduced Functionality Timeout */         /* 0x08 */
    U8                          EEDPTagMode;    /* bits 0-1 initiator, bits 4-5 target         0x09 */
    U8                          AWTValue;       /* its only 4 bits, but easier to define as 8 for now 0x0A */
    U8                          DoNotUse1;                                                  /* 0x0B */

    U8                          MaxCmdFrames;   /* 0=F/W Default; 31 max */                 /* 0x0C */
    U8                          DoNotUse2;                                                  /* 0x0D */
    U16                         AddlFlags;                                                  /* 0x0E */

    U32                         SysRefClk;      /* System Reference Clock in Hz */          /* 0x10 */

    U64                         DoNotUse3;                                                  /* 0x14 */
    U64                         DoNotUse4;                                                  /* 0x1C */
    U64                         DoNotUse5;                                                  /* 0x24 */

    U16                         AddlFlags2;                                                 /* 0x2C */
    U8                          AddlFlags3;                                                 /* 0x2E */
    U8                          Reserved1;                                                  /* 0x2F */
    /*
     * The following 3 entries are the SOD settings for the ring buffer masks
     */
    U64                         IopDebugEnable;                                             /* 0x30 */
    U64                         PlDebugEnable;                                              /* 0x38 */
    U64                         IrDebugEnable;                                              /* 0x40 */
    U32                         BoardPowerRequirement;  /* Board Power Req (in mWatts) */  /* 0x48 */

} IOC_CONFIG_PAGE_MFG_PAGE_11, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_11;

/* These allow people to enable usage of various slow or fast memories that might be available */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_NVSRAM             (0x0002)    /* 1: Use NVSRAM if found.            0: Don't detect and use NVSRAM */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_DDR                (0x0004)    /* 1: Use DDR if found.               0: Don't detect and use DDR    */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_HCDW               (0x0008)    /* 1: Use HCDW for DATA if enabled.   0: Don't use HCDW for DATA     */
/* NOTE: If running from HCDW the code must be copied to the bottom of the HCDW initially so the choice is XMEM or HCDW bottom      */
#define IOC_CFG_MFG_PG11_FLAG_XMEMCOPY                  (0x0010)    /* 1: Copy code from source to memory 0: Execute from source         */

/* Turn off the check for the broken AAMUX */
#define IOC_CFG_MFG_PG11_FLAG_DISABLE_LSI_AAMUX_CHECK   (0x0020)

/* Enable RAID Accelerator and allocate memory at SOD. */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_RAID_ACCELERATOR   (0x0040)    /* 1: Enable RAID Accelerator     0: RAID Accelerator Disabled */

/* This is a special mode that MPI 2.0 devices may use */
#define IOC_CFG_MFG_PG11_FLAG_ROL                       (0x0080)    /* 1: FMU uses ROL Mode           0: FMU uses normal MPI 2.0 mode */

/* The following are special modes that influence route table programming */
/* Enable Simplified Routing */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_SIMPL_ROUTING      (0x0100)
/* Enable Wide Port Optimization */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_WIDE_PORT_OPT      (0x0200)
/* Default value of iop direct print flag - set to 1 to debug resets with UART */
#define IOC_CFG_MFG_PG11_FLAG_DEFAULT_IOP_DIR_PRINT     (0x0400)
/* Prevent host from issuing adapter reset on a FAULT */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_MASK          (0x1800)
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT_OFF  (0x0000)    /* This is OFF */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT      (0x0800)    /* This halts with a FAULT even if due to adapter reset IRQ */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_IGNORE_RESET  (0x1000)    /* This is a flag for the the next one */
#define IOC_CFG_MFG_PG11_FLAG_DISABLE_POST_MEM_CHECK    (0x2000)    /* Disables thorough memory testing during hostboot. */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_INITIAL_AWT        (0x4000)    /* Enable configurable initial AWT value */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_L3_OCM_IS_FAST     (0x8000)    /* When L3 is used as memory, consider it 1:fast 0:slow */

/* mask for AWT value to get least significant 4 bits */
#define IOC_CFG_MFG_PG11_AWT_VALUE_MASK                 (0x0f)

/* Mask for Expander AWT value to get three most significant bits */
#define IOC_CFG_MFG_PG11_EXP_AWT_VALUE_MASK             (0xE0)
#define IOC_CFG_MFG_PG11_EXP_AWT_SHIFT                  (0x05)
/*
 * The following may be used to block reset after a fault not caused by adapter reset
 * This is necessary due to the unexptected IRQ caused by the 10ms hw delay after adapter reset assertion
 */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT_NOAR (IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT|IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_IGNORE_RESET)

/* EEDP Tag mode settings for intiator and target */
#define IOC_CFG_MFG_PG11_EEDP_MODE_INIT_0       (0x00)
#define IOC_CFG_MFG_PG11_EEDP_MODE_INIT_1       (0x01)
#define IOC_CFG_MFG_PG11_EEDP_MODE_INIT_2       (0x02)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_0       (0x00)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_1       (0x10)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_2       (0x20)
#define IOC_CFG_MFG_PG11_EEDP_MODE_MASK         (0x03)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_SHIFT   (4)

/* Mfg Page 11 AddlFlags */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_EXTEND_2108_CREDIT_OBSOLETE  (0x0001)    /* Bit  0 Allows the 2108 to have credit > 1024 (not recommended) */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_SEND_EXECUTE_DEV_DIAG_CMD    (0x0002)    /* Bit  1 Send ATA Execute Device Diagnostic cmd during SATA init */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_TM_FAIL_MISSING              (0x0004)    /* Bit  2 Reply with TM failure if device is missing */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_USE_DEVICE_SLOT_NUM          (0x0008)    /* Bit  3 Use the Connector Index instead of the Device Slot Number */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_FEATURE1                     (0x0010)    /* Bit  4 Reserved for a feature */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_FEATURE2                     (0x0020)    /* Bit  5 Reserved for a feature */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_IGNORE_SAS_BCAST_CHANGE      (0x0040)    /* Bit  6 Ignore SAS BCAST(CHANGE) primitive in Target Mode */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_CUST_TGTPORT_HARD_RESET      (0x0080)    /* Bit  7 Custom handling of Target Port hard reset */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_ENABLE_NOTIFY_EVENT          (0x0100)    /* Bit  8 Allows sending Notify(Enable Spinup) event in target mode */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_EXP_AWT                      (0x0400)    /* Bit 10 Use Initial AWT for expanders */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_DISABLE_RXEEDP_PI_CHECK      (0x0800)    /* Bit 11 Don't check for PI errors in RxEEDP block */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_SAT_WRITE_SAME_FOREGROUND    (0x1000)    /* Bit 12 Execute translated SCT Write Same command in the foreground */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_PCIE_RELAXED_ORDERING        (0x2000)    /* Bit 13 */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_CLOSE_OPTIMIZE               (0x4000)    /* Bit 14 CloseOptimize BT[SASCore3]969 */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_RESERVED_15                  (0x8000)    /* Bit 15 Formerly BT1680 Detect Only on SAS  */

/* Mfg Page 11 AddlFlags 2*/
#define IOC_CFG_MFG_PG11_ADDLFLAGS2_ACM_POWER_MANAGEMENT_ENABLE (0x0001)    /* Bit  0 Active Cable Power Management Enable bit to enable or disable the Active Cable Power Management */
#define IOC_CFG_MFG_PG11_ADDLFLAGS2_SHORT_LINK_UP               (0x0002)    /* Bit  1 0.25s Discovery delay on LinkUp */
#define IOC_CFG_MFG_PG11_ADDLFLAGS2_RESERVED02TO15              (0xFFFC)    /* Bit  2 to 15 Reserved */

/* Mfg Page 11 AddlFlags 3 */
#define IOC_CFG_MFG_PG11_ADDLFLAGS3_RESERVED0TO7                (0xFF)      /* Bit  0 to  7 Reserved*/

#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT0                         (0x0)    /* 0x0050 AWT */
#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT1                         (0x1)    /* 0x7FFF AWT */
#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT2                         (0x2)    /* 0xFFFE AWT */
#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT3                         (0x3)    /* 0xRsvd AWT */
#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT4                         (0x4)    /* 0xRsvd AWT */
#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT5                         (0x5)    /* 0xRsvd AWT */
#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT6                         (0x6)    /* 0xRsvd AWT */
#define IOC_CFG_MFG_PG11_INIT_VAL_EXP_AWT7                         (0x7)    /* 0xRsvd AWT */

/*****************************************************************************
*
*  Manufacturing Page 12 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_MAN_PAGE_12_SGPIO_INFO_ENTRIES
#define IOC_MAN_PAGE_12_SGPIO_INFO_ENTRIES         (1)
#endif

/* Defines for the Flags field */
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DRIVE_PRESENCE_CHECK         (0x00000800) /* Check Drive presence to set Fault LED */
#define IOC_CFG_MFG_12_FLAGS_SGPIO_SFF8485_ACT_INVERT           (0x00000400)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DISABLE_GROUP_ID             (0x00000200)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DISABLE_PORT_ACT_LED         (0x00000100)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_USE_SLOT_INFO                (0x00000080) /* Obsolete in SAS 3.5 */
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DATA_IN_LATCH_CTRL           (0x00000040)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_GP_MODE_CTRL                 (0x00000020)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_SLOAD_MODE_CTRL              (0x00000010)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_CLK_FILTER_ENABLE            (0x00000008)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_CTRL_OD_ENABLE               (0x00000004)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_CTRL_DOUT_OD_ENABLE          (0x00000002)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_MODE_VU                      (0x00000001)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_MODE_SFF8485                 (0x00000000)

typedef struct _SGPIO_CONFIG_INFO
{
    U8                                  DeviceCount;                                       /* 00h */
    U8                                  Reserved1[3];                                      /* 01h */
    U32                                 Reserevd2;                                         /* 04h */
    U32                                 LEDDataOrder[8];                                   /* 08h */
} SGPIO_CONFIG_INFO, *PTR_SGPIO_CONFIG_INFO;

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_12
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U32                                 Flags;                                             /* 04h */
    U32                                 Reserved1;                                         /* 08h */
    U32                                 Reserved2;                                         /* 0Ch */
    U8                                  NumSGPIO;                                          /* 10h */
    U8                                  SGPIOType;                                         /* 11h */
    U16                                 Reserved3;                                         /* 12h */
    U32                                 SClockFrequency;                                   /* 14h */
    U32                                 SGPIOCfg1;                                         /* 18h */
    U32                                 WidePortActivityCfg;                               /* 1Ch */
    U32                                 DefaultLEDStatus;                                  /* 20h */
    U32                                 CustomBlinkPattern[8];                             /* 24h */
    U32                                 Reserved4;                                         /* 44h */
    U32                                 Reserved5;                                         /* 48h */
    SGPIO_CONFIG_INFO                   SGPIOInfo[IOC_MAN_PAGE_12_SGPIO_INFO_ENTRIES];     /* 4Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_12, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_12;

#define IOC_CONFIG_PAGE_MFG12_SGPIO_DISABLE (0)

/* SGPIO Type Codes */
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_NONE                 (0)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_HP                   (1)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_RSA_GENERIC          (4)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_RSA_FSC              (5)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_GIGABYTE             (6)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_HITACHI              (7)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_SUN                  (8)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_IBM                  (9)

/*****************************************************************************
*
*  Manufacturing Page 13 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_MAN_PAGE_13_SGPIO_ENTRIES
#define IOC_MAN_PAGE_13_SGPIO_ENTRIES         (1)
#endif

typedef struct _SGPIO_TRANSLATION_DATA
{
    U32                                 Mask;                                              /* 00h */
    U32                                 SlotStatus;                                        /* 04h */
    U8                                  LEDControl[4];                                     /* 08h */
} SGPIO_TRANSLATION_DATA, *PTR_SGPIO_TRANSLATION_DATA;

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_13
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U8                                  NumSgpioEntries;                                   /* 04h */
    U8                                  Reserved0;                                         /* 05h */
    U16                                 Reserved1;                                         /* 06h */
    U32                                 Reserved2;                                         /* 08h */
    SGPIO_TRANSLATION_DATA              SGPIOData[IOC_MAN_PAGE_13_SGPIO_ENTRIES];          /* 0Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_13, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_13;


/* Definition of SGPIO LED Types in LedControl (Renamed) */
#define IOC_CFG_MFG_13_SGPIO_LEDCONTROL_TYPE_MASK            (0x30)
#define IOC_CFG_MFG_13_SGPIO_LEDCONTROL_TYPE_NONE            (0x00)
#define IOC_CFG_MFG_13_SGPIO_LEDCONTROL_TYPE_ACTIVITY        (0x10)
#define IOC_CFG_MFG_13_SGPIO_LEDCONTROL_TYPE_LOCATE          (0x20)
#define IOC_CFG_MFG_13_SGPIO_LEDCONTROL_TYPE_ERROR           (0x30)
#define IOC_CFG_MFG_13_SGPIO_LEDCONTROL_MASK                 (0x0F)

/*
 * Definiton of SGPIO Tranmit control bit encodings for Activity, Locate
 * and Error LED's in SFF-8485 mode.
 */

/* Activity */
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_SHIFT                     (5)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_MASK                      (0x07)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_DISABLE                   (0x00)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_ENABLE                    (0x01)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_A_1ST_HALF          (0x02)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_A_2ND_HALF          (0x03)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_DEASSERT_ACTIVITY         (0x04)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_ASSERTED_ACTIVITY         (0x05)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_B_1ST_HALF          (0x06)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_B_2ND_HALF          (0x07)

/* Locate */
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_SHIFT                     (3)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_MASK                      (0x03)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_DISABLE                   (0x00)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_ENABLE                    (0x01)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_BLINK_A_1ST_HALF          (0x02)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_BLINK_A_2ND_HALF          (0x03)

/* Error */
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_SHIFT                     (0)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_MASK                      (0x07)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_DISABLE                   (0x00)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_ENABLE                    (0x01)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_A_1ST_HALF          (0x02)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_A_2ND_HALF          (0x03)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_B_1ST_HALF          (0x06)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_B_2ND_HALF          (0x07)

/* Vendor Unique Mode */
/*
 * Definiton of SGPIO Transmit control bit encoding for Activity, Error
 * and Locate LEDs in Vendor Unique mode.
 */


/*****************************************************************************
*
*  Manufacturing Page 14 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_MAN_PAGE_14_SEP_ENTRIES
#define IOC_MAN_PAGE_14_SEP_ENTRIES         (1)
#endif

/* Max Direct Attached SEP supported */
#define MAX_NUM_DA_SEP                      (6)

typedef struct _SEP_TRANSLATION_DATA
{
    U32                                 Mask;                                              /* 00h */
    U32                                 SlotStatus;                                        /* 04h */
    U32                                 SESValue;                                          /* 08h */
} SEP_TRANSLATION_DATA, *PTR_SEP_TRANSLATION_DATA;

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_14
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U8                                  NumSepEntries;                                     /* 04h */
    U8                                  SEPType;                                           /* 05h */
    U16                                 Reserved1;                                         /* 06h */
    U32                                 Reserved2;                                         /* 08h */
    SEP_TRANSLATION_DATA                SEPData[IOC_MAN_PAGE_14_SEP_ENTRIES];              /* 0Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_14, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_14;

/* SEP Type defines  */
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_NONE                 (0)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_HP                   (1)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_RSA_GENERIC          (2)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_DELL                 (3)

#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA                 (0x80)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA_VITESSE         (IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA | 0x10)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA_IBM             (IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA | 0x18)

#ifndef IOC_CONFIG_PAGE_VENDOR_WORDS
#define IOC_CONFIG_PAGE_VENDOR_WORDS  (1)
#endif


typedef struct _IOPI_SPD_DDR2
{
    U8 SpdDataBytes;              //  0 - 0x00 number of bytes in this SPD structure (0-255)
    U8 SizeSerialMemory;          //  1 - 0x01 encoded value = #serial memory data bytes Size==2^N bytes
    U8 FundamentalMemoryType;     //  2 - 0x02 memory type IOPI_SPD_DDR2_FUND_MEMORY_TYPE_* constant
    U8 NumberOfRowAddresses;      //  3 - 0x03 Number of RAS lines (bank 1=hi nibble, bank 2 = low nibble )
    U8 NumberOfColumnAddresses;   //  4 - 0x04 Number of CAS lines (bank 1=hi nibble, bank 2 = low nibble )
    U8 NumberOfDimmRanks;         //  5 - 0x05 Number of DIMM Ranks see IOPI_SPD_DDR_DIMM_RANK_*
    U8 ModuleWidthInBitsLSB;      //  6 - 0x06 Module Data Width of DIMM in number of bits
    U8 ModuleWidthInBitsMSB;      //  7 - 0x07 Module Data Width continued (if > 16 bytes (256bits), else 0)
    U8 VoltageInterfaceStd;       //  8 - 0x08 IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_* constant
    U8 SdramCycleTime;            //  9 - 0x09 Tclk 1/10 ns BCD (LSN.MSN ns) (N=nibble)
    U8 SdramAccessFromClock;      // 10 - 0x0A Tac 1/100ns BCD  (0.LSN MSN ns) (N=nibble)
    U8 DimmConfigurationType;     // 11 - 0x0B IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_* constant
    U8 RefreshRateAndType;        // 12 - 0x0C IOPI_SPD_DDR2_DIMM_REFRESH_RATE_* constant
    U8 PrimarySdramWidth;         // 13 - 0x0D Width of individual memory types (x4, x8, etc..) Module width 0x72 SDRAM Width 0x9 bytes (MSB bank2 2x)
    U8 ErrorCheckingSdramWidth;   // 14 - 0x0E Number of bytes error is checked in */
    U8 Reserved15;                // 15 - 0x0F Reserved in JEDEC
    U8 BurstLengthsSupported;     // 16 - 0x10 Burst Lengths supported IOPI_SPD_DDR2_BURST_LENGTH_SUPPORTED_*
    U8 NumberOfBanksEachChip;     // 17 - 0x11 (1-255)
    U8 CasLatencySupported;       // 18 - 0x12 each bit represents SDRAM (1 + N * 0.5) supported
    U8 DimmMechanicalChar;        // 19 - 0x13 Dimm Mechanical Characteristics
    U8 DimmType;                  // 20 - 0x14 Unbuffer Differential:0x20 - Registered DDR Dimmm:0x26 IOPI_SPD_DDR2_DIMM_TYPE_*
    U8 ModuleAttributes;          // 21 - 0x15 SDRAM Module Attributes
    U8 DeviceAttributes;          // 22 - 0x16 See IOPI_SPD_DDR2_DEV_ATTR_*
    U8 CycleTime2ndHighest;       // 23 - 0x17 cycle time at reduced CAS latency (X-1)  (LSN.MSN ns) (N=nibble)
    U8 AccessFromClock2ndHighest; // 24 - 0x18 access time at reduced CAS latency (X-1) (0.LSN MSN ns) (N=nibble)
    U8 CycleTime3rdHighest;       // 25 - 0x19 cycle time at reduced CAS latency (X-2)  (LSN.MSN ns) (N=nibble)
    U8 AccessFromClock3rdHighest; // 26 - 0x1A access time at reduced CAS latency (X-2) (0.LSN MSN ns) (N=nibble)
    U8 tRP;                       // 27 - 0x1B Min Row Precharge Time N/4 ns
    U8 tRRD;                      // 28 - 0x1C Min Row Active to Row Active Delay N/4 ns
    U8 tRCD;                      // 29 - 0x1D Min RAS to CAS delay N/4 ns
    U8 tRAS;                      // 30 - 0x1E Min Active to Precharge Time (ns)
    U8 ModuleRankDensity;         // 31 - 0x1F
    U8 tIS;                       // 32 - 0x20 Address and Command Input Setup Time (0.LSN MSN ns)
    U8 tIH;                       // 33 - 0x21 Address and command Hold Time After Clock (0.LSN MSN ns)
    U8 tDS;                       // 34 - 0x22 Input Setup Time before Data Strobe Before Clock (0.LSN MSN ns)
    U8 tDH;                       // 35 - 0x23 Address and Command HOld Time AFter Clock (0.LSN MSN ns)
    U8 tWR;                       // 36 - 0x24 Write Recovery Time
    U8 tWTR;                      // 37 - 0x25 Internal Write To Read Command Delay (N/4 ns)
    U8 tRTP;                      // 38 - 0x26 Internal Read to Precharge command delay (N/4 ns)
    U8 MemAnalProbeChar;          // 39 - 0x27 Memory analysis Probe Characteristics
    U8 ExtensiontRCtRFC;          // 40 - 0x28 Extension of Byte 41 tRC and Byte 42 tRFC
                                  //           tRFC=bit0*256ns+(bit1)?(Bits[2:3]+1)/2*0.25ns:Bits[2:3]*0.33ns
                                  //           tRC=(bit4)?(Bits[5:6]+1)/2*0.25ns:Bits[5:6]*0.33ns
    U8 tRC;                       // 41 - 0x29 Min Active to Active Auto Refresh Time (ns)
    U8 tRFC;                      // 42 - 0x2A Min Refresh to Active Refresh Command Period (ns)
    U8 tCKmax;                    // 43 - 0x2B Min Device Cycle Time (N/4 ns)
    U8 tDQSQ;                     // 44 - 0x2C Min Skew Between DQS and DQ (N/100 ns)
    U8 tQHS;                      // 45 - 0x2D Max Read Data Hold Skew Factor (0.LSN MSN ns)
    U8 tPllRelock;                // 46 - 0x2E PLL Relock Time
    U8 ThermalCharacteristics[15];// 47:61 0x2F  - Misc Thermal Characteristics
    U8 Revision;                  // 62 - 0x3E SPD Revision Code BCD (MSN . LSN)
    U8 Checksum;                  // 63 - 0x3F Simple Sum Checksum for 0 to 62 (MSB of sum(0:62))
    /* Note this must be a multiple of 4 bytes in length! */
} IOPI_SPD_DDR2, * PTR_IOPI_SPD_DDR2;

#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_MASK               (0x07)  /* Number of RAnks is N+1 */
#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_CARD_ON_CARD       (0x08)
#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_DRAM_STACK         (0x10)  /* 0x10 stack, else planar */
#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_MOD_HEIGHT_MASK    (0xE0)

#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_RESERVED             0
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_FPM_DRAM             1
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_EDO                  2
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_PIPEL_NIB            3
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_SDRAM                4
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_ROM                  5
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_SGRAM_DDR            6
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_SDRAM_DDR            7
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_DDR2_SDRAM           8

#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_5VTTL           0
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_LVTTL           1
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_HSTL_1P5V       2
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_SSTL_3P3V       3
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_SSTL_2P5V       4
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_SSTL_1P8V       5

#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_NONE          0
#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_PARITY        1
#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_ECC           2
#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_ECC_ADDR_PARITY_DIMM (0x06)    // Address command Parity with ECC

#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_SELF_FLAG           0x80
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MASK                0x7F
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_NORMAL              0x00    /*  15.625 us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_REDUCE_QUARTER      0x01    /*   3.9   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_REDUCE_HALF         0x02    /*   0.7   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MULT_TWO            0x03    /*  31.3   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MULT_FOUR           0x04    /*  62.5   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MULT_EIGHT          0x05    /* 125     us */

#define IOPI_SPD_DDR2_BURST_LENGTH_SUPPORTED_4              0x04
#define IOPI_SPD_DDR2_BURST_LENGTH_SUPPORTED_8              0x08

#define IOPI_SPD_DDR2_DIMM_TYPE_UNDEFINED                   0x00
#define IOPI_SPD_DDR2_DIMM_TYPE_RDIMM                       0x01
#define IOPI_SPD_DDR2_DIMM_TYPE_UDIMM                       0x02
#define IOPI_SPD_DDR2_DIMM_TYPE_SODIMM                      0x04
#define IOPI_SPD_DDR2_DIMM_TYPE_MICRO_DIMM                  0x08
#define IOPI_SPD_DDR2_DIMM_TYPE_MINI_RDIMM                  0x10
#define IOPI_SPD_DDR2_DIMM_TYPE_MINI_UDIMM                  0x20

#define IOPI_SPD_DDR2_DIMM_ATTR_NUM_ACTIVE_REGS_MASK        0x03
#define IOPI_SPD_DDR2_DIMM_ATTR_NUM_PLLS_MASK               0x0E
#define IOPI_SPD_DDR2_DIMM_ATTR_FET_SW_EXT_EN               0x10
#define IOPI_SPD_DDR2_DIMM_ATTR_TBD5                        0x20
#define IOPI_SPD_DDR2_DIMM_ATTR_ANALSYS_PROBE_INST          0x40
#define IOPI_SPD_DDR2_DIMM_ATTR_TBD7                        0x80

#define IOPI_SPD_DDR2_DEV_ATTR_WEAK_DRIV_SUP                0x01
#define IOPI_SPD_DDR2_DEV_ATTR_50_OHM_OUT_SUP               0x02
#define IOPI_SPD_DDR2_DEV_ATTR_PASR_SUP                     0x04
#define IOPI_SPD_DDR2_DEV_ATTR_RESERVED                     0xF1

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_16
{
    MPI2_CONFIG_PAGE_HEADER             Header;                 /* 0x00 */
    U32                                 Flags;                  /* 0x04 */
    U32                                 MaxFirmwareDDRSize;     /* 0x08 - Maximum Firmware Owned DDR in bytes */
#if VENTURA
    U8                                  DDR_SPD[512];
#else
    U8                                  DDR_SPD[256];          /* To IOP, this is just an array of bytes.  The ddr3 or mss library will cast it to
                                                                  it's own SPD structure for use */
#endif
} IOC_CONFIG_PAGE_MFG_PAGE_16, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_16;

#define IOC_CONFIG_PAGE_MFG16_FLAGS_MFG_SPD     (0x00000001)    /* Use IOPI_SPD_DDR2 structure for SPD data */
#define IOC_CONFIG_PAGE_MFG16_FLAGS_IIC_SPD     (0x00000002)    /* Retrieve SPD information from IIC based on ISTWI type (Ext Mfg Page 43) */
#define IOC_CONFIG_PAGE_MFG16_FLAGS_TRAIN_MASK  (0x0000000F)    /* Mask for training modes */
#define IOC_CONFIG_PAGE_MFG16_FLAGS_RSV_MASK    (0xFFFFFFF0)

/*****************************************************************************
*
*  Manufacturing Page 17 Config Page
*
*****************************************************************************/

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_17
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                 /* 0x00 */
    U8                                  RetryMode;                              /* 0x04 */
    U8                                  SMPTimeout;
    U8                                  SMPRetries;
    U8                                  Resereved1;
    U32                                 EnALFMask;                              /* 0x08 */
    U32                                 Flags;                                  /* 0x0C */
    U32                                 Reserved3;                              /* 0x10 */
    U8                                  OpenRejectRetryInterval[3];             /* 0x14 */
#if VENTURA
    U8                                  ForceCompletionTimeout;                 /* 0x17 */
#else
    U8                                  Reserved4;
#endif
    U8                                  OpenRejectRetryLmit[3];                 /* 0x18 */
    U8                                  Reserved5;
    U8                                  OpenRejectSRBInterval;                  /* 0x1C*/
    U8                                  OpenRejectSRBLimit[3];
    U8                                  OpenRejectNoDestInterval[3];            /* 0x20 */
    U8                                  Reserved6;
    U8                                  OpenRejectNoDestLimit[3];               /* 0x24 */
    U8                                  Reserved7;
    U32                                 PortLinkAvailMask;                      /* 0x28 */
    U16                                 PortLinkAvailCount;                     /* 0x2C */
    U16                                 Reserved9;

}IOC_CONFIG_PAGE_MFG_PAGE_17, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_17;

#define IOC_CONFIG_PAGE_MFG17_MAX_RETRIES_SMP_ERROR_MASK                    (0x0F)
#define IOC_CONFIG_PAGE_MFG17_MAX_RETRIES_SMP_TIMEOUT_MASK                  (0xF0)
#define IOC_CONFIG_PAGE_MFG17_MAX_RETRIES_SMP_TIMEOUT_OFFSET                (0x04)
#define IOC_CONFIG_PAGE_MFG17_RETRY_MODE_ORR_TIMED                          (0x00000001)
#define IOC_CONFIG_PAGE_MFG17_RETRY_MODE_ORND_TIMED                         (0x00000002)
#define IOC_CONFIG_PAGE_MFG17_RETRY_MODE_ORSRB_TIMED                        (0x00000004)

#define IOC_CONFIG_PAGE_MFG17_FLAGS_ORCRNS_NO_RESET                         (0x00000001)
#define IOC_CONFIG_PAGE_MFG17_FLAGS_ORR_NO_RESET                            (0x00000002)
#define IOC_CONFIG_PAGE_MFG17_FLAGS_ORND_TRACKING                           (0x00000004)
#define IOC_CONFIG_PAGE_MFG17_FLAGS_FCT_RESOLUTION_MASK                     (0x00000600)        // Ventura
#define IOC_CONFIG_PAGE_MFG17_FLAGS_FCT_RESOLUTION_SHIFT                    (9)                 // Ventura
/* Reserved for possible future tracking use */
#define IOC_CONFIG_PAGE_MFG17_FLAGS_OR_RSVD                                 (0x00000038)
#define IOC_CONFIG_PAGE_MFG17_FLAGS_SMP_RETRIES                             (0x00000040)
#define IOC_CONFIG_PAGE_MFG17_FLAGS_PHY_VACANT                              (0x00000080)
#define IOC_CONFIG_PAGE_MFG17_FLAGS_RESET_AWT                               (0x00000100)

/*****************************************************************************
*
*  Manufacturing Page 19 Config Page
*
*****************************************************************************/
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_19
{
    MPI2_CONFIG_PAGE_HEADER             Header;             /* 0x00 */
    U32                                 Reserved04;         /* 0x04 */
    U32                                 Flags;              /* 0x08 */
    U16                                 MaxPayloadSize;     /* 0x0C */
    U8                                  NumMessages;        /* 0x0E */
    U8                                  MaxSenseLength;     /* 0x0F */
    U8                                  EndpointID;         /* 0x10 */
    U8                                  Reserved11;         /* 0x11 */
    U16                                 MaxPacketSize;      /* 0x12 */
    U32                                 Reserved14;         /* 0x14 */
    U32                                 Reserved18;         /* 0x18 */
    U32                                 Reserved1C;         /* 0x1C */
    U32                                 Reserved20;         /* 0x20 */
    U32                                 Reserved24;         /* 0x24 */
    U32                                 Reserved28;         /* 0x28 */
    U32                                 Reserved2C;         /* 0x2C */
}IOC_CONFIG_PAGE_MFG_PAGE_19, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_19;

#define IOC_MFG19_FLAGS_MCTP_BIND_MASK              (0x00000003)
#define IOC_MFG19_FLAGS_MCTP_BIND_SMBUS             (0x00000000)
#define IOC_MFG19_FLAGS_MCTP_BIND_VDM               (0x00000001)
#define IOC_MFG19_FLAGS_INTEGRITY_CHECK_MASK        (0x0000000C)
#define IOC_MFG19_FLAGS_INTEGRITY_CHECK_NEVER       (0x00000000)
#define IOC_MFG19_FLAGS_INTEGRITY_CHECK_ALWAYS      (0x00000004)
#define IOC_MFG19_FLAGS_INTEGRITY_CHECK_OPTIONAL    (0x00000008)
#define IOC_MFG19_FLAGS_RESPOND_SLAVE               (0x00000010)
#define IOC_MFG19_FLAGS_PADLEN_WORKAROUND           (0x00000020)
#define IOC_MFG19_FLAGS_LSI_HDR_IS_LITTLE_ENDIAN    (0x00000100)
#define IOC_MFG19_FLAGS_DEBUG_ENABLE                (0x00000200)
#define IOC_MFG19_FLAGS_ABORT_TM                    (0x00010000)
#define IOC_MFG19_FLAGS_ENABLE_MCTP                 (0x80000000)

#define IOC_MFG19_NUMMSG_REPLYQ_MASK                (0x0F)
#define IOC_MFG19_NUMMSG_REPLYQ_SHIFT               (0)



/* Miscellanous support for Fpe */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_30
{
    MPI2_CONFIG_PAGE_HEADER Header;                                                     /* 0x00 */

    U32                     Reserved1;                                                  /* 0x04 */

    U16                     EDFBFlags;      /* EndDeviceFrameBuffering flags   */       /* 0x08 */
    U8                      MaxFCS;         /* Max # of Fast Context Switches  */       /* 0x0A */
    U8                      DequeuingModes;                                             /* 0x0B */

    U16                     EDFBMaxGroupUnload; /* Max # entries per dma group */       /* 0x0C */
    U8                      EDFBThresholdSAS;   /* Max # SAS data frame xfers  */       /* 0x0E */
    U8                      EDFBThresholdSATA;  /* Max # SATA data frame xfers */       /* 0x0F  */

    U32                     Reserved3;                                                  /* 0x10 */

    U16                     FastPathFlags;      /* Settings for FPE */                  /* 0x14 */
    U8                      RaidContextOffset;  /* Offset in dwords in SCSIio request*/ /* 0x16 */
    U8                      EncRAIDContextOffset; /* RAID context offset for Encapsulated IOs */ /* 0x17 */

    U16                     MaxExceptionsHandled; /* # per ExceptionQueue process */    /* 0x18 */
    U16                     MaxAsyncEventsHandled;/* # per AsycQueue process  */        /* 0x1A */

    U16                     MaxIOPCompletionsHandled; /* # per IopCompQueue process */  /* 0x1C */
    U8                      MaxTranslationExceptionsHandled;                            /* 0x1E */
    U8                      MaxFxCoreTranslationExceptionsHandled;                                                  /* 0x1F */

    U32                     Reserved5[2];                                               /* 0x20 */

} IOC_CONFIG_PAGE_MFG_PAGE_30, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_30;

#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_DUALCTX_AUTO_12G               (0x0080) /* Only used with DUALCTX_AUTO below */
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_DUALCTX_MASK                   (0x0060)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_DUALCTX_FORCE_OFF              (0x0040)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_DUALCTX_FORCE_ON               (0x0020)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_DUALCTX_AUTO                   (0x0000)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_DMAGROUP_SCHEME                (0x001C) /* Bits [4:2] used for configuring DMAGroup scheme */
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_EXPANDER_DMA_GRPS_DISABLED     (0x0004)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_FCS_DISABLED                   (0x0002)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_EDFB_DISABLED                  (0x0001)

#define IOC_CONFIG_PAGE_MFG30_FASTPATH_FLAGS_INTERLEAVE_THRESHOLD_MASK  (0xF000)
#define IOC_CONFIG_PAGE_MFG30_FASTPATH_FLAGS_INTERLEAVE_THRESHOLD_SHIFT (12)
#define IOC_CONFIG_PAGE_MFG30_DEQUEUE_MODE_MASK                         (0x03)
#define IOC_CONFIG_PAGE_MFG30_DEQUEUE_MODE_DISABLE_SPREAD_FLAG          (0x01)
#define IOC_CONFIG_PAGE_MFG30_DEQUEUE_MODE_ALLOW_COHAB_FLAG             (0x02)

#define IOC_CONFIG_PAGE_MFG30_FASTPATH_FLAGS_PL_REACTIVE_FPE            (0x0004)
#define IOC_CONFIG_PAGE_MFG30_FASTPATH_FLAGS_FPE_TIMER_ENABLED          (0x0002)
#define IOC_CONFIG_PAGE_MFG30_FASTPATH_FLAGS_FP_MESSAGING_MODE          (0x0001)

#define IOC_CONFIG_PAGE_MFG30_DMAGROUP_SCHEME_EXPANDERBASED             (0)
#define IOC_CONFIG_PAGE_MFG30_DMAGROUP_SCHEME_DEVICEBASED               (1)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_DMAGROUP_SCHEME_OFFSET         (2)
#define IOC_CONFIG_PAGE_MFG30_DMAGROUP_SCHEME_DEFAULT                   (IOC_CONFIG_PAGE_MFG30_DMAGROUP_SCHEME_EXPANDERBASED)

#ifndef IOC_MAN_PAGE_31_NUM_UARTS
#define IOC_MAN_PAGE_31_NUM_UARTS 2
#endif

/* UART settings */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_31
{
    MPI2_CONFIG_PAGE_HEADER Header;                                                     /* 0x00 */

    U32                     Reserved1[4];                                               /* 0x04 */
    U8                      NumUarts;       /* # UartSettings structures  */            /* 0x14 */
    U8                      Reserved2[3];                                               /* 0x15 */
    U32                     ExtUartClk;     /* External Uart Clock in Hz    */          /* 0x18 */

    /* Uart BR & Flags  */
    IOC_CONFIG_UART_SETTINGS            UartSettings[IOC_MAN_PAGE_31_NUM_UARTS];        /* 0x1C */

} IOC_CONFIG_PAGE_MFG_PAGE_31, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_31;



/*****************************************************************************
*
*  Extended Manufacturing Page 40/41 Config Page
*
*****************************************************************************/

/* Phy settings */
typedef struct _IOC_CONFIG_PAGE_EXT_MFG_PAGE_40
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                                         /* 00h */
    U8                                  NumPhys;                                        /* 08h */
    U8                                  FeatureFlags;                                   /* 09h */
    U16                                 Reserved;                                       /* 0Ah */
    U64                                 Phy[IOC_NUM_PHYS];                              /* 0Ch */
} IOC_CONFIG_PAGE_EXT_MFG_PAGE_40, *PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_40;

#define IOC_CONFIG_PAGE_EXT_MFG40_FEATURE_FLAGS_EN_12G_DFE_TUNING   (0x00000001)    /* Bit 0, Enable 12G DFE tuning feature. */

/*******************/
/* Phy Lower Dword */
/*******************/

/* Definition of phy group in Phy*/
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_MASK                    (0x000000FF)    /* Bit 0~7, Specify phy group to use this phy*/
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_0                       (0x00000000)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_1                       (0x00000001)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_2                       (0x00000002)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_3                       (0x00000003)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_4                       (0x00000004)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_5                       (0x00000005)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_6                       (0x00000006)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_7                       (0x00000007)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_8                       (0x00000008)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_9                       (0x00000009)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_10                      (0x0000000A)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_11                      (0x0000000B)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_12                      (0x0000000C)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_13                      (0x0000000D)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_14                      (0x0000000E)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_15                      (0x0000000F)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_16                      (0x00000010)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_17                      (0x00000011)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_18                      (0x00000012)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_19                      (0x00000013)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_20                      (0x00000014)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_21                      (0x00000015)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_22                      (0x00000016)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_GROUP_23                      (0x00000017)


/* definition of SAS SNW participation in Phy */
//#define IOC_PHY_SAS_SNW1_PARTICIPATION          (0x00000100)    /* Bit  8, For emulation only. Do not use during normal operation. */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_SAS_SNW2_PARTICIPATION        (0x00000200)    /* Bit  9       */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_SAS_SNW3_PARTICIPATION        (0x00000400)    /* Bit 10       */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_SAS_SNW_PROGRAMMING_SHIFT     (8)

/* Definition of InvRxData, InvTxData in Phy */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_INV_RX_DATA                   (0x00000800)    /* Bit 11       */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_INV_TX_DATA                   (0x00001000)    /* Bit 12       */

#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_SAS_ONLY                      (0x00002000)    /* Bit 13, Do not link up to SATA */

/* 12.0 Gps */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_G4_120_WITH_SSC               (0x00010000)    /* Bit 16       */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_G4_120_WITHOUT_SSC            (0x00020000)    /* Bit 17       */

/* 6.0 Gps */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_G3_60_WITH_SSC                (0x00040000)    /* Bit 18       */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_G3_60_WITHOUT_SSC             (0x00080000)    /* Bit 19       */

/* 3.0 Gps */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_G2_30_WITH_SSC                (0x00100000)    /* Bit 20       */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_G2_30_WITHOUT_SSC             (0x00200000)    /* Bit 21       */

/* 1.5 Gps, Emulation only */
//#define IOC_CONFIG_PAGE_MFG40_PHY_G1_15_WITH_SSC                  (0x00400000)    /* Bit 22       */
//#define IOC_CONFIG_PAGE_MFG40_PHY_G1_15_WITHOUT_SSC               (0x00800000)    /* Bit 23       */

#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_SATA_SSC_EN                   (0x01000000)    /* Bit 24, Spread Spectrum Clocking */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_BACK_CHANNEL_ENABLE           (0x02000000)    /* Bit 25       */

#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_DATA_SETTLE_TIME_MASK         (0x7C000000)    /* Bit 26~30    */

/*******************/
/* Phy Upper Dword */
/*******************/

#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRESET_SELECT_MASK            (0x00000003)    /* Bit 32~33    */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRESET_SELECT_REF1            (0x00000001)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRESET_SELECT_REF2            (0x00000002)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRESET_SELECT_NOEQ            (0x00000003)

#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRELOAD_SELECT_MASK           (0x0000000C)    /* Bit 34~35    */
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRELOAD_SELECT_SHIFT          (2)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRELOAD_SELECT_REF1           (0x00000001)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRELOAD_SELECT_REF2           (0x00000002)
#define IOC_CONFIG_PAGE_EXT_MFG40_PHY_PRELOAD_SELECT_NOEQ           (0x00000004)


typedef struct _IOC_CONFIG_PAGE_EXT_MFG41_PHY_GROUP
{
    U32                                 Misc[4];                /* 0x00 */
    U32                                 Sas1_OOB_Rx_Reserved;   /* 0x10 */
    U32                                 Sas1_OOB_Tx;            /* 0x14 */
    U32                                 Sas1_3G_Rx;             /* 0x18 */
    U32                                 Sas1_3G_Tx;             /* 0x1C */
    U32                                 Sas2_3G_Rx;             /* 0x20 */
    U32                                 Sas2_3G_Tx;             /* 0x24 */
    U32                                 Sas2_6G_Rx;             /* 0x28 */
    U32                                 Sas2_6G_Tx;             /* 0x2C */
    U32                                 Sas2_12G_Rx;            /* 0x30 */
    U32                                 Sas2_12G_Tx;            /* 0x34 */
    U32                                 Sata_OOB_Rx_Reserved;   /* 0x38 */
    U32                                 Sata_OOB_Tx;            /* 0x3C */
    U32                                 Sata_3G_Rx;             /* 0x40 */
    U32                                 Sata_3G_Tx;             /* 0x44 */
    U32                                 Sata_6G_Rx;             /* 0x48 */
    U32                                 Sata_6G_Tx;             /* 0x4C */
    U32                                 Sata_12G_Rx_Reserved;   /* 0x50 */
    U32                                 Sata_12G_Tx_Reserved;   /* 0x54 */
    U32                                 Optical_3G_Rx;          /* 0x58 */
    U32                                 Optical_3G_Tx;          /* 0x5C */
    U32                                 Optical_6G_Rx;          /* 0x60 */
    U32                                 Optical_6G_Tx;          /* 0x64 */
    U32                                 Optical_12G_Rx;         /* 0x68 */
    U32                                 Optical_12G_Tx;         /* 0x6C */
} IOC_CONFIG_PAGE_EXT_MFG41_PHY_GROUP, *PTR_IOC_CONFIG_PAGE_EXT_MFG41_PHY_GROUP;


/* Phy Groups */
/* The number of phy groups is the same as the number of phys. This could change in the future. */
typedef struct _IOC_CONFIG_PAGE_EXT_MFG_PAGE_41
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                                         /* 00h */
    U8                                  NumGroups;                                      /* 08h */
    U8                                  Reserved1[3];
    IOC_CONFIG_PAGE_EXT_MFG41_PHY_GROUP PhyGroup[IOC_NUM_PHYS];                         /* 0Ch */
} IOC_CONFIG_PAGE_EXT_MFG_PAGE_41, *PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_41;


/* Definition of Misc in PHY_GROUP */
#define IOC_PHY_GRP_MISC_ALWAYS_WSYNC_EN            (0x00000001)    /* Bit  0, Always word synchronize */
#define IOC_PHY_GRP_MISC_TX_INVCM2                  (0x00000002)    /* Bit  1       */
#define IOC_PHY_GRP_MISC_MANUAL_SD_OPMODE_MASK      (0x000000FC)    /* Bit  2~7     */
#define IOC_PHY_GRP_MISC_MANUAL_SD_PROTOCOL_MASK    (0x00003F00)    /* Bit  8~13    */
#define IOC_PHY_GRP_MISC_MANUAL_SD_SSCTYPE_MASK     (0x0000C000)    /* Bit 14~15    */
#define IOC_PHY_GRP_MISC_RX_LOS_FILTER_RISE_MASK    (0x001F0000)    /* Bit 16~20    */
#define IOC_PHY_GRP_MISC_SD_RX_LOS_FILTER_EN        (0x00200000)    /* Bit 21       */

#define IOC_PHY_GRP_MISC_RX_LOS_FILTER_FALL_MASK    (0x1F000000)    /* Bit 24~28    */
#define IOC_PHY_GRP_MISC_MANUAL_OPMODE_EN           (0x20000000)    /* Bit 29       */
#define IOC_PHY_GRP_MISC_MANUAL_PROTOCOL_EN         (0x40000000)    /* Bit 30       */
#define IOC_PHY_GRP_MISC_MANUAL_SSC_EN              (0x80000000)    /* Bit 31       */


/*****************************************************************************
*
*  Extended Manufacturing Page 42 Config Page
*
*****************************************************************************/

/* Istwi Controller information */
typedef struct _MFG_42_ISTWI_CONTROLLER_INFO
{
    U16                         SlaveAddress;                               /* 00h */
    U16                         Flags;                                      /* 02h */
    U32                         Reserved1;                                  /* 04h */
    U32                         Reserved2;                                  /* 08h */
} MFG_42_ISTWI_CONTROLLER_INFO, *PTR_MFG_42_ISTWI_CONTROLLER_INFO;

/* ISTWI controller flags fields */
#define IOC_CFG_EXT_MFG_42_ISTWI_CTRL_FLAGS_DISABLE                 (0x0080)    /* Disable this controller when set */
#define IOC_CFG_EXT_MFG_42_ISTWI_CTRL_FLAGS_FIFO_SIZE_80_BYTE       (0x0004)    /* Indicates 80 byte FIFOs, cleared means 8 byte FIFOs */
#define IOC_CFG_EXT_MFG_42_ISTWI_CTRL_FLAGS_ISTWI_SIO_ROUTING       (0x0002)    /* 0 = route SGPIO signals. 1 = route ISTWI signals */

#define IOC_CFG_EXT_MFG_42_ISTWI_CTRL_FLAGS_SLAVE_ADDR_SIZE_MASK    (0x0001)
#define IOC_CFG_EXT_MFG_42_ISTWI_CTRL_FLAGS_ADDR_SIZE_07_BIT        (0x0000)
#define IOC_CFG_EXT_MFG_42_ISTWI_CTRL_FLAGS_ADDR_SIZE_10_BIT        (0x0001)

typedef struct _IOC_CONFIG_PAGE_EXT_MFG_PAGE_42
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                             /* 00h */
    U8                                  NumISTWIControllers;                /* 08h */
    U8                                  Reserved1;                          /* 09h */
    U16                                 Reserved2;                          /* 0Ah */
    U32                                 Reserved3;                          /* 0Ch */
    U32                                 Reserved4;                          /* 10h */
    U32                                 Reserved5;                          /* 14h */
    MFG_42_ISTWI_CONTROLLER_INFO        ISTWIControllerInfo[IOC_NUM_ISTWI_CONTROLLERS_MAX]; /* 18h */
} IOC_CONFIG_PAGE_EXT_MFG_PAGE_42, *PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_42;


/*****************************************************************************
*
*  Extended Manufacturing Page 43 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_EXT_MAN_PAGE43_ISTWI_DEV_INFO_ENTRIES
#define IOC_EXT_MAN_PAGE43_ISTWI_DEV_INFO_ENTRIES       (1)
#endif

/* Istwi Device information */
typedef struct _MFG_43_ISTWI_DEVICE_INFO
{
    U32                         Flags;                                      /* 00h */
    U8                          DeviceType;                                 /* 04h */
    U8                          ISTWIControllerNum;                         /* 05h */
    U16                         DeviceAddr;                                 /* 06h */
    U8                          MuxType;                                    /* 08h */
    U8                          MuxChannel;                                 /* 09h */
    U16                         MuxAddr;                                    /* 0Ah */
    U32                         DeviceSpecific[2];                          /* 0Ch */
    U32                         Reserved1;                                  /* 14h */
    U32                         Reserved2;                                  /* 18h */
} MFG_43_ISTWI_DEVICE_INFO, *PTR_MFG_43_ISTWI_DEVICE_INFO;

/* ISTWI Device Flags field defines */
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_DEV_BUS_SPEED_MASK            (0x000000F0)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_DEV_BUS_SPEED_100_KHZ         (0x00000000)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_DEV_BUS_SPEED_400_KHZ         (0x00000010)

#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_DEV_ADDR_SIZE_MASK            (0x00000004)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_DEV_ADDR_SIZE_07_BIT          (0x00000000)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_DEV_ADDR_SIZE_10_BIT          (0x00000004)

#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_MUX_ADDR_SIZE_MASK            (0x00000002)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_MUX_ADDR_SIZE_07_BIT          (0x00000000)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_MUX_ADDR_SIZE_10_BIT          (0x00000002)

#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_MUX_PRESENT_MASK              (0x00000001)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_MUX_PRESENT                   (0x00000001)
#define IOC_CFG_EXT_MFG_43_ISTWI_FLAGS_MUX_NOT_PRESENT               (0x00000000)

/* ISTWI device types */
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_UNKNOWN                    (0x00)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_BOOT_ROM                   (0x01)  /* SBR */
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_SEP                        (0x02)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_DDR_SPD                    (0x03)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_MUX                        (0x04)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_SIGNAL_CONDITIONER         (0x05)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_SEEPROM                    (0x06)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_TEMP_SENSOR                (0x07)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_SERVICE_PROCESSOR          (0x08)
#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_FAN_CTRLR                  (0x09)

#define IOC_CFG_EXT_MFG_43_ISTWI_DEV_TYPE_VENDOR_SPECIFIC            (0xE0)  /* 0xE0 - 0xFF are Vendor Specific */

/* Temperature Sensor Types within the Device Specific field */
#define IOC_CFG_EXT_MFG_43_ISTWI_TEMP_SENSOR_MAX6654                (0x00)  /* Max 6654 Temperature sensor */
#define IOC_CFG_EXT_MFG_43_ISTWI_TEMP_SENSOR_EMC1442                (0x01)  /* SMSC EMC1442 Temperature sensor (2 Channels)*/
#define IOC_CFG_EXT_MFG_43_ISTWI_TEMP_SENSOR_ADT7476                (0x02)

/* Fan Controller Specific fields within the Device Specific field */
#define IOC_CFG_EXT_MFG_43_FANCTRL_FANCTRLTYPE_MASK             (0x0000000F)
#define IOC_CFG_EXT_MFG_43_FANCTRL_FANCTRLTYPE_MAX6650          (0x00000000)
#define IOC_CFG_EXT_MFG_43_FANCTRL_PERCFANSPEED_MASK            (0x00000070)
#define IOC_CFG_EXT_MFG_43_FANCTRL_PERCFANSPEED_SHIFT           (0x04)
#define IOC_CFG_EXT_MFG_43_FANCTRL_DONOTUSE_MASK                (0x00000080)
#define IOC_CFG_EXT_MFG_43_FANCTRL_DONOTUSE_INITIALIZE          (0x00000000)
#define IOC_CFG_EXT_MFG_43_FANCTRL_DONOTUSE_NOT_INIT            (0x00000080)
#define IOC_CFG_EXT_MFG_43_FANCTRL_FANSPEED_MASK                (0x0000FF00)
#define IOC_CFG_EXT_MFG_43_FANCTRL_FANSPEED_SHIFT               (0x08)

/* MAX6650 Fan Controller Specific fields within Fan Controller Type Specific field */
#define IOC_CFG_EXT_MFG_43_FANMAX6650_PRESCALER_MASK            (0x00000007)
#define IOC_CFG_EXT_MFG_43_FANMAX6650_FANCTRLVOL_MASK           (0x00000008)
#define IOC_CFG_EXT_MFG_43_FANMAX6650_FANCTRLV_5V               (0x00000000)
#define IOC_CFG_EXT_MFG_43_FANMAX6650_FANCTRLV_12V              (0x00000008)
#define IOC_CFG_EXT_MFG_43_FANMAX6650_COUNTTIME_MASK            (0x00000300)
#define IOC_CFG_EXT_MFG_43_FANMAX6650_COUNTTIME_SHIFT           (0x08)


typedef struct _IOC_CONFIG_PAGE_EXT_MFG_PAGE_43
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                             /* 00h */
    U8                                  NumIstwiDevices;                    /* 08h */
    U8                                  Reserved1;                          /* 09h */
    U16                                 Reserved2;                          /* 0Ah */
    U32                                 Reserved3;                          /* 0Ch */
    U32                                 Reserved4;                          /* 10h */
    U32                                 Reserved5;                          /* 14h */
    MFG_43_ISTWI_DEVICE_INFO            ISTWIDeviceInfo[IOC_EXT_MAN_PAGE43_ISTWI_DEV_INFO_ENTRIES]; /* 18h */
} IOC_CONFIG_PAGE_EXT_MFG_PAGE_43, *PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_43;


/*****************************************************************************
*
*  Extended Manufacturing Page 44 Config Page
*
*****************************************************************************/

typedef struct _MFG_44_CONFIG_PAGE_PCIE_ALLOC_INFO
{
    U64                                 PcieBase;                           /* 00h */
    U64                                 PcieSize;                           /* 08h */
    U32                                 PcieFlags;                          /* 10h */
    U32                                 Reserved;                           /* 14h 64-bit align */
} MFG_44_CONFIG_PAGE_PCIE_ALLOC_INFO, *PTR_MFG_44_CONFIG_PAGE_PCIE_ALLOC_INFO;

/* Indicates that the address pool element contains 64 bit address space. */
#define IOC_CFG_EXT_MFG_44_ALLOC_INFO_64_BIT    (0x00000002)
#define IOC_CFG_EXT_MFG_44_ALLOC_INFO_32_BIT    (0x00000000)

#ifndef IOC_NUM_PCIE_ALLOC_INFO_MAX
#define IOC_NUM_PCIE_ALLOC_INFO_MAX     (3)
#endif

typedef struct _IOC_CONFIG_PAGE_EXT_MFG_PAGE_44
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                             /* 00h */
    U8                                  AllocMode;                          /* 08h */
    U8                                  NumPcieAlloc;                       /* 09h */
    U8                                  VfidShift;                          /* 0Ah */
    U8                                  VfidMax;                            /* 0BH */
    U32                                 Reserved_0C;                        /* 0CH */
    U32                                 Reserved_10;                        /* 10h */
    U32                                 Reserved_14;                        /* 14h */
    MFG_44_CONFIG_PAGE_PCIE_ALLOC_INFO  PcieAllocInfo[IOC_NUM_PCIE_ALLOC_INFO_MAX]; /* 18h */
} IOC_CONFIG_PAGE_EXT_MFG_PAGE_44, *PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_44;

#define IOC_CFG_EXT_MFG_44_ALLOC_FLAGS_MODE_FAULT       (0x80)           /* Firmware should fault on configuration failure */
#define IOC_CFG_EXT_MFG_44_ALLOC_FLAGS_MODE_BAR_MASK    (0x7F)
#define IOC_CFG_EXT_MFG_44_ALLOC_FLAGS_MODE_BAR_ONLY    (0x00)           /* Indicates that the system should fail PDB if BAR-allocation method fails */
#define IOC_CFG_EXT_MFG_44_ALLOC_FLAGS_MODE_BAR_FALL    (0x01)           /* Indicates that the system should attempt to use NVDATA settings as a fallback if BAR-allocation fails */
#define IOC_CFG_EXT_MFG_44_ALLOC_FLAGS_MODE_NVDATA_ONLY (0x02)           /* Indicates that the system should always use NVDATA settings */

/****************************************************************************
*   Vendor Specific Config Pages
****************************************************************************/
typedef struct _IOC_CONFIG_PAGE_VENDOR
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                                            /* 0x00 */
    U32                                 VendorData[IOC_CONFIG_PAGE_VENDOR_WORDS ];         /* 0x08 */
} IOC_CONFIG_PAGE_VENDOR;



/****************************************************************************
*   Product Specific MPI Definitions
****************************************************************************/
/* Host Based Discovery Action */
#define IOC_HBD_ADDINFO_DEV_ON_BUFFERED_PHY       (0x00000001) /* DataBolt: Device is attached to a buffered phy */
#define IOC_HBD_ADDINFO_RESERVED01                (0x00000002) /* Reserved for future use */



/****************************************************************************
*   Configuration Page Type Union
****************************************************************************/
typedef union _PAGE_IOC
{
    U32                                 HeaderU32;
    MPI2_CONFIG_PAGE_HEADER             Header;
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    ExtHeader;
    MPI2_CONFIG_PAGE_MAN_0              MfgPage0;
    MPI2_CONFIG_PAGE_MAN_1              MfgPage1;
    MPI2_CONFIG_PAGE_MAN_2              MfgPage2;
    IOC_CONFIG_PAGE_MFG_PAGE_3          MfgPage3;     /* GEN2 only */
    MPI2_CONFIG_PAGE_MAN_4              MfgPage4;
    MPI2_CONFIG_PAGE_MAN_5              MfgPage5;
    IOC_CONFIG_PAGE_MFG_PAGE_6          MfgPage6;
    MPI2_CONFIG_PAGE_MAN_7              MfgPage7;
    IOC_CONFIG_PAGE_MFG_PAGE_9          MfgPage9;
    IOC_CONFIG_PAGE_MFG_PAGE_10         MfgPage10;
    IOC_CONFIG_PAGE_MFG_PAGE_11         MfgPage11;
    IOC_CONFIG_PAGE_MFG_PAGE_12         MfgPage12;
    IOC_CONFIG_PAGE_MFG_PAGE_13         MfgPage13;
    IOC_CONFIG_PAGE_MFG_PAGE_14         MfgPage14;
    IOC_CONFIG_PAGE_MFG_PAGE_16         MfgPage16;
    IOC_CONFIG_PAGE_MFG_PAGE_17         MfgPage17;
    IOC_CONFIG_PAGE_MFG_PAGE_19         MfgPage19;

    IOC_CONFIG_PAGE_MFG_PAGE_30         MfgPage30;
    IOC_CONFIG_PAGE_MFG_PAGE_31         MfgPage31;
    IOC_CONFIG_PAGE_EXT_MFG_PAGE_40     ExtMfgPage40;
    IOC_CONFIG_PAGE_EXT_MFG_PAGE_41     ExtMfgPage41;
    IOC_CONFIG_PAGE_EXT_MFG_PAGE_42     ExtMfgPage42;
    IOC_CONFIG_PAGE_EXT_MFG_PAGE_43     ExtMfgPage43;
    IOC_CONFIG_PAGE_EXT_MFG_PAGE_44     ExtMfgPage44;

    MPI2_CONFIG_PAGE_IO_UNIT_0          IoUnitPage0;
    MPI2_CONFIG_PAGE_IO_UNIT_1          IoUnitPage1;
    MPI2_CONFIG_PAGE_IO_UNIT_3          IoUnitPage3;
    MPI2_CONFIG_PAGE_IO_UNIT_5          IoUnitPage5;
    MPI2_CONFIG_PAGE_IO_UNIT_6          IoUnitPage6;
    MPI2_CONFIG_PAGE_IO_UNIT_7          IoUnitPage7;
    MPI2_CONFIG_PAGE_IO_UNIT_8          IoUnitPage8;
    MPI2_CONFIG_PAGE_IO_UNIT_9          IoUnitPage9;
#if CBB_COMPILE_OPTION_SRIOV_EN
    MPI2_CONFIG_PAGE_IO_UNIT_10         IoUnitPage10;
#endif
    MPI26_CONFIG_PAGE_IO_UNIT_11        IoUnitPage11;
    MPI2_CONFIG_PAGE_IOC_0              IocPage0;
    MPI2_CONFIG_PAGE_IOC_1              IocPage1;
    MPI2_CONFIG_PAGE_IOC_7              IocPage7;
    MPI2_CONFIG_PAGE_IOC_8              IocPage8;
    MPI2_CONFIG_PAGE_BIOS_1             Bios1;
    MPI2_CONFIG_PAGE_BIOS_2             Bios2;
    MPI2_CONFIG_PAGE_BIOS_3             Bios3;
    MPI2_CONFIG_PAGE_BIOS_4             Bios4;
    MPI2_CONFIG_PAGE_SASIOUNIT_0        SasIoUnitPage0;
    MPI2_CONFIG_PAGE_SASIOUNIT_1        SasIoUnitPage1;
    MPI2_CONFIG_PAGE_SASIOUNIT_5        SasIoUnitPage5;
    MPI2_CONFIG_PAGE_SASIOUNIT_6        SasIoUnitPage6;
    MPI2_CONFIG_PAGE_SASIOUNIT_7        SasIoUnitPage7;
    MPI2_CONFIG_PAGE_SASIOUNIT_8        SasIoUnitPage8;
    MPI2_CONFIG_PAGE_SASIOUNIT16        SasIoUnitPage16;
    MPI2_CONFIG_PAGE_EXPANDER_0         SasExpanderPage0;
    MPI2_CONFIG_PAGE_EXPANDER_1         SasExpanderPage1;
    MPI2_CONFIG_PAGE_SAS_DEV_0          SasDevicePage0;
    MPI2_CONFIG_PAGE_SAS_DEV_1          SasDevicePage1;
    MPI2_CONFIG_PAGE_SAS_PHY_0          SasPhyPage0;
    MPI2_CONFIG_PAGE_SAS_PHY_1          SasPhyPage1;
    MPI2_CONFIG_PAGE_SAS_PHY_2          SasPhyPage2;
    MPI2_CONFIG_PAGE_SAS_PHY_3          SasPhyPage3;
    MPI2_CONFIG_PAGE_SAS_PHY_4          SasPhyPage4;
    MPI2_CONFIG_PAGE_SAS_ENCLOSURE_0    SasEnclPage0;
    MPI2_CONFIG_PAGE_SAS_PORT_0         SasPortPage0;

    MPI2_CONFIG_PAGE_LOG_0              Log0;
    MPI2_CONFIG_PAGE_DRIVER_MAPPING_0   DriverMapping0;

    MPI2_CONFIG_PAGE_ETHERNET_0         EthernetPage0;
    MPI2_CONFIG_PAGE_ETHERNET_1         EthernetPage1;

    MPI26_CONFIG_PAGE_PIOUNIT_0         PcieIoUnitPage0;
    MPI26_CONFIG_PAGE_PIOUNIT_1         PcieIoUnitPage1;
    MPI26_CONFIG_PAGE_PIOUNIT_4         PcieIoUnitPage4;

    MPI26_CONFIG_PAGE_PSWITCH_0         PcieSwitchPage0;
    MPI26_CONFIG_PAGE_PSWITCH_1         PcieSwitchPage1;

    MPI26_CONFIG_PAGE_PCIEDEV_0         PcieDevicePage0;
    MPI26_CONFIG_PAGE_PCIEDEV_1         PcieDevicePage1;
    MPI26_CONFIG_PAGE_PCIEDEV_2         PcieDevicePage2;
    MPI26_CONFIG_PAGE_PCIELINK_1        PcieLinkPage1;
    MPI26_CONFIG_PAGE_PCIELINK_2        PcieLinkPage2;
    MPI26_CONFIG_PAGE_PCIELINK_3        PcieLinkPage3;

    IOC_CONFIG_PAGE_VENDOR              VendorPage;
#if (RAID)
    MPI2_CONFIG_PAGE_IOC_6              IocPage6;
    MPI2_CONFIG_PAGE_RAID_VOL_0         RaidVolPage0;
    MPI2_CONFIG_PAGE_RAID_VOL_1         RaidVolPage1;
    MPI2_CONFIG_PAGE_RD_PDISK_0         RaidPhysDiskPage0;
    MPI2_CONFIG_PAGE_RD_PDISK_1         RaidPhysDiskPage1;
    MPI2_CONFIG_PAGE_RAID_CONFIGURATION_0 RaidConfigPage0;
#endif /* (RAID) */

} PAGE_IOC, *PTR_PAGE_IOC;

#endif /* !defined(__LANGUAGE_ASM__) */


/*
 * If any of the "IOC" pages above or pages in mpi2_cnfg.h have new
 * fields added or change in size we must increment the major field
 * of IOPI_CFGI_NVDATA_VERSION. From "How NVDATA is used":
 *   The Major version number changes when the MPT Configuration Pages have changed.
 *   Any of the following will result in a change to the Major version number:
 *      *A Configuration Page has increased in length
 *      *A new value has been added to a Configuration Page
 *      *A new Configuration Page has been created
 *      *A default value has been updated
 * Major - 16 bits, [15:8]==Phase [7:6]=sub-phase [5:0]==NVDATA change within subphase
 */
#if TIGER
#define IOPI_CFGI_NVDATA_VERSION_MAJOR  (0x0020)    /* Major version reflects format of config pages */
#define IOPI_CFGI_NVDATA_VERSION_MINOR  (0x0000)    /* Minor version reflects specific values, used by customer */
#elif VENTURA
#define IOPI_CFGI_NVDATA_VERSION_MAJOR  (0x0025)    /* Major version reflects format of config pages */
#define IOPI_CFGI_NVDATA_VERSION_MINOR  (0x0000)    /* Minor version reflects specific values, used by customer */
#elif INTRUDER
#define IOPI_CFGI_NVDATA_VERSION_MAJOR  (0x0B03)    /* Major version reflects format of config pages, see comment block above */
#define IOPI_CFGI_NVDATA_VERSION_MINOR  (0x0000)    /* Minor version reflects specific values, used by customer */
#endif

#define IOPI_CFGI_NVDATA_VERSION        ((IOPI_CFGI_NVDATA_VERSION_MAJOR << 16)|IOPI_CFGI_NVDATA_VERSION_MINOR)

#endif /* end of file */
