/***************************************************************************
 *                                                                         *
 *  Copyright 2003-2014 Avago Technologies.  All rights reserved.          *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiIocLogInfo.h
 *          Title:  SAS Firmware IOP Interface IOC Log Info Definitions
 *     Programmer:
 *  Creation Date:  September 24, 2003
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface IOC Log Info codes
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPI_IOCLOGINFO_H_INCLUDED
#define IOPI_IOCLOGINFO_H_INCLUDED


/****************************************************************************/
/*  IOC LOGINFO defines, 0x00000000 - 0x0FFFFFFF                            */
/*  Format:                                                                 */
/*      Bits 31-28: MPI2_IOCLOGINFO_TYPE_SAS (3)                            */
/*      Bits 27-24: IOC_LOGINFO_ORIGINATOR: 0=IOP, 1=PL, 2=IR               */
/*      Bits 23-16: LOGINFO_CODE                                            */
/*      Bits 15-0:  LOGINFO_CODE Specific                                   */
/****************************************************************************/

/****************************************************************************/
/* IOC_LOGINFO_ORIGINATOR defines                                           */
/****************************************************************************/

/* See pliIocLogInfo.h */

/****************************************************************************/
/* LOGINFO_CODE defines                                                     */
/****************************************************************************/

/* See pliIocLogInfo.h */

/****************************************************************************/
/* IOP LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IOP          */
/****************************************************************************/
#define IOP_LOGINFO_CODE_INVALID_SAS_ADDRESS               (0x00010000) /* Invalid SAS address. */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE               (0x00030000) /* Invalid configuration page. */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_RT            (0x00030100) /* Route table entry was not found. The lowest eight bits represent the page type. */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_PN            (0x00030200) /* Invalid page number. */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_FORM          (0x00030300) /* Invalid form in the page address. */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_DEFAULT       (0x00030700) /* Default page was not found. */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_STATE         (0x00030800) /* FW Downgrade active, so can't access persistent config pages */

#define IOP_LOGINFO_CODE_DIAG_MSG_ERROR                    (0x00040000) /* Error handling diagnostic message is ORed with diagnostic status. */

#define IOP_LOGINFO_CODE_TASK_TERMINATED                   (0x00050000) /* SCSI I/O task terminated. */

#define IOP_LOGINFO_CODE_ENCL_MGMT_READ_ACTION_ERROR       (0x00060001) /* Read action is not supported for the SEP message. */
#define IOP_LOGINFO_CODE_ENCL_MGMT_INVALID_DEVH_ERROR      (0x00060002) /* Invalid device handle in the SEP message. */

#define IOP_LOGINFO_CODE_SCSI_IO_NOT_REGISTERED            (0x00070000) /* Unregistered SCSI I/O. */
#define IOP_LOGINFO_CODE_SCSI_IO_DEVICE_BLOCKED            (0x00070001) /* Blocked SCSI I/O device. */

#define IOP_LOGINFO_CODE_VP_SAS_ADDRESS_INVALID            (0x00080001) /* Invalid virtual port SAS address. */
#define IOP_LOGINFO_CODE_VP_SAS_ADDRESS_NOT_AVAILABLE      (0x00080002) /* Unavailable virtual port SAS address. */

/*flash error*/
#define IOP_LOGINFO_CODE_FLASH_NOT_PRESENT                 (0x00010101) /* Flash is not present. */
#define IOP_LOGINFO_CODE_FLASH_ERASE_FAILED                (0x00010102) /* Erasing Flash failed. */
#define IOP_LOGINFO_CODE_FLASH_WRITE_FAILED                (0x00010103) /* Writing to Flash failed. Bad init image header */
#define IOP_LOGINFO_CODE_FLASH_WRITE_FAILED_4              (0x00010104) /* Writing to Flash failed. Bad init image write */
#define IOP_LOGINFO_CODE_FLASH_WRITE_FAILED_5              (0x00010105) /* Writing to Flash failed. MR Header */
#define IOP_LOGINFO_CODE_FLASH_WRITE_FAILED_6              (0x00010106) /* Writing to Flash failed. CBB*/
#define IOP_LOGINFO_CODE_FLASH_WRITE_FAILED_7              (0x00010107) /* Writing to Flash failed. Mr Header */
#define IOP_LOGINFO_CODE_FLASH_WRITE_FAILED_8              (0x00010108) /* Writing to Flash failed. General Data Write */

/*image error while trying to FW download*/
#define IOP_LOGINFO_CODE_IMAGE_BOOTLOADER_NOT_PRESENT      (0x00010401) /* The bootloader image was not found. */
#define IOP_LOGINFO_CODE_IMAGE_MAIN_CHECKSUM_FAILURE       (0x00010402) /* The main image checksum failed.*/
#define IOP_LOGINFO_CODE_IMAGE_EXT_CHECKSUM_FAILURE        (0x00010403) /* The extended image checksum failed. */
#define IOP_LOGINFO_CODE_IMAGE_INIT_NOT_PRESENT            (0x00010404) /* The initialization image is not present. */
#define IOP_LOGINFO_CODE_IMAGE_INIT_ADDRESS_ERROR          (0x00010405) /* The start address and image size of the initialization image do not meet restrictions.*/
#define IOP_LOGINFO_CODE_IMAGE_NOT_MODEL_TO_FWDL           (0x00010406) /* The host tried to perform a firmware download when code is run from Flash.  */
#define IOP_LOGINFO_CODE_IMAGE_FLASH_SIZE_NOT_MATCH        (0x00010407) /* The Flash size does not match the total size in the complete firmware download request.*/
#define IOP_LOGINFO_CODE_IMAGE_ENCRYPT_HASH_NOT_PRESENT    (0x00010408) /* The encrypted hash image is not found. */
#define IOP_LOGINFO_CODE_IMAGE_ENCRYPT_HASH_INVALID        (0x00010409) /* Invalid encrypted hash in the image. */
#define IOP_LOGINFO_CODE_IMAGE_BIOS_CHECKSUM_FAIL          (0x0001040A) /* The BIOS image checksum failed. */
#define IOP_LOGINFO_CODE_IMAGE_ENCRYPT_HASH_MEM_ALLOC_FAIL (0x0001040B) /* The memory allocation failed. */
#define IOP_LOGINFO_CODE_IMAGE_ENCRYPT_HASH_BIOS_INVALID   (0x0001040C) /* Invalid encrypted hash in the BIOS image. */
#define IOP_LOGINFO_CODE_IMAGE_BIOS_SEGMENT_SIZE_INVALID   (0x0001040D) /* The BIOS segment size is invalid. */
#define IOP_LOGINFO_CODE_IMAGE_BIOS_SHA_CALC_FAILED        (0x0001040E) /* The BIOS SHA256 calculation failed. */
#define IOP_LOGINFO_CODE_IMAGE_BIOS_HASH_DECRYPT_FAILED    (0x0001040F) /* The BIOS Hash Decryption failed. */

/* Toolbox Request error */
#define IOP_LOGINFO_CODE_TOOLBOX_CLEAN_INVALID             (0x00010501) /* FW Downgrade bit - bit 26 cannot be set along with other flag bits in Toolbox Clean request. */

/* Init problems */
#define IOP_LOGINFO_CODE_PORT_ENABLE_RESOURCES_INSUF       (0x00010600) /* Insufficient resources for full port enable (backend PCIe). */
#define IOP_LOGINFO_CODE_PORT_ENABLE_PDB_PLL_NOT_LOCKED    (0x00010601) /* PDB PLL not locked, so backend PCIe disabled. Check that SBR loaded correctly */


/*ISTWI error */
#define IOP_LOGINFO_CODE_ISTWI_BUS_NOT_RESERVED            (0x00090000) /* ISTWI bus is not reserved. */
#define IOP_LOGINFO_CODE_ISTWI_DEVICE_NOT_PRESENT          (0x00090001) /* ISTWI device is not present. */
#define IOP_LOGINFO_CODE_ISTWI_BUS_RESERVED_TWICE          (0x00090002) /* ISTWI bus is reserved twice. */
#define IOP_LOGINFO_CODE_ISTWI_INVALID_SGL                 (0x00090003) /* Invalid SGL for the ISTWI device. */
#define IOP_LOGINFO_CODE_ISTWI_INVALID_SUBSYSTEM           (0x00090004) /* Invalid subsystem in the ISTWI request. */


/*iButton error*/
#define IOP_LOGINFO_CODE_IBUTTON_INVALID                   (0x00090010) /* Reserved. */

#define IOP_LOGINFO_CODE_PWR_MGMT                       (0x000A0000) /* Any well-defined, known power management status. See the following table. */
#define IOP_LOGINFO_CODE_UNDEF_PWR_MGMT                 (0x000A8000) /* An undefined power management status. */
#define IOP_LOGINFO_SUBCODE_PWRM_LINK_SPEED_TRAIN_ERR   (0x00000001) /* A request to change the PCIe link speed failed because of a PCIe training error. */
#define IOP_LOGINFO_SUBCODE_PWRM_LNK_WIDTH_TRAIN_ERR_2  (0x00000002) /* A request to change the PCIe link width failed. The link had a training error. */
#define IOP_LOGINFO_SUBCODE_PWRM_LNK_WIDTH_TRAIN_ERR    (0x00000003) /* A request to change the PCIe link width failed. A link training error occurred after attempting the change. */
#define IOP_LOGINFO_SUBCODE_PWRM_INVAL_PCIE_LINK_SPEED  (0x00000004) /* A bad PCIe link speed field in the request message. */
#define IOP_LOGINFO_SUBCODE_PWRM_INVAL_PCIE_LINK_WIDTH  (0x00000005) /* A bad PCIe link width field in the request message. */
#define IOP_LOGINFO_SUBCODE_PWRM_HW_GATEWAY_DISABL_FAIL (0x00000006) /* Unable to disable the AHB gateway. */
#define IOP_LOGINFO_SUBCODE_PWRM_RESULT_DIF_FROM_REQ    (0x00000007) /* The requested PCIe link speed or width does not equal the actual link speed or width. */
#define IOP_LOGINFO_SUBCODE_PWRM_HSM_NOT_SUPPORTED      (0x00000008) /* The half-speed mode is not supported because the CPU is not running at 800 MHz. */
#define IOP_LOGINFO_SUBCODE_PWRM_INVALID_CPU_SPEED      (0x00000009) /* Invalid requested CPU speed.  */
#define IOP_LOGINFO_SUBCODE_PWRM_INVALID_FEATURE        (0x0000000A) /* The Feature field value in the MPI Power Management Control request is bad. */
#define IOP_LOGINFO_SUBCODE_PWRM_CFG_PG_CONFLICT        (0x0000000B) /* The requested field in the MPI Power Management Control request conflicts with Configuration Page settings. See SAS IO Unit Page 7 or SAS IO Unit Page 8 for various Power Management configuration settings. */

#define IOP_LOGINFO_SUBCODE_PWRM_HOST_CTRL_CONFLICT     (0x0000000C) /* The requested action conflicts with the Global Power Management PM_CONTROL state. */

#define IOP_LOGINFO_CODE_RACC_ERROR                        (0x000B0000) /* RAID Accelerator error. Use one of the sub codes defined in the following table. */
/**************************************************************************/
/* These RACC Sub-Codes are bit definitions, to allow combinations        */
/* i.e. FAIL_REQUEST, EEDP_GUARD_ERROR, DMA_0_HALT can all be set at once */
/**************************************************************************/
#define IOP_LOGINFO_SUBCODE_RACC_COMPLETE_W_ERROR          (0x00000001) /* The I/O completed, but with an error. */
#define IOP_LOGINFO_SUBCODE_RACC_FAIL_REQUEST              (0x00000002) /* The RAID accelerator request failed because of an error. */
#define IOP_LOGINFO_SUBCODE_RACC_EEDP_GUARD_ERROR          (0x00000004) /* Set when the EEDP Guard Miscompare Halt Condition is detected. */
#define IOP_LOGINFO_SUBCODE_RACC_EEDP_REF_TAG_ERROR        (0x00000008) /* Set when the EEDP Ref Tag Miscompare Halt Condition is detected. */
#define IOP_LOGINFO_SUBCODE_RACC_EEDP_APP_TAG_ERROR        (0x00000010) /* Set when the EEDP App Tag Miscompare Halt Condition is detected. */
#define IOP_LOGINFO_SUBCODE_RACC_PLB_SLAVE_0_ERROR         (0x00000020) /* PLB Slave 0 detected an error. */
#define IOP_LOGINFO_SUBCODE_RACC_CFG_ACCESS_ERROR          (0x00000040) /* AI slave connected to PLB0 detected an access error. */
#define IOP_LOGINFO_SUBCODE_RACC_DMA_1_COMP_Q_FULL         (0x00000080) /* DMA 1 completion queue full. */
#define IOP_LOGINFO_SUBCODE_RACC_DMA_0_COMP_Q_FULL         (0x00000100) /* DMA 0 completion queue full. */
#define IOP_LOGINFO_SUBCODE_RACC_DMA_1_HALT_ERROR          (0x00000200) /* DMA 1 detected an error and halted. */
#define IOP_LOGINFO_SUBCODE_RACC_DMA_0_HALT_ERROR          (0x00000400) /* DMA 0 detected an error and halted. */


/****************************************************************************/
/* PL LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = PL            */
/****************************************************************************/

/* See pliIocLogInfo.h */


/****************************************************************************/
/* IR LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IR            */
/****************************************************************************/
#define IR_LOGINFO_RAID_ACTION_ERROR                           (0x00010000) /* RAID action error. */
#define IR_LOGINFO_VOLUME_CREATE_INVALID_LENGTH                (0x00010001) /* The amount of information passed down to create volume is too large. */
#define IR_LOGINFO_VOLUME_CREATE_NO_SLOTS                      (0x00010003) /* Volume creation failed because the maximum number of supported volumes was exceeded. */
#define IR_LOGINFO_VOLUME_CREATE_DMA_ERROR                     (0x00010004) /* Volume creation failed because of a DMA error when trying to read from host. */
#define IR_LOGINFO_VOLUME_MFG_PAGE4_ERROR                      (0x00010006) /* Volume creation failed because of an error when reading Manufacturing Page 4. */
#define IR_LOGINFO_VOLUME_CREATE_NO_BGI                        (0x00010008) /* Volume creation failed because the required Background Init flag was not set. */
#define IR_LOGINFO_VOLUME_CREATE_ODD_NUM_PHYS_DISKS            (0x00010009) /* Volume creation failed because an odd number of drives in the volume is not permitted. */

#define IR_LOGINFO_VOLUME_ACTIVATING_AN_ACTIVE_VOLUME          (0x00010010) /* Volume activation failed because the volume is already active. */
#define IR_LOGINFO_VOLUME_ACTIVATING_INVALID_VOLUME_TYPE       (0x00010011) /* Volume activation failed because the volume type is unsupported. */
#define IR_LOGINFO_VOLUME_ACTIVATING_TOO_MANY_VOLUMES          (0x00010012) /* Volume activation failed because of activating too many volumes. */
#define IR_LOGINFO_VOLUME_ACTIVATE_VOLUME_FAILED               (0x00010014) /* Volume activation failed. Call to the activateVolume function returned a failure. */
#define IR_LOGINFO_VOLUME_ACTIVATING_IMPORT_VOLUME_FAILED      (0x00010015) /* Volume activation failed when trying to import the volume. */
#define IR_LOGINFO_VOLUME_ACTIVATING_TOO_MANY_PHYS_DISKS       (0x00010016) /* Volume activation failed because of trying to activate too many physical disks.*/
#define IR_LOGINFO_VOLUME_ACTIVATING_FAILED_STATE_VOLUME       (0x00010017) /* Volume activation failed. The volume is in Failed state. */
#define IR_LOGINFO_VOLUME_ACTIVATING_INVALID_METADATA_SIZE     (0x00010018) /* Volume activation failed because activation of a volume with 64 MB of metadata is not permitted. */
#define IR_LOGINFO_VOLUME_ACTIVATING_INVALID_PHYS_DISK         (0x00010019) /* Volume activation failed because the physical disk pages are not readable. */

#define IR_LOGINFO_PHYSDISK_CREATE_TOO_MANY_DISKS              (0x00010020) /* Physical disk creation failed because of too many physical disks. */
#define IR_LOGINFO_PHYSDISK_CREATE_CONFIG_PAGE_ERROR           (0x00010024) /* Physical disk creation failed because of an error while creating the RAID physical disk configuration page.*/
#define IR_LOGINFO_PHYSDISK_DUAL_PORT_FAILOVER                 (0x00010026) /* Physical disk I/O returned. Need time to remove device processing so that the I/O can fail over to the passive path on this dual-ported device. */
#define IR_LOGINFO_PHYSDISK_CREATE_DEV_HANDLE_INVALID          (0x00010027) /* Physical disk creation failed because of an invalid device handle. */
#define IR_LOGINFO_PHYSDISK_CREATE_DUPLICATE_DEV_HANDLE        (0x00010028) /* Physical disk creation failed because of an existing physical disk with this device handle. */
#define IR_LOGINFO_PHYSDISK_CREATE_PI_NOT_SUPPORTED            (0x00010029) /* Creation failed because the physical disk does not support protection information. */


#define IR_LOGINFO_COMPAT_ERROR_RAID_DISABLED                  (0x00010030) /* Compatibility error. IR is disabled. */
#define IR_LOGINFO_COMPAT_ERROR_INQUIRY_FAILED                 (0x00010031) /* Compatibility error. Inquiry command failed. */
#define IR_LOGINFO_COMPAT_ERROR_NOT_DIRECT_ACCESS              (0x00010032) /* Compatibility error. The device is not a direct-access device. */
#define IR_LOGINFO_COMPAT_ERROR_REMOVABLE_FOUND                (0x00010033) /* Compatibility error. A removable device was found. */
#define IR_LOGINFO_COMPAT_ERROR_NEED_SCSI_2_OR_HIGHER          (0x00010034) /* Compatibility error. The device SCSI version is not version 2 or higher. */
#define IR_LOGINFO_COMPAT_ERROR_SATA_48BIT_LBA_NOT_SUPPORTED   (0x00010035) /* Compatibility error. SATA device, 48-bit LBA is not supported. */
#define IR_LOGINFO_COMPAT_ERROR_DEVICE_NOT_512_BYTE_BLOCK      (0x00010036) /* Compatibility error. The device does not have 512-byte block sizes. */
#define IR_LOGINFO_COMPAT_ERROR_VOLUME_TYPE_CHECK_FAILED       (0x00010037) /* Compatibility error. The volume type check failed. */
#define IR_LOGINFO_COMPAT_ERROR_DISK_TOO_SMALL                 (0x00010039) /* Compatibility error. The disk drive capacity is too small to be used in RAID volume. */
#define IR_LOGINFO_COMPAT_ERROR_MEMBERSHIP_COUNT               (0x0001003B) /* Compatibility error. Membership count error because of too many or too few disks for the volume type. */
#define IR_LOGINFO_COMPAT_ERROR_INCOMPATIBLE_STRIPE_SIZE       (0x0001003E) /* Compatibility error. The disk stripe size must be 64 KB or 128 KB. */
#define IR_LOGINFO_COMPAT_ERROR_UNSUPPORTED_PROTOCOL           (0x0001003F) /* Compatibility error. The firmware does not support the disk protocol type.*/
#define IR_LOGINFO_COMPAT_ERROR_MIXED_PROTOCOL                 (0x00010040) /* Compatibility error. Mixed protocol volumes are not supported. */
#define IR_LOGINFO_COMPAT_ERROR_DISK_MAP                       (0x00010041) /* Compatibility error. The physical disk map or RAID set number is incorrect.*/
#define IR_LOGINFO_COMPAT_ERROR_DEVICE_NOT_PRESENT             (0x00010042) /* Compatibility error. The device is not present. */
#define IR_LOGINFO_COMPAT_ERROR_NOT_END_DEVICE                 (0x00010043) /* Compatibility error. The device is not an end device. */
#define IR_LOGINFO_COMPAT_ERROR_MEDIA_TYPE                     (0x00010044) /* Compatibility error. Incompatible media.*/
#define IR_LOGINFO_COMPAT_ERROR_ALTA_NOT_SUPPORTED             (0x00010045) /* Compatibility error. The LSISS2520 or LSISS2530 SAS-to-SATA bridge is detected. */
#define IR_LOGINFO_COMPAT_ERROR_DRIVE_BLOCKED                  (0x00010046) /* Compatibility error. A drive larger than 2.1 TB is detected and therefore blocked. */
#define IR_LOGINFO_COMPAT_ERROR_DRIVE_SENSE_DATA_INCOMPATIBLE  (0x00010047) /* Compatibility error. Found an incompatible Drive Sense Data format that the IR firmware could not change. The drive did not let the IR firmware change the Sense Data format.*/
#define IR_LOGINFO_COMPAT_ERROR_MIXED_PI                       (0x00010048) /* Compatibility error. Mixed protection information types are not supported.*/
#define IR_LOGINFO_COMPAT_ERROR_MIXED_BLOCKSIZE                (0x00010049) /* Compatibility error. Mixed block sizes are not supported. */
#define IR_LOGINFO_COMPAT_ERROR_LOWEST_ALIGNED_NOT_ZERO        (0x0001004B) /* Compatibility error. Lowest Aligned Logical Block Address from ReadCap16 is not zero. */
#define IR_LOGINFO_COMPAT_ERROR_INVALID_BLOCKSIZE              (0x0001004A) /* Compatibility error. Drive is not reporting a supported block size. */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_DFU_IN_PROGRESS           (0x00010050) /* Device firmware update. The update can only be started once. */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_DEVICE_IN_INVALID_STATE   (0x00010051) /* Device firmware update. The volume must be in Optimal state, Active state, or Nonquiesced state. */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_INVALID_TIMEOUT           (0x00010052) /* Device firmware update. The time-out value cannot be zero.*/
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_NO_TIMERS                 (0x00010053) /* Device firmware update. Failed to create timer. */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_READING_CFG_PAGE          (0x00010054) /* Device firmware update. Failed to read SAS I/O Unit Page 1 values. */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_PORT_IO_TIMEOUTS_REQUIRED (0x00010055) /* Device firmware update. Invalid SAS I/O Unit Page 1 values.*/
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_ALLOC_CFG_PAGE            (0x00010056) /* Device firmware update. Unable to allocate memory for a page.*/


#define IR_LOGINFO_COMPAT_CHECK_INVALID_REQUEST                (0x00010060) /* Compatibility check error. Invalid input. */
#define IR_LOGINFO_COMPAT_CHECK_INVALID_SOURCE_DEVH            (0x00010061) /* Compatibility check error. The input flags might be the cause of an invalid source device handle. */
#define IR_LOGINFO_COMPAT_CHECK_INVALID_CANDIDATE_DEVH         (0x00010062) /* Compatibility check error. The input flags might be the cause of an invalid candidate device handle. */
#define IR_LOGINFO_COMPAT_CHECK_VOLUME_NOT_ACTIVE              (0x00010063) /* Compatibility check error. The volume must be active to analyze. */


#define IR_LOGINFO_DELETE_PHYSDISK_INVALID                      (0x00010070) /* Deletion failed because of an invalid physical disk. */
#define IR_LOGINFO_DELETE_HOT_SPARE_INVALID                     (0x00010071) /* Hot spare deletion failed because it belongs to a volume member. */
#define IR_LOGINFO_DELETE_VOLUME_INVALID_CCOH_ENABLED           (0x00010072) /* CCOH enabled on WD. */
#define IR_LOGINFO_HOTSPARE_CREATE_TOO_MANY_HOTSPARES           (0x00010073) /* Hot spare creation failed because the maximum number of hot spares was exceeded. */

#define IR_LOGINFO_VOLUME_CREATE_FAILED_VOLUME_PRESENT          (0x00010075) /* Failed to create a volume because a failed volume was present. */
#define IR_LOGINFO_VOLUME_ACTIVATING_FAILED_VOLUME_PRESENT      (0x00010076) /* Failed to activate a volume because a failed volume was present. */

#define IR_LOGINFO_RAID_ACTION_IR_DISABLED                      (0x00010080) /* RAID action failed because IR was disabled. */

#if COMPILE_OPTION_WARHAWK
/*
 * NOTE:  All IR_LOGINFO codes with the prefix 0x00011 are reserved for Warpdrive firmware.
 *        Any Warpdrive specific loginfo should be defined with this prefix.
 */

/* CCoH Licensing Loginfo codes */
#define IR_LOGINFO_RAID_ACTION_LICENSE_NOT_SET                  (0x00011000) /* RAID action failed because the license was not set. */
#define IR_LOGINFO_LICENSE_INVALID_LENGTH                       (0x00011001) /* Invalid License length. */
#define IR_LOGINFO_LICENSE_UNSUPPORTED_SUBACTION                (0x00011002) /* Unsupported RAID sub action. */
#define IR_LOGINFO_LICENSE_ERROR_INVALID_KEY                    (0x00011003) /* Licensing: Invalid License key. */
#define IR_LOGINFO_LICENSE_ERROR_VENDORID_MISMATCH              (0x00011004) /* Licensing: Vendor ID mismatch. */
#define IR_LOGINFO_LICENSE_ERROR_DEVICEID_MISMATCH              (0x00011005) /* Licensing: Device ID mismatch. */
#define IR_LOGINFO_LICENSE_ERROR_SUBVENDORID_MISMATCH           (0x00011006) /* Licensing: SubVendor ID mismatch. */
#define IR_LOGINFO_LICENSE_ERROR_SUBDEVICEID_MISMATCH           (0x00011007) /* Licensing: SubDevice ID mismatch. */

/* Recover Volume RAID Action Loginfo codes */
#define IR_LOGINFO_DEV_REC_VOL_ERR_RV_IN_PROGRESS               (0x00011010) /* Recover Volume: Volume already started. */
#define IR_LOGINFO_DEV_REC_VOL_ERR_DEVICE_IN_INVALID_STATE      (0x00011011) /* Recover Volume: Volume must be in an Optimal, Active, and non-Quiesced state. */
#define IR_LOGINFO_DEV_REC_VOL_ERR_NO_TIMERS                    (0x00011012) /* Recover Volume: Create timer failed. */
#define IR_LOGINFO_DEV_REC_VOL_ERR_INVALID_VOL_HANDLE           (0x00011013) /* Recover Volume: Invalid volume device handle. */
#define IR_LOGINFO_DEV_REC_VOL_ERR_NOT_SUPPORTED                (0x00011014) /* Recover Volume: RVRA not supported in IT mode. */

#endif // WARHAWK

#if 0
/* Obsolete #defines - searched unused during cleanup */

//#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_PT         (0x00030400) /* Invalid Page Type */
//#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_DNM        (0x00030500) /* Device Not Mapped */
//#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_PERSIST    (0x00030600) /* Persistent Page not found */
//#define IR_LOGINFO_VOLUME_CREATE_DUPLICATE                     (0x00010002) /* Creation of duplicate volume attempted (Bus/Target ID checked) */
//#define IR_LOGINFO_VOLUME_CREATE_INVALID_VOLUME_TYPE           (0x00010005) /* Creation failed due to invalid volume type passed down */
//#define IR_LOGINFO_VOLUME_INTERNAL_CONFIG_STRUCTURE_ERROR      (0x00010007) /* Creation failed when trying to create internal structures */
//#define IR_LOGINFO_VOLUME_ACTIVATING_VOLUME_ID_IN_USE          (0x00010013) /* Activation failed due to Volume ID in use already */
//#define IR_LOGINFO_PHYSDISK_CREATE_INVALID_LENGTH              (0x00010021) /* Amount of information passed down for Create Pnysdisk is too large */
//#define IR_LOGINFO_PHYSDISK_CREATE_DMA_ERROR                   (0x00010022) /* Creation failed due to DMA error in trying to read from host */
//#define IR_LOGINFO_PHYSDISK_CREATE_BUS_TID_INVALID             (0x00010023) /* Creation failed due to invalid Bus TargetID passed down */
//#define IR_LOGINFO_PHYSDISK_CREATE_DUAL_PORT_ERROR             (0x00010025) /* Creation failed due to error: both ports of a Dual Ported device selected */
//#define IR_LOGINFO_COMPAT_ERROR_UNSUPPORTED_VOLUME_TYPE        (0x00010038) /* Compatibility Error : Volume Type is unsupported by firmware */
//#define IR_LOGINFO_COMPAT_ERROR_PHYS_DISK_NOT_FOUND            (0x0001003A) /* Compatibility Error : Phys disk for Create Volume not found */
//#define IR_LOGINFO_COMPAT_ERROR_NON_64K_STRIPE_SIZE            (0x0001003C) /* Compatibility Error : Disk stripe sizes must be 64KB */
//#define IR_LOGINFO_COMPAT_ERROR_IME_VOL_NOT_CURRENTLY_SUPPORTED (0x0001003D) /* Compatibility Error : IME size limited to < 2TB */

/* end Obsolete defines */
#endif

/****************************************************************************/
/* Defines for convenience                                                  */
/****************************************************************************/

/* See pliIocLogInfo.h */

#endif /* end of file */

