/***************************************************************************
 *                                                                         *
 *  Copyright 2005-2014 Avago Technologies.  All rights reserved.          *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiLogging.h
 *          Title:  SAS MPI Logging definitions
 *     Programmer:  Peter Rivera
 *  Creation Date:  August 16, 2005
 *
 *
 * Description
 * ------------
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPI_IF_IOP_IOPILOGGING_H_INCLUDED
#define IOPI_IF_IOP_IOPILOGGING_H_INCLUDED


/*
 * Definitions for the Log Qualifier fields
 */
/* 0xFF00 - 0xFFFF - Reserved for use by Logging code */

#define IOPI_QUALIFIER_RSA_LOG_ENTRY 0x8001




typedef enum _IOPI_PD_CACHE
{
    IOPI_PD_CACHE_UNCHANGED   = 0,    // don't change disk cache policy - use disk's default
    IOPI_PD_CACHE_ENABLE      = 1,    // enable disk write cache
    IOPI_PD_CACHE_DISABLE     = 2     // disable disk write cache
} IOPI_PD_CACHE;

/*
 * defines the logical drive properties structure
 */
typedef struct _IOPI_LD_PROPERTIES
{
    U8      DiskCachePolicy;
} IOPI_LD_PROPERTIES;

#define IOPI_TEMP_FLAG_SENSOR   (0x01)
#define IOPI_TEMP_FLAG_SLOT     (0x02)

/*
 * define the event class values
 *
 * Definitions:
 *  DEBUG   - Event is for debugging only.  This event is not saved in NVRAM.
 *  PROGRESS- Event is a progress posting event.  This event is not saved in NVRAM.
 *  INFO    - Informational message
 *  WARNING - Some component may be close to a failure point
 *  CRITICAL- A component has failed but the system has not lost data
 *  FATAL   - A component has failed and data loss has (or will) occur
 *  DEAD    - A catastrophic error has occurred and the controller has dies.  This event
 *            will only be seen after the controller has been restarted.
 */
typedef enum _IOPI_LOG_CLASS
{
    IOPI_LOG_CLASS_DEBUG       = -2,       /* debug information */
    IOPI_LOG_CLASS_PROGRESS    = -1,       /* rebuild/bgi/cc/fgi/reconstruct progress posting */
    IOPI_LOG_CLASS_INFO        = 0,
    IOPI_LOG_CLASS_WARNING     = 1,
    IOPI_LOG_CLASS_CRITICAL    = 2,
    IOPI_LOG_CLASS_FATAL       = 3,
    IOPI_LOG_CLASS_DEAD        = 4        /* only visible after next controller boot */
} IOPI_LOG_CLASS;

/*
 * define event locale codes
 */
typedef enum _IOPI_LOG_LOCALE
{
    IOPI_LOG_LOCALE_LD        = 0x01,     /* LD field valid */
    IOPI_LOG_LOCALE_PD        = 0x02,     /* PD field valid */
    IOPI_LOG_LOCALE_ENCL      = 0x04,     /* ENCL Index valid */
    IOPI_LOG_LOCALE_BBU       = 0x08,     /* BBU events */
    IOPI_LOG_LOCALE_SAS       = 0x10,     /* SAS events codes */
    IOPI_LOG_LOCALE_CTRL      = 0x20,     /* BOOT/SHUTDOWN events, etc */
    IOPI_LOG_LOCALE_CONFIG    = 0x40,     /* configuration changes */
    IOPI_LOG_LOCALE_CLUSTER   = 0x80     /* cluster events */
} IOPI_LOG_LOCALE;

/*
 * Define the class/locale structure.
 */
typedef struct _IOPI_LOG_CLASS_LOCALE_ARG_TYPE
{
    U16     Locale;             /* locale of the log (IOPI_LOG_LOCALE bit map)  */
    U8      ArgType;            /* argument of log (IOPI_LOG_ARGS)              */
    S8      LogClass;           /* class of log (IOPI_LOG_CLASS)                */
} IOPI_LOG_CLASS_LOCALE_ARG_TYPE;

/*
 * define the argument types
 */
typedef enum _IOPI_LOG_ARGS
{
    IOPI_LOG_ARGS_NONE            = 0x00,
    IOPI_LOG_ARGS_CDB_SENSE       = 0x01,
    IOPI_LOG_ARGS_LD              = 0x02,
    IOPI_LOG_ARGS_LD_LBA_PD_LBA   = 0x06,
    IOPI_LOG_ARGS_LD_STATE        = 0x08,
    IOPI_LOG_ARGS_PD              = 0x0A,
    IOPI_LOG_ARGS_PD_ERR          = 0x0B,
    IOPI_LOG_ARGS_PD_LBA          = 0x0C,
    IOPI_LOG_ARGS_PD_LBA_LD       = 0x0D,
    IOPI_LOG_ARGS_PD_STATE        = 0x0F,
    IOPI_LOG_ARGS_PCI             = 0x10,
    IOPI_LOG_ARGS_STR             = 0x12,
    IOPI_LOG_ARGS_LD_PROP         = 0x15,
    IOPI_LOG_ARGS_PD_SPARE        = 0x16,
    IOPI_LOG_ARGS_PD_LBA_LBA      = 0x18,
    IOPI_LOG_ARGS_LD_LBA          = 0x19,
    IOPI_LOG_ARGS_LD_LBA_SECTORS  = 0x1A,
    IOPI_LOG_ARGS_PI_LOG          = 0x1B,
    IOPI_LOG_ARGS_POST_DATA       = 0x1C,
    IOPI_LOG_ARGS_SLOT_DRIVELIFE  = 0x20,
    IOPI_LOG_ARGS_SLOT_THROTTLE   = 0x21,
    IOPI_LOG_ARGS_TEMP_SENSOR     = 0x22,
    IOPI_LOG_ARGS_THROTTLE        = 0x23,
    IOPI_LOG_ARGS_SLOT_TEMP       = 0x24,
    IOPI_LOG_ARGS_LOCKED_DEVICE   = 0x25,
    IOPI_LOG_ARGS_REC_VOL         = 0x26
} IOPI_LOG_ARGS;

/*
 * define assorted argument structures
 */
typedef struct _IOPI_LOG_ARG_LD {
    U16      DevHandle;           /* logical drive device handle */
    U16      Reserved;
} IOPI_LOG_ARG_LD;

typedef struct _IOPI_LOG_ARG_PD {
    U16     DevHandle;          /* Physical drive device handle */
    U8      EnclIndex;          /* From SAS Device Page 0 */
    U8      SlotNumber;
} IOPI_LOG_ARG_PD;

/*
 * Maximum number of memory post logs
 * MPI Specifies MPI2_EVENT_DATA_LOG_DATA_LENGTH bytes for the structure and there are two U16's defines
 * Plus the detail structure has an event code and CLAT
 * Divide the rest by the size of each entry (two U32's)
 */
#define IOPI_LOGGING_MEM_POST_MAX   ((MPI2_EVENT_DATA_LOG_DATA_LENGTH-sizeof(U32)*2-sizeof(IOPI_LOG_CLASS_LOCALE_ARG_TYPE))/(sizeof(U32)*2))

/*
 * this structure describes the format of an event returned to the application
 */
typedef struct _IOPI_LOG_DETAIL {

    U32                             EventCode; /* event code */
    IOPI_LOG_CLASS_LOCALE_ARG_TYPE   Clat;      /* class : argument type : locale */

    /*
     * Define the arguments unions.  Each log event has a specific 'argument' type
     * which is used to reference this union.
     */
    union {

        IOPI_LOG_ARG_LD      LdArg;

        IOPI_LOG_ARG_PD      PdArg;

        char                ArgString[20];

        struct
        {
            IOPI_LOG_ARG_PD  Pd;
            U8              Cdb_0;      /* CDB[0]            */
            U8              SenseKey;
            U8              Asc;
            U8              Ascq;
        } CdbSense;                     /* IOPI_LOG_CDB_SENSE */


        struct
        {
            U64               Lba;
            IOPI_LOG_ARG_LD   Ld;
        } LdLba;                       /* IOPI_LOG_ARGS_LD_LBA */

        struct
        {
            U64              LdLba;
            U64              PdLba;
            U16              DevHandle; /* Volume device handle */
            U8               EnclIndex;
            U8               SlotNumber;
        } LdLbaPdLba;                   /* IOPI_LOG_ARGS_LD_LBA_PD_LBA */

        struct
        {
            IOPI_LOG_ARG_LD    Ld;
            IOPI_LD_PROPERTIES PrevProp;
            IOPI_LD_PROPERTIES NewProp;
        } LdProp;                      /* IOPI_LOG_ARGS_LD_PROP */

        struct
        {
            IOPI_LOG_ARG_LD  Ld;
            U32              PrevState;  /* IOPI_LOG_LD_STATE  */
            U32              NewState;   /* IOPI_LOG_LD_STATE  */
        } LdState;                      /* IOPI_LOG_ARGS_LD_STATE */

        struct
        {
            IOPI_LOG_ARG_PD  Pd;
            U32             Err;
        } PdErr;                        /* IOPI_LOG_ARGS_PD_ERR */

        struct
        {
            U64             Lba;
            IOPI_LOG_ARG_PD  Pd;
        } PdLba;                        /* IOPI_LOG_ARGS_PD_LBA */

        struct
        {
            U64             Lba;
            IOPI_LOG_ARG_PD  Pd;
            IOPI_LOG_ARG_LD  Ld;
        } PdLbaLd;                      /* IOPI_LOG_ARGS_PD_LBA_LD */

        struct
        {
            IOPI_LOG_ARG_PD  Pd;
            U32             PrevState;  /* IOPI_LOG_PD_STATE */
            U32             NewState;   /* IOPI_LOG_PD_STATE */
        } PdState;                      /* IOPI_LOG_ARGS_PD_STATE */


        struct
        {
            U16     VendorId;
            U16     DeviceId;
            U16     SubVendorId;
            U16     SubDeviceId;
        } Pci;                          /* IOPI_LOG_ARGS_PCI */

        struct
        {
            IOPI_LOG_ARG_PD   Pd;
        } PdSpare;                      /* IOPI_LOG_ARGS_PD_SPARE */


        struct
        {
            U64             PdLba1;     /* 'bad' LBA */
            U64             PdLba2;     /* 'remapped' LBA */
            IOPI_LOG_ARG_PD  Pd;         /* IOPI_LOG_ARGS_PD_LBA_LBA */
        } PdLbaLba;

        struct
        {
            U64             LdLba;
            U64             Sectors;
            IOPI_LOG_ARG_LD  Ld;
        } LdLbaSectors;

        struct
        {
            U16             Slot;
            U16             DriveLife;
            U16             WarningLevel;
            U16             ErrorLevel;
        } SlotDriveLife;

        struct
        {
            U16             Slot;
            U16             Throttle;
        } SlotThrottle;

        struct
        {
            U8              SensorNum;
            U16             ThresholdsExceeded;
            U16             Temperature;
        } WHTemperature;

        struct
        {
            U16             PercentThrottle;
            U16             PCISlotAvailPower;
        } Throttle;

        struct
        {
            U8              Flags;
            U8              SensorOrSlot;   /* SensorNumber or Slot Number, depending on flags value*/
            U8              CurrTemp;       /* Celcius */
            U8              ThresholdTemp;  /* Celcius */
        } Temperature;


        struct
        {
            IOPI_LOG_ARG_PD Pd;
            U64             Wwid;           /* World Wide Identifier */
            IOPI_LOG_ARG_LD Ld;
        } Pd_WwID_Ld;                       /* IOPI_LOG_CODE_SSD_SECURITY_LOCKED */

        struct
        {
            U64             Wwid;           /* World Wide Identifier */
            IOPI_LOG_ARG_LD Ld;
        } WwID_Ld;


        struct
        {
            U64            Lba;            /* Start LBA of the failing IO  */
            U32            Sectors;        /* Length of the IO              */
            U16            DevHandle;      /* Volume DevHandle              */
            U8             EnclosureIndex; /* Enclosure Index of the device */
            U8             SlotNumber;     /* Slot number of the device    */
            U8             SenseKey;
            U8             SenseCode;
            U8             SenseQualifier;
        } PiLog;

        struct
        {
            U16             NumMemTypes;    /* Number of memory types */
            U16             Reserved2;
            /* Memory sector vectors (in pairs) */
            U32             SectorVectors[IOPI_LOGGING_MEM_POST_MAX*2];
        } MemPostLog;

        U8          Bytes[20];

    } Args;

} IOPI_LOG_DETAIL;  /* size = 28 bytes */


/*
 * define event codes
 */
typedef enum _IOPI_LOG_CODE {
    IOPI_LOG_CODE_BOOT                                         = 0x0001,
    IOPI_LOG_CODE_CTRL_FLASH_GENERAL_ERROR                     = 0x0002,
    IOPI_LOG_CODE_CTRL_FLASH_IMAGE                             = 0x0003,
    IOPI_LOG_CODE_CTRL_FLASH_OK                                = 0x0004,
    IOPI_LOG_CODE_LD_BGI_CORRECTED_MEDIUM_ERROR                = 0x0005,
    OBSOLETE1                                                  = 0x0006,    /* IOPI_LOG_CODE_LD_BGI_DONE_ERRORS */
    IOPI_LOG_CODE_LD_BGI_DOUBLE_MEDIUM_ERRORS                  = 0x0007,
    IOPI_LOG_CODE_LD_INIT_FAILED                               = 0x0008,
    IOPI_LOG_CODE_LD_INIT_START_FAST                           = 0x0009,
    IOPI_LOG_CODE_LD_INIT_SUCCESSFUL                           = 0x000A,
    IOPI_LOG_CODE_LD_PROP_CHANGE                               = 0x000B,
    IOPI_LOG_CODE_LD_STATE_CHANGE                              = 0x000C,
    IOPI_LOG_CODE_PD_NOT_SUPPORTED                             = 0x000D,
    IOPI_LOG_CODE_PD_PREDICTIVE_THRESHOLD_EXCEEDED             = 0x000E,
    IOPI_LOG_CODE_PD_RBLD_DONE_PD                              = 0x000F,
    OBSOLETE2                                                  = 0x0010,    /* IOPI_LOG_CODE_PD_RBLD_FAILED_BAD_SOURCE */
    IOPI_LOG_CODE_PD_RBLD_FAILED_BAD_TARGET                    = 0x0011,
    OBSOLETE3                                                  = 0x0012,    /* IOPI_LOG_CODE_PD_RBLD_RESUME */
    IOPI_LOG_CODE_PD_RBLD_START                                = 0x0013,
    OBSOLETE4                                                  = 0x0014,    /* IOPI_LOG_CODE_PD_REASSIGN_WRITE_FAILED */
    IOPI_LOG_CODE_PD_STATE_CHANGE                              = 0x0015,
    IOPI_LOG_CODE_PD_SPARE_GLOBAL_CREATED                      = 0x0016,
    IOPI_LOG_CODE_PD_SPARE_GLOBAL_DISABLED                     = 0x0017,
    IOPI_LOG_CODE_PD_SPARE_GLOBAL_NOT_COVERING_ALL_ARRAYS      = 0x0018,
    IOPI_LOG_CODE_LD_CREATED                                   = 0x0019,
    IOPI_LOG_CODE_LD_DELETED                                   = 0x001A,
    IOPI_LOG_CODE_ENCL_SAS_SATA_MIXING_DETECTED                = 0x001B,
    IOPI_LOG_CODE_PD_BBM_LOG_80_PERCENT_FULL                   = 0x001C,
    IOPI_LOG_CODE_NVRAM_MISMATCH                               = 0x001E,
    OBSOLETE5                                                  = 0x001D,    /* IOPI_LOG_CODE_NVRAM_CORRUPT */
    IOPI_LOG_CODE_SATA_BAD_BLOCK_REMAPPED                      = 0x001F,
    IOPI_LOG_CODE_PD_TOO_SMALL_FOR_AUTO_REBUILD                = 0x0020,
    IOPI_LOG_CODE_PD_NO_REBUILD_SAS_SATA_MIX_NOT_ALLOWED_IN_LD = 0x0021,
    IOPI_LOG_CODE_PD_SCSI_ERROR                                = 0x0022,
    IOPI_LOG_CODE_PD_BBM_LOG_FULL                              = 0x0023,
    IOPI_LOG_CODE_PD_INSERTED                                  = 0x0024,
    IOPI_LOG_CODE_PD_REMOVED                                   = 0x0025,
    IOPI_LOG_CODE_PD_TRANSMISSION_ERROR                        = 0x0026,
    IOPI_LOG_CODE_PD_REBUILD_MEDIUM_ERROR                      = 0x0027,
    OBSOLETE6                                                  = 0x0028,    /* IOPI_LOG_CODE_PD_NO_PD_REPLCMNT_FOR_FAILED_VOL */
    IOPI_LOG_CODE_FOREIGN_CFG_IMPORTED                         = 0x0029,
    IOPI_LOG_CODE_FOREIGN_CFG_CLEARED                          = 0x002A,
    IOPI_LOG_CODE_DUAL_PORT_REMOVED                            = 0x002B,
    IOPI_LOG_CODE_DUAL_PORT_ADDED                              = 0x002C,
    IOPI_LOG_CODE_PD_SPARE_GLOBAL_PASSIVE_PATH                 = 0x002D,
    IOPI_LOG_CODE_PD_NO_REBUILD_SSD_NONSSD_MIX_NOT_ALLOWED_IN_LD = 0x002E,
    IOPI_LOG_CODE_LD_BGI_STARTED                               = 0x002F,
    IOPI_LOG_CODE_LD_BGI_COMPLETED                             = 0x0030,
    IOPI_LOG_CODE_LD_MDC_STARTED                               = 0x0031,
    IOPI_LOG_CODE_LD_MDC_COMPLETED                             = 0x0032,
    IOPI_LOG_CODE_LD_MDC_MISCOMPARE                            = 0x0033,
    IOPI_LOG_CODE_LD_IRCC_STARTED                              = 0x0034,
    IOPI_LOG_CODE_LD_IRCC_COMPLETED                            = 0x0035,
    IOPI_LOG_CODE_LD_IRCC_MISCOMPARE                           = 0x0036,
    IOPI_LOG_CODE_LD_IRCC_ABORTED                              = 0x0037,
    IOPI_LOG_CODE_LD_IRCC_ABORTED_BBTFULL                      = 0x0038,
    IOPI_LOG_CODE_LD_CC_FAILED                                 = 0x0039,
    IOPI_LOG_CODE_NO_NVRAM                                     = 0x003A,
    IOPI_LOG_CODE_PD_NOT_CERTIFIED                             = 0x003B,
    IOPI_LOG_CODE_FOREIGN_CFG_TABLE_OVERFLOW                   = 0x003C,
    IOPI_LOG_CODE_LD_OCE_STARTED                               = 0x003D,
    IOPI_LOG_CODE_LD_OCE_COMPLETED                             = 0x003E,
    IOPI_LOG_CODE_WRITE_JOURNAL_ENTRIES                        = 0x003F,
    IOPI_LOG_CODE_FAST_RESYNC_JOURNAL                          = 0x0040,
    #if (COMPILE_OPTION_WARHAWK)
    IOPI_LOG_CODE_SSD_THROTTLING                               = 0x0041,
    IOPI_LOG_CODE_SSD_DRIVE_LIFE                               = 0x0042,
    IOPI_LOG_CODE_SSD_DRIVE_LIFE_WARNING                       = 0x0043,
    IOPI_LOG_CODE_SSD_DRIVE_LIFE_FATAL                         = 0x0044,
    IOPI_LOG_CODE_SSD_DRIVE_SMART_POLL_ERROR                   = 0x0045,
    IOPI_LOG_CODE_SC_VOLTAGE_GOOD_DEPRECATED                   = 0x0046,  // The remaining log codes are saved
    IOPI_LOG_CODE_SC_VOLTAGE_WARNING_DEPRECATED                = 0x0047,  // for backwards compatibility
    IOPI_LOG_CODE_SC_VOLTAGE_ERROR_DEPRECATED                  = 0x0048,  // with phase 6.5 GCA
    IOPI_LOG_CODE_SC_HEALTH_GOOD_DEPRECATED                    = 0x0049,
    IOPI_LOG_CODE_SC_HEALTH_WARNING_DEPRECATED                 = 0x004A,
    IOPI_LOG_CODE_SC_HEALTH_ERROR_DEPRECATED                   = 0x004B,
    IOPI_LOG_CODE_SC_PSOC_FAILURE_DEPRECATED                   = 0x004C,
    IOPI_LOG_CODE_BACKUP_RAIL_MONITOR_FAILURE                  = 0x004D,
    IOPI_LOG_CODE_TEMP_THRESHOLD_EXCEEDED                      = 0x004E,
    IOPI_LOG_CODE_POWER_THROTTLING                             = 0x004F,
    IOPI_LOG_CODE_TEMP_WARNING                                 = 0x0050,
    IOPI_LOG_CODE_TEMP_CRITICAL                                = 0x0051,
    IOPI_LOG_CODE_SSD_SECURITY_LOCKED                          = 0x0052,
    IOPI_LOG_CODE_SSD_SECURITY_UNLOCKED                        = 0x0053,
    IOPI_LOG_CODE_CTRL_PFK_SOD_SASADDR_MISMATCH                = 0x0054,
    IOPI_LOG_CODE_CTRL_PFK_SOD_ROMID_MISMATCH                  = 0x0055,
    IOPI_LOG_CODE_CTRL_PFK_LICENSE_APPLIED                     = 0x0056,
    IOPI_LOG_CODE_CTRL_PFK_INVALID_LICENSE_KEY                 = 0x0057,
    IOPI_LOG_CODE_RECOVER_VOLUME_INITIATED                     = 0x0058,
    IOPI_LOG_CODE_RECOVER_VOLUME_SUCCESS                       = 0x0059,
    IOPI_LOG_CODE_RECOVER_VOLUME_FAIL                          = 0x005A,
#endif /* COMPILE_OPTION_WARHAWK */

    /*
   * To avoid overlap of these definitions going forward the following ranges have
   * been preallocated to their respective projects:
   *
   * IR/IT    0x005B - 0x00FF
   * WD      0x0100 - 0x01FF
   * SBA      0x0200 - 0x02FF
   *
   * If these ranges are going to be exceeded a new preallocation will need to be
   * agreed upon.  There can be no overlap since SLIR-2 communicates with all
   * of these projects.
   */
    IOPI_LOG_CODE_PI_ERROR                                     = 0x005B,

    IOPI_LOG_CODE_PD_DFU_MODE_ENABLED                          = 0x005D,
    IOPI_LOG_CODE_PD_DFU_MODE_DISABLED                         = 0x005E,
    IOPI_LOG_CODE_MEMORY_POST_ERROR                            = 0x005F,
    IOPI_LOG_CODE_LAST_ENTRY   // THIS MUST BE LAST

} IOPI_LOG_CODE;

#endif
