/***************************************************************************
 *                                                                         *
 *  Copyright 2003-2015 Avago Technologies.  All rights reserved.          *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pliIocLogInfo.h
 *          Title:  SAS Firmware IOP Interface IOC Log Info Definitions
 *     Programmer:
 *  Creation Date:  November 30, 2007
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface IOC Log Info codes
 *
 *-------------------------------------------------------------------------
 */

#ifndef PLI_IOCLOGINFO_H_INCLUDED
#define PLI_IOCLOGINFO_H_INCLUDED


/****************************************************************************/
/*  IOC LOGINFO defines, 0x00000000 - 0x0FFFFFFF                            */
/*  Format:                                                                 */
/*      Bits 31-28: MPI2_IOCLOGINFO_TYPE_SAS (3)                            */
/*      Bits 27-24: IOC_LOGINFO_ORIGINATOR: 0=IOP, 1=PL, 2=IR               */
/*      Bits 23-16: LOGINFO_CODE                                            */
/*      Bits 15-0:  LOGINFO_CODE Specific                                   */
/****************************************************************************/

/****************************************************************************/
/* IOC_LOGINFO_ORIGINATOR defines                                           */
/****************************************************************************/
#define IOC_LOGINFO_ORIGINATOR_IOP                      (0x00000000)
#define IOC_LOGINFO_ORIGINATOR_PL                       (0x01000000)
#define IOC_LOGINFO_ORIGINATOR_IR                       (0x02000000)

#define IOC_LOGINFO_ORIGINATOR_MASK                     (0x0F000000)

/****************************************************************************/
/* LOGINFO_CODE defines                                                     */
/****************************************************************************/
#define IOC_LOGINFO_CODE_MASK                           (0x00FF0000)
#define IOC_LOGINFO_CODE_SHIFT                          (16)

/****************************************************************************/
/* IOP LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IOP          */
/****************************************************************************/

/* See iopiIopLogInfo.h */

/****************************************************************************/
/* PL LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = PL            */
/****************************************************************************/
#define PL_LOGINFO_CODE_OPEN_FAILURE                                (0x00010000)    /* Open failure - See sub-codes starting with PL_LOGINFO_SUB_CODE_OPEN_FAILURE. */

#define PL_LOGINFO_CODE_SATA_NON_NCQ_RW_ERR_BIT_SET                 (0x00060000)    /* Error bit is set in SATA D->H FIS for non-NCQ, automated read/write. */

#define PL_LOGINFO_CODE_SCSI_IO_NOT_REGISTERED                      (0x00070000)    /* Unregistered SCSI I/O. */
#define PL_LOGINFO_CODE_SCSI_IO_DEVICE_BLOCKED                      (0x00070001)    /* Blocked SCSI I/O device. */

#define PL_LOGINFO_CODE_SATA_NCQ_FAIL_ALL_CMDS_AFTR_ERR             (0x00080000)    /* All NCQ I/Os fail when one NCQ I/O encounters an error. */

#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE                         (0x000F0000)    /* Invalid PL configuration page.*/
#define PL_LOGINFO_CODE_CONFIG_PL_NOT_INITIALIZED                   (0x000F0001)    /* PL is not initialized and cannot perform a configuration page request. */
#define PL_LOGINFO_CODE_CONFIG_BACKEND_PCIE_SUPPORT_DISABLED        (0x000F0002)    /* Backend PCIe support is disabled; the page request cannot complete. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_PT                      (0x000F0100)    /* Invalid page type. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NUM_PHYS                (0x000F0200)    /* Invalid number of phys. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NOT_IMP                 (0x000F0300)    /* The page was not handled. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NO_DEV                  (0x000F0400)    /* No device found. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_FORM                    (0x000F0500)    /* Invalid form. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_PHY                     (0x000F0600)    /* Invalid phy. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_NO_OWNER                (0x000F0700)    /* No owner found. */
#define PL_LOGINFO_CODE_CONFIG_INVALID_PAGE_LINK                    (0x000F0800)    /* Invalid Link*/

#define PL_LOGINFO_CODE_RESET                                       (0x00110000)    /* Target reset. The firmware ORs this code with a subcode (lower 16 bits) for explanation. For more information, see Table 9.  */
#define PL_LOGINFO_CODE_ABORT                                       (0x00120000)    /* Task abort. The firmware ORs this code with a subcode (lower 16 bits) for explanation. For more information, see Table 9.*/
#define PL_LOGINFO_CODE_IO_NOT_YET_EXECUTED                         (0x00130000)    /* Aborted I/O from a pending queue and is not yet submitted to the hardware. */
#define PL_LOGINFO_CODE_IO_EXECUTED                                 (0x00140000)    /* Aborted I/O is submitted to the hardware. */
#define PL_LOGINFO_CODE_PERS_RESV_OUT_NOT_AFFIL_OWNER               (0x00150000)    /* Affiliation conflict on Persistent Reserve Out command. */
#define PL_LOGINFO_CODE_NOT_AFFIL_OWNER                             (0x00160000)    /* Affiliation conflict. */

#define PL_LOGINFO_CODE_IO_DEVICE_MISSING_DELAY_RETRY               (0x00170000)    /* Device is missing (device missing delay timer option is enabled). When the timer starts, an internal TM cleans up pending I/Os. The I/Os returned to the host should be retried. This log information is also sent when the I/O reply timer expires. */

#define PL_LOGINFO_CODE_TARGET_INVALID_MID                          (0x00180001)    /* An invalid MID was requested in a Target mode abort. */
#define PL_LOGINFO_CODE_TARGET_INVALID_TYPE                         (0x00180002)    /* An invalid abort type during a Target mode abort. */
#define PL_LOGINFO_CODE_TARGET_MODE_ABORT_ALL_IO                    (0x00180003)    /* Target mode abort request with an ABORT_ALL_IO abort type.*/
#define PL_LOGINFO_CODE_TARGET_MODE_ABORT_EXACT_IO                  (0x00180004)    /* Target mode abort request with an ABORT_EXACT_IO abort type. */
#define PL_LOGINFO_CODE_TARGET_MODE_ABORT_EXACT_IO_REQ              (0x00180005)    /* Target mode abort request with an ABORT_EXACT_IO_REQUEST abort type. */
#define PL_LOGINFO_CODE_TARGET_ASSIST_TERMINATED                    (0x00180007)    /* Target assist terminated. */
#define PL_LOGINFO_CODE_TARGET_STATUS_SEND_TERMINATED               (0x00180008)    /* Target status send terminated. */

#define PL_LOGINFO_CODE_SENSE_DATA_LENGTH_ZERO                      (0x00190000)    /* SSP sense data length is 0. */

/* Codes 0x001A0000 to 0x001A0014 are reserved for BT1680 in Gen 2 */
//#define PL_LOGINFO_CODE_BT1680                                      (0x001A0000)    /* GEN2 ONLY! BT1680 condition detected. */
//#define PL_LOGINFO_CODE_BT1680_BREAK_RECEIVED                       (0x001A0001)    /* GEN2 ONLY! BT1680 condition detected. */
//#define PL_LOGINFO_CODE_BT1680_CRC_MISMATCH                         (0x001A0002)    /* GEN2 ONLY! BT1680 reread of the sector resulted in a CRC mismatch. */
//#define PL_LOGINFO_CODE_BT1680_NO_RECON_OPEN                        (0x001A0003)    /* GEN2 ONLY! BT1680 target did not reconnect during BT1680 recovery. */
//#define PL_LOGINFO_CODE_BT1680_NO_SUSPEND                           (0x001A0004)    /* GEN2 ONLY! BT1680 RxEEDP failed to suspend. */
//#define PL_LOGINFO_CODE_BT1680_RESEND_IO_FAILED                     (0x001A0005)    /* GEN2 ONLY! BT1680 failed to reissue the original I/O. */
//#define PL_LOGINFO_CODE_BT1680_RESEND_IO_CONN_STUCK                 (0x001A0006)    /* GEN2 ONLY! BT1680 connection did not close after the I/O reissue. */
//#define PL_LOGINFO_CODE_BT1680_INQ_FAILED                           (0x001A0007)    /* GEN2 ONLY! BT1680 failed to detect the device type. */
//#define PL_LOGINFO_CODE_BT1680_NO_BREAKUP_FRAME                     (0x001A0008)    /* GEN2 ONLY! BT1680 failed to acquire frame for SATA breakup. */
//#define PL_LOGINFO_CODE_BT1680_SATA_SHORT_FRAME                     (0x001A0009)    /* GEN2 ONLY! BT1680 failed because of a short Data frame. */
//#define PL_LOGINFO_CODE_BT1680_SATA_UNDERRUN                        (0x001A000A)    /* GEN2 ONLY! BT1680 failed because of an underrun. */
//#define PL_LOGINFO_CODE_BT1680_SATA_OVERRUN                         (0x001A000B)    /* GEN2 ONLY! BT1680 failed because of an overrun. */
//#define PL_LOGINFO_CODE_BT1680_WRONG_MID_OR_DEV                     (0x001A000C)    /* GEN2 ONLY! BT1680 failed because of a frame from the incorrect device or for the incorrect MID. */
//#define PL_LOGINFO_CODE_BT1680_RETRIES_EXHAUSTED                    (0x001A000D)    /* GEN2 ONLY! BT1680 failed because of excessive retries on a SCSI error. */
//#define PL_LOGINFO_CODE_BT1680_DEVICE_MISSING                       (0x001A000E)    /* GEN2 ONLY! BT1680 failed because of a missing device. */
//#define PL_LOGINFO_CODE_BT1680_CANT_STOP_RXDMA                      (0x001A000F)    /* GEN2 ONLY! BT1680 failed because the RxDMA did not stop quickly enough. */
//#define PL_LOGINFO_CODE_BT1680_RESET_ON_FAILURE                     (0x001A0010)    /* GEN2 ONLY! BT1680 failed, reset on exit. */
//#define PL_LOGINFO_CODE_BT1680_SATA_INIT_FAILURE                    (0x001A0011)    /* GEN2 ONLY! BT1680 failed to run SATA initialization. */
//#define PL_LOGINFO_CODE_BT1680_QUIESCE_FAILED                       (0x001A0012)    /* GEN2 ONLY! BT1680 failed because it could not quiesce in time. */
//#define PL_LOGINFO_CODE_BT1680_SAS_UNDERRUN                         (0x001A0013)    /* GEN2 ONLY! BT1680 failed because of an underrun. */
//#define PL_LOGINFO_CODE_BT1680_SAS_OVERRUN                          (0x001A0014)    /* GEN2 ONLY! BT1680 failed because of an overrun. */
//#define PL_LOGINFO_CODE_BT1680_NO_RECON_DATA                        (0x001A0015)    /* GEN2 ONLY! BT1680 target did not reconnect with data during the BT1680 recovery. */

#define PL_LOGINFO_CODE_TM_INVALID_REQUEST                          (0x001B0000)    /* Invalid TM request; see PL_LOGINFO_SUBCODE_INVALID_TM_NOT_END_DEVICE in Table 9 for more information. */

/* SMP failure codes */
#define PL_LOGINFO_CODE_SMP_FAILED                                  (0x001C0000)    /* The SMP failed. Lower 2 bytes are a PLSTATUS sub-code. */
#define PL_LOGINFO_CODE_SMP_FAILED_ABORT                            (0x001C000A)    /* The SMP failed. Abort. */
#define PL_LOGINFO_CODE_SMP_FAILED_XFER_ERROR                       (0x001C000D)    /* The SMP failed. The Tx Context Frame Manager detected a transfer error. */
#define PL_LOGINFO_CODE_SMP_FAILED_TIMEOUT                          (0x001C000E)    /* The SMP failed. A time-out occurred. */
#define PL_LOGINFO_CODE_SMP_FAILED_DMA_ERROR                        (0x001C0014)    /* The SMP failed. The Tx DMA engine detected an error. */
#define PL_LOGINFO_CODE_SMP_FAILED_HW_TIMEOUT                       (0x001C0020)    /* The SMP failed. The link timed out. */
#define PL_LOGINFO_CODE_SMP_FAILED_LINK_ERROR                       (0x001C0021)    /* The SMP failed. An error occurred on the link. */
#define PL_LOGINFO_CODE_SMP_FAILED_BACK_OFF                         (0x001C0030)    /* The SMP failed. An interrupt was raised to back-off the transceiver. NOTE: Description changed. */
#define PL_LOGINFO_CODE_SMP_FAILED_NO_CONNECTION                    (0x001C0031)    /* The SMP failed. No connection available for transceiver. */
#define PL_LOGINFO_CODE_SMP_FAILED_OPEN_FAIL                        (0x001C0100)    /* The SMP failed. Failed to open a connection. The lowest byte is the specific failure code. */

#define PL_LOGINFO_CODE_POWER_MANAGEMENT                            (0x00340000)    /* Indicates a power management error. See Table 10 for PL_LOGINFO_SUB_CODE_PWRMGMT sub-codes. */
#define PL_LOGINFO_CODE_POWER_MANAGEMENT_UNKNOWN_STATUS             (0x00348000)    /* Indicates an unknown power management status code in the lower 4 bits. */

#define PL_LOGINFO_CODE_SMP_REQUEST_TOO_LONG_4_IMMEDIATE            (0x00400000)    /* Immediate mode cannot be used on an SMP request because its length is larger than a message frame. */

/* Note: the intent here for the OPEN_FAILURES is to use PL_LOGINFO_SUB_CODE_OPEN_FAILURE ORed with the OpenFailCode from the port layer */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE                            (0x00000100)    /* Open failure. Lower two nibbles provide explanation. For more information, see code 0x00000101 to code 0x0000011C that follow. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_NO_DEST_TIMEOUT            (0x00000101)    /* Open failure. OPEN_REJECT (no destination) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_PATHWAY_BLOCKED            (0x00000102)    /* Open failure. OPEN REJECT (pathway blocked) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_CONT0                 (0x00000103)    /* Open failure. OPEN REJECT (reserved continue 0) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_CONT1                 (0x00000104)    /* Open failure. OPEN REJECT (reserved continue 1) received.*/
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_INIT0                 (0x00000105)    /* Open failure. OPEN REJECT (reserved initialize 0) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_INIT1                 (0x00000106)    /* Open failure. OPEN REJECT (reserved initialize 1) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_STOP0                 (0x00000107)    /* Open failure. OPEN REJECT (reserved stop 0) received.  */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RSVD_STOP1                 (0x00000108)    /* Open failure. OPEN REJECT (reserved stop 1) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ORR                        (0x00000109)    /* Open failure. OPEN REJECT (retry) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_BREAK                      (0x0000010A)    /* Open failure. BREAK received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_TIMEOUT                    (0x0000010C)    /* Open failure. Open time-out. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_CANCELCON                  (0x0000010D)    /* Open failure. PortCancelCon received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_DEV_DOWN                   (0x0000010E)    /* Open failure. Device table down.  */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_BAD_DEST                   (0x00000111)    /* Open failure. OPEN REJECT (bad destination) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_RATE_NOT_SUPPORTED         (0x00000112)    /* Open failure. OPEN REJECT (connection rate not supported) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_PROTOCOL_NOT_SUPPORTED     (0x00000113)    /* Open failure. OPEN REJECT (protocol not supported) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ZONE_VIOLATION             (0x00000114)    /* Open failure. OPEN REJECT (zone violation) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ABANDON1                   (0x00000115)    /* Open failure. OPEN REJECT (reserved abandon 1) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ABANDON2                   (0x00000116)    /* Open failure. OPEN REJECT (reserved abandon 2) received.*/
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ABANDON3                   (0x00000117)    /* Open failure. OPEN REJECT (reserved abandon 3) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_STP_RESOURCES_BSY          (0x00000118)    /* Open failure. OPEN REJECT (STP resources busy) received. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_WRONG_DEST                 (0x00000119)    /* Open failure. OPEN REJECT (wrong destination) received.*/
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_ORR_TIMEOUT                (0x0000011A)    /* Open failure. OPEN REJECT (retry) time-out.  */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_AWT_MAXED                  (0x0000011C)    /* Open failure. Arbitration wait timer expired. */
#define PL_LOGINFO_SUB_CODE_OPEN_FAILURE_REQ_PORT_DOWN              (0x000001FF)    /* Open failure. Request sent to a port where all the phys are down. */

#define PL_LOGINFO_SUB_CODE_TARGET_BUS_RESET                        (0x00000120)    /* Received HARD_RESET primitive in Target mode. */

#define PL_LOGINFO_SUB_CODE_UART_TEST                               (0x00000140)    /* I/O aborted by a UART command. */

#define PL_LOGINFO_SUB_CODE_TX_TRANSPORT_LAYER                      (0x00000300)  /* The lower nibble (1-f) is reserved for specific errors. For more information, see code 0x00000301 to code 0x00000322 that follow. */
#define PL_LOGINFO_SUB_CODE_TX_CONNECTED                            (0x00000301)  /* Subcode for TxTransportLayer. TxCtx connected. */
#define PL_LOGINFO_SUB_CODE_TX_CONNECTED_ACK                        (0x00000302)  /* Subcode for TxTransportLayer. TxCtx connected ACK. */
#define PL_LOGINFO_SUB_CODE_TX_NAK                                  (0x00000303)  /* Subcode for TxTransportLayer. TxCtx NAK.   */
#define PL_LOGINFO_SUB_CODE_TX_SMP_BACKOFF                          (0x00000304)  /* Subcode for TxTransportLayer. TxCtx SMP back off. */
#define PL_LOGINFO_SUB_CODE_TX_WRG_REL_OFF                          (0x00000305)  /* Subcode for TxTransportLayer. TxCtx WrgRelOff. */
#define PL_LOGINFO_SUB_CODE_TX_BACK_OFF                             (0x00000306)  /* Subcode for TxTransportLayer. TxCtx back off.  */
#define PL_LOGINFO_SUB_CODE_TX_ABORT_FM                             (0x00000307)  /* Subcode for TxTransportLayer. TxCtx abort FM. */
#define PL_LOGINFO_SUB_CODE_TX_T_LEN_GT_DL                          (0x00000308)  /* Subcode for TxTransportLayer. TxCtx T LEN is greater than the data length. */
#define PL_LOGINFO_SUB_CODE_TX_WDL_GT_DL                            (0x00000309)  /* Subcode for TxTransportLayer. TxCtx WDL is greater than the data length. */
#define PL_LOGINFO_SUB_CODE_TX_EEDP                                 (0x0000030A)  /* Subcode for TxTransportLayer. TxCtx EEDP error, the IOC status indicates which type. */
#define PL_LOGINFO_SUB_CODE_TX_DISCONNECTION                        (0x00000310)  /* Subcode for TxTransportLayer. TxFM disconnected. */
#define PL_LOGINFO_SUB_CODE_TX_NAK_RCVD                             (0x00000311)  /* Subcode for TxTransportLayer. TxFM NAK received. */
#define PL_LOGINFO_SUB_CODE_TX_FM_ABORT_FM                          (0x00000312)  /* Subcode for TxTransportLayer. TxFM abort FM. */
#define PL_LOGINFO_SUB_CODE_TX_BAD_SGE                              (0x00000320)  /* Subcode for TxTransportLayer. TxDMA bad SGE fetch. */
#define PL_LOGINFO_SUB_CODE_TX_FETCH_ERROR                          (0x00000321)  /* Subcode for TxTransportLayer. TxDMA fetch error. */
#define PL_LOGINFO_SUB_CODE_TX_SYS_DMA                              (0x00000322)  /* Subcode for TxTransportLayer. TxDMA SYS DMA error. */
#define PL_LOGINFO_SUB_CODE_TX_SNC_COLLATERAL_IO                    (0x00000323)  /* Subcode for TxTransportLayer. SnC method collateral failed I/O. */

#define PL_LOGINFO_SUB_CODE_RX_TRANSPORT_LAYER                      (0x00000430)  /* The lower nibble (1-f) is reserved for specific errors. For more information, see code 0x00000431 to code 0x0000043F that follow. */
#define PL_LOGINFO_SUB_CODE_RX_WRONG_REL_OFFSET                     (0x00000431)  /* Subcode for RxTransportLayer. Data offset error. */
#define PL_LOGINFO_SUB_CODE_RX_FRAME_LENGTH                         (0x00000432)  /* Subcode for RxTransportLayer. Frame length mismatch. */
#define PL_LOGINFO_SUB_CODE_RX_ZERO_WDL                             (0x00000433)  /* Subcode for RxTransportLayer. Zero WDL in Xfer Rdy. */
#define PL_LOGINFO_SUB_CODE_RX_OVERRUN                              (0x00000434)  /* Subcode for RxTransportLayer. Overrun. */
#define PL_LOGINFO_SUB_CODE_RX_CDP_FRAME                            (0x00000435)  /* Subcode for RxTransportLayer. CDP frame received. */
#define PL_LOGINFO_SUB_CODE_RX_FRAME_SCHED_ERR                      (0x00000436)  /* Subcode for RxTransportLayer. Frame schedule error. */
#define PL_LOGINFO_SUB_CODE_RX_CUR_FRAME_DENY                       (0x00000437)  /* Subcode for RxTransportLayer. Current frame deny. */
#define PL_LOGINFO_SUB_CODE_RX_CTX_WRONG_FRAME_TYPE                 (0x00000438)  /* Subcode for RxTransportLayer. Wrong frame type. */
#define PL_LOGINFO_SUB_CODE_RETRANSMIT_XFER_RDY                     (0x00000439)  /* Subcode for RxTransportLayer. Retransmit bit set in XFER_RDY. */
#define PL_LOGINFO_SUB_CODE_RX_DATA_FRM_OVERRUN                     (0x0000043A)  /* Subcode for RxTransportLayer. FPDMA overrun (also occurs for SAS). */
#define PL_LOGINFO_SUB_CODE_RX_EEDP                                 (0x0000043B)  /* Subcode for RxTransportLayer. EEDP error, the IOC status indicates which type. */
#define PL_LOGINFO_SUB_CODE_RX_EEDP_RESET                           (0x0000043C)  /* Subcode for RxTransportLayer. EEDP error, the IOC status indicates which type. */
#define PL_LOGINFO_SUB_CODE_RX_BAD_SGE                              (0x0000043D)  /* Subcode for RxTransportLayer. Bad SGE fetch. */
#define PL_LOGINFO_SUB_CODE_RX_FETCH_ERROR                          (0x0000043E)  /* Subcode for RxTransportLayer. Fetch error. */
#define PL_LOGINFO_SUB_CODE_RX_SYS_DMA                              (0x0000043F)  /* Subcode for RxTransportLayer. SYS DMA error. */
#define PL_LOGINFO_SUB_CODE_RX_WRONG_DATA_XFER                      (0x00000440)  /* Subcode for RxTransportLayer. Wrong data transfer. */
#define PL_LOGINFO_SUB_CODE_RX_FINAL_EEDP_FRAME                     (0x00000441)  /* Subcode for RxTransportLayer. Final EEDP frame. */
#define PL_LOGINFO_SUB_CODE_RX_DMA_OVERFLOW                         (0x00000442)  /* Subcode for RxTransportLayer. RxDMA Overflow. */
#define PL_LOGINFO_SUB_CODE_RX_DMA_PARITY_ERROR                     (0x00000443)  /* Subcode for RxTransportLayer. RxDMA Parity Error detected. */

#define PL_LOGINFO_SUB_CODE_INTERNAL_IO_TIMED_OUT                   (0x00000500)  /* Internal SCSI I/O timed out. */

#define PL_LOGINFO_SUB_CODE_SATA_READ_LOG_RECEIVE_DATA_ERR          (0x00000610)  /* Error receiving data on the ReadLogExt SATA command. */
#define PL_LOGINFO_SUB_CODE_SATA_READ_LOG_RECEIVE_DATA_BAD_TAG      (0x00000611)  /* Device reported an inactive tag in the ReadLogExt data after an NCQ error. */
#define PL_LOGINFO_SUB_CODE_SATA_ERR_IN_RCV_SET_DEV_BIT_FIS         (0x00000630)  /* Error in the Set Device Bits FIS for the ReadLogExt SATA command. */

/* Backend PCIe device related sub codes */
#define PL_LOGINFO_SUB_CODE_NVME_ASYNC_PERSISTENT_ERROR             (0x00000800)    /* NVMe device asynchronously indicated a persistent device error. */
#define PL_LOGINFO_SUB_CODE_NVME_INIT_TIMEOUT_OR_FAILURE            (0x00000801)    /* NVMe Init failed or timed out. */
#define PL_LOGINFO_SUB_CODE_NVME_FW_UPDATE_RESET                    (0x00000802)    /* NVMe device was reset during a firmware update command. */
#define PL_LOGINFO_SUB_CODE_NVME_IOERROR_BAD_SQID                   (0x00000803)    /* Bad submission queue ID on FxCore NVMe I/O completion. */
#define PL_LOGINFO_SUB_CODE_NVME_IOERROR_BAD_CID                    (0x00000804)    /* Bad command ID on FxCore NVMe I/O completion. */
#define PL_LOGINFO_SUB_CODE_NVME_IOERROR_SUBQ_HEAD_TOO_LARGE        (0x00000805)    /* The submission queue head pointer is larger than the submission queue size. */
#define PL_LOGINFO_SUB_CODE_NVME_IOERROR_SUBQ_HEAD_BAD_ADVANCE_1    (0x00000806)    /* The submission queue head pointer jumped ahead of the tail pointer. */
#define PL_LOGINFO_SUB_CODE_NVME_IOERROR_SUBQ_HEAD_BAD_ADVANCE_2    (0x00000807)    /* The submission queue head pointer jumped ahead of the tail pointer. */
#define PL_LOGINFO_SUB_CODE_NVME_ADMIN_COMPL_BAD_SQID               (0x00000808)    /* An admin command completion contained a bad submission queue ID. */
#define PL_LOGINFO_SUB_CODE_NVME_ADMIN_COMPL_BAD_CID                (0x00000809)    /* An admin command completion contained a bad command ID. */
#define PL_LOGINFO_SUB_CODE_NVME_ADMIN_COMPL_UNEXPECTED             (0x0000080A)    /* No callback registered for an admin command completion. */
#define PL_LOGINFO_SUB_CODE_NVME_ASYNC_EVENT_TIMEOUT                (0x0000080B)    /* A timeout occurred during NVMe async event handling. */
#define PL_LOGINFO_SUB_CODE_NVME_FORMAT_UNIT_COMPLETE_RESET         (0x0000080D)    /* NVMe device was reset following successful completion of a Format Unit command. */
#define PL_LOGINFO_SUB_CODE_NVME_GET_LOG_PAGE_FAILED                (0x0000080E)    /* Get Log Page command failed during persistent condition monitoring */
#define PL_LOGINFO_SUB_CODE_NVME_SET_FEATURES_FAILED                (0x0000080F)    /* Set Features command failed during persistent condition monitoring */
#define PL_LOGINFO_SUB_CODE_NVME_ADMIN_SUBMISSION_QUEUE_FULL1       (0x00000810)    /* Admin submission queue was full when attempting to send a Get Log Page command */
#define PL_LOGINFO_SUB_CODE_NVME_ADMIN_SUBMISSION_QUEUE_FULL2       (0x00000811)    /* Admin submission queue was full when attempting to send a Set Features command */
#define PL_LOGINFO_SUB_CODE_FXCORE_REQUESTOR_ACCESS_ERROR           (0x00000812)    /* FxCore requestor engine encountered a device access error. */

#define PL_LOGINFO_SUB_CODE_AHCI_PHY_RDY_CHANGE                     (0x00000825)    /* AHCI phy ready status changed. */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERROR_HOST_BUS_FATAL          (0x00000826)    /* AHCI - HBFS error occurred */
#define PL_LOGINFO_SUB_CODE_AHCI_INTF_NON_FATAL_ERR                 (0x00000827)    /* AHCI - INFS error occurred */
#define PL_LOGINFO_SUB_CODE_AHCI_INTF_FATAL_ERR                     (0x00000828)    /* AHCI - IFS error occurred */
#define PL_LOGINFO_SUB_CODE_AHCI_ERR_CR_CLEAR_FAILED                (0x00000829)    /* AHCI - CR did not clear during AHCI ERR bit recovery */
#define PL_LOGINFO_SUB_CODE_AHCI_ERR_BSY_CLEAR_FAILED               (0x0000082A)    /* AHCI - BSY or DRQ did not clear during AHCI ERR bit recovery */

#define PL_LOGINFO_SUB_CODE_RX_FM_CURRENT_FRAME_ERROR               (0x00000B10)  /* Rx Frame Manager current frame error. */
#define PL_LOGINFO_SUB_CODE_RX_FM_INVALID_TAG_DMA_SETUP_FIS         (0x00000B41)  /* Rx Frame Manager invalid tag in the FPDMA FIS error. */
#define PL_LOGINFO_SUB_CODE_RX_FM_NCQ_DISABLED                      (0x00000B50)  /* Received FPDMA or Set Device Bits FIS, but NCQ is disabled. */
#define PL_LOGINFO_SUB_CODE_RX_FM_INVALID_TAG_SET_DEVICE_BITS       (0x00000B42)  /* Rx Frame Manager invalid tag in the SDB FIS error. */
#define PL_LOGINFO_SUB_CODE_RX_FM_WRONG_DATA_DIRECTION              (0x00000B60)  /* Rx Frame Manager wrong data direction. */
#define PL_LOGINFO_SUB_CODE_SATA_LINK_DOWN                          (0x00000D00)  /* SATA direct-attached link went down. */
#define PL_LOGINFO_SUB_CODE_ABORT_SATA_RX_STAT                      (0x00000D01)  /* SATA receive transfer aborted. */
#define PL_LOGINFO_SUB_CODE_SATA_UNEXPECTED_FRAME                   (0x00000D02)  /* Received an unexpected SATA FIS. */
#define PL_LOGINFO_SUB_CODE_DISCOVERY_SATA_INIT_W_IOS               (0x00000E00)  /* SATA initialization started with outstanding I/Os. */
#define PL_LOGINFO_SUB_CODE_DISCOVERY_REMOTE_SEP_RESET              (0x00000E01)  /* Remote SEP requires reset. */
#define PL_LOGINFO_SUB_CODE_RCVD_UNCONFIRMED_RESP_FRAME             (0x00000E02)  /* Received unconfirmed Response frame. */
#define PL_LOGINFO_SUB_CODE_DISCOVERY_SATA_INIT_NEEDED_W_IOS        (0x00000E03)  /* SATA initialization started with outstanding I/Os. */
#define PL_LOGINFO_SUB_CODE_PCIE_DEV_INIT_NEEDED_W_IOS              (0x00000E04)  /* Backend PCIe device init started with outstanding I/Os. */

#define PL_LOGINFO_SUB_CODE_DA_BACKEND_PCIE_LINK_DOWN               (0x00000E10)  /* The backend PCIe link went down. */

#define PL_LOGINFO_SUB_CODE_INVALID_TM_NOT_END_DEVICE               (0x00000E80)  /* TM sent to a device handle that is not an end device. */

#define PL_LOGINFO_SUB_CODE_AHCI_NOTIFY_REG_READ_FAILED             (0x00000E90)  /* A register read failed during AHCI SNotification handling */
#define PL_LOGINFO_SUB_CODE_AHCI_NOTIFY_SACTIVE_NOT_CLEARED         (0x00000E91)  /* An AHCI device didn't clear SAct bits after SNotification reporting */
#define PL_LOGINFO_SUB_CODE_AHCI_WRITE_PXSERR_FAILED                (0x00000E92)  /* Failed to P0SERR while handling a port error */
#define PL_LOGINFO_SUB_CODE_AHCI_BAD_STATUS_CANT_READ_TAGS          (0x00000E93)  /* Failed to read P0SACT register during bad status handling for AHCI */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_PXIS_READ_FAIL            (0x00000E94)  /* Failed to read P0IS register during port error handling for AHCI */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_PXSERR_WRITE_FAIL         (0x00000E95)  /* Failed to write P0SERR during port error handling for AHCI */
#define PL_LOGINFO_SUB_CODE_AHCI_SSNTF_WRITE_FAIL                   (0x00000E96)  /* Failed to write P0SSNTF during IO error handling for AHCI */
#define PL_LOGINFO_SUB_CODE_AHCI_SACTIVE_CLEAR_FAILED               (0x00000E97)  /* Device failed to clear SActive flags during SNotification handling */
#define PL_LOGINFO_SUB_CODE_AHCI_SACTIVE_READ_FAILED                (0x00000E98)  /* Failed to read P0SACT while handing bad tag interrupt */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_PXIS_CLEAR_FAILED         (0x00000E99)  /* Failed to write P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_READ_PXTFD_FAILED         (0x00000E9A)  /* Failed to read P0TFD while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_READ_PXCMD_FAILED         (0x00000E9B)  /* Failed to read P0CMD while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_WRITE_PXCMD_FAILED        (0x00000E9C)  /* Failed to write P0CMD while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_WRITE_PXIS_FAILED         (0x00000E9D)  /* Failed to write P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_RESTART_FAILED            (0x00000E9E)  /* Failed to write P0CMD while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_N_FAILED            (0x00000E9F)  /* Failed to clear PhyRdy bit in P0SERR while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_X_FAILED            (0x00000EA0)  /* Failed to clear exchanged bit in P0SERR while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_DHRS_FAILED         (0x00000EA1)  /* Failed to clear DHRS in P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_PSS_FAILED          (0x00000EA2)  /* Failed to clear PSS in P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_DSS_FAILED          (0x00000EA3)  /* Failed to clear DSS in P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_SDBS_FAILED         (0x00000EA4)  /* Failed to clear SDBS in P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_UFS_FAILED          (0x00000EA5)  /* Failed to clear UFS in P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_DPS_FAILED          (0x00000EA6)  /* Failed to clear DPS in P0IS while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_PORT_ERR_CLEAR_F_FAILED            (0x00000EA7)  /* Failed to clear F in P0SERR while handling port error */
#define PL_LOGINFO_SUB_CODE_AHCI_CHANGE_NCQ_PXIE_READ_FAILED        (0x00000EA8)  /* Failed to read P0IE while changing NCQ setting */
#define PL_LOGINFO_SUB_CODE_AHCI_CHANGE_NCQ_PXIE_WRITE_FAILED       (0x00000EA9)  /* Failed to write P0IE while changing NCQ setting */
#define PL_LOGINFO_SUB_CODE_AHCI_CHANGE_NONAUTO_PXIE_READ_FAILED    (0x00000EAB)  /* Failed to read P0IE while changing non-auto setting */
#define PL_LOGINFO_SUB_CODE_AHCI_CHANGE_NONAUTO_PXIE_WRITE_FAILED   (0x00000EAC)  /* Failed to write P0IE while changing non-auto setting */
#define PL_LOGINFO_SUB_CODE_AHCI_BAD_PORT_NUMBER                    (0x00000EAD)  /* AHCI device gave interrupt for bad port number */
#define PL_LOGINFO_SUB_CODE_AHCI_BAD_TAG_INVALID_SACTIVE            (0x00000EAE)  /* Device's SActive doesn't match expected value */

/* SUB Codes for stuck link conditions (0xF00-0xF10). */
#define PL_LOGINFO_SUB_CODE_LINK_HUNG                               (0x00000F00)  /* TM sent to a device handle connected to a stuck link. */

#define PL_LOGINFO_SUB_CODE_PWRMGMT_DEVICE_NOT_CONNECTED            (0x00000F22)  /* A current phy power condition change request was denied because no device is attached to the specified phy. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_IOC_PARTIAL_SLUMBER_DISABLED    (0x00000F24)  /* A current phy power condition change request was denied because power management is disabled for the specified phy. Check that the appropriate bits are asserted in the SAS I/O Unit Page 5 ControlFlags field that corresponds to the specified phy. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_DRIVE_PARTIAL_SLUMBER_DISABLED  (0x00000F26)  /* A current phy power condition change request was denied because the end device does not support power management. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_STATE_PROHIBITS_PHY_POWER_MODE  (0x00000F2A)  /* A current phy power condition change request was denied because the request would transition the phy directly from Partial mode to Slumber mode, or conversely. To enter the desired state, wake up the phy by changing to ACTIVE phy power condition. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_HW_BUSY_RETRY_REQ_LATER         (0x00000F2C)  /* A current phy power condition change request failed because the IOC or the drive transmitted a frame before the transition could complete. Retry the request at a later time. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_QUEUE_FULL_RETRY_REQ_LATER      (0x00000F2E)  /* A current phy power condition change request failed because the firmware's policy change request queue is full. Retry the request at a later time. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_UNDEFINED_STATUS_CODE           (0x00000F30)  /* A power management policy change request failed because of an unknown error for which no suitable translation exists. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_SLUMBER_WAKEUP_FAILED           (0x00000F40)  /* A wake-up from Slumber mode failed due to a hardware configuration issue with the IOC's GigaBlaze(R) transceiver. An internal task management task aborts any outstanding I/Os, resets the phy and sets a flag to prevent future transitions into Slumber mode. */
//#define PL_LOGINFO_SUB_CODE_PWRMGMT_PARTIAL_TRANSITION_FAILED       (0x00000F50)  /* A wake-up from Partial mode failed due to a hardware issue (BT1656). The firmware initiates an internal management task that aborts any outstanding I/Os, resets the phy, and sets a flag to prevent future transitions into Partial power management mode. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_PORT_WIDTH_MOD_DISALLOWED       (0x00000F60)  /* A request to control the port width modulation of a group failed because the group cannot be controlled at this time. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_HOST_IS_NOT_CONTROLLING_GROUP   (0x00000F62)  /* A host request to relinquish control of a port width modulation group failed because the host was not previously in control. */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_NEW_AND_OLD_REGIONS_THE_SAME    (0x00000F63)  /* A host request to alter a port's modulation failed because the requested modulation level is the same as the current level (refer to SAS I/O Unit Page 6). */
#define PL_LOGINFO_SUB_CODE_PWRMGMT_GROW_OPERATION_FAILED           (0x00000F64)  /* A host-initiated grow request failed because the phys did not properly link up. */

#define PL_LOGINFO_SUB_CODE_DSCVRY_SATA_INIT_TIMEOUT                (0x00001000)  /* SATA initialization time-out. */


#define PL_LOGINFO_SUB_CODE_PCIE_ERR_FATAL_MALFORMED_TLP_RECIEVED         (0x00001010)    /* PCIe uncorrectable fatal error - Malformed TLP - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_FATAL_RECIEVER_OVERFLOW_ERROR        (0x00001011)    /* PCIe uncorrectable fatal error - recieveroverflow error status - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_FATAL_COMPLETION_TIMEOUT_ERROR       (0x00001012)    /* PCIe uncorrectable fatal error - completion timeout - occured */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_FATAL_FLOW_CONTROL_PROT_ERROR        (0x00001013)    /* PCIe uncorrectable fatal error - flow control protocol error - occured */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_FATAL_DATA_LINK_PROT_ERROR           (0x00001014)    /* PCIe uncorrectable fatal error - data link protocol error  - occured */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_FATAL_SURPRISE_DOWN_ERROR            (0x00001015)    /* PCIe uncorrectable fatal error - surprise down error  - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_FATAL_ERROR_MSG_FATAL_RECIEVED       (0x00001016)    /* PCIe uncorrectable fatal error message recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_NFATAL_TLP_PREFIX_BLOCKED_ERROR      (0x00001017)    /* PCIe uncorrectable non-fatal error - TLP Prefix blocked error - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_NFATAL_UNSUPPORTED_REQUEST_ERROR     (0x00001018)    /* PCIe uncorrectable non-fatal error -unsupported request - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_NFATAL_ECRC_ERROR                    (0x00001019)    /* PCIe uncorrectable non-fatal error - ECRC error - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_NFATAL_UNEXPECTED_COMPLETION_ERROR   (0x0000101A)    /* PCIe uncorrectable non-fatal error - unexpected completion - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_NFATAL_COMPLETOR_ABORT_ERROR         (0x0000101B)    /* PCIe uncorrectable non-fatal error - completor abort error - recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_NFATAL_POISONED_TLP_ERROR            (0x0000101C)    /* PCIe uncorrectable non-fatal error - poisoned TLP -recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_NFATAL_ERROR_MSG_NFATL_RECIEVED      (0x0000101D)    /* PCIe uncorrectable non-fatal error message  recieved */
#define PL_LOGINFO_SUB_CODE_PCIE_ERR_CORR_ERR_MSG_CNTER_THRSHLD_REACHED   (0x0000101E)    /* PCIe correctable error message counter threshold reached */
#define PL_LOGINFO_SUB_CODE_PCI_ERROR_AER_FATAL                           (0x0000101F)    /* PCIe root port detected FATAL error or received ERR_FATAL error message. */
#define PL_LOGINFO_SUB_CODE_PCI_ERROR_AER_NON_FATAL                       (0x00001020)    /* PCIe root port detected NON-FATAL error or received ERR_NON-FATAL error message. */
#define PL_LOGINFO_SUB_CODE_RCC_SHEX_STALLED                              (0x00001021)    /* PCIe root port detected an error that also caused SHEX block to stall */


#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_FRAME_FAILURE                 (0x00200000)  /* Cannot acquire an SMP frame. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_READ_ERROR                    (0x00200010)  /* Error occurred on an SMP read. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_WRITE_ERROR                   (0x00200020)  /* Error occurred on an SMP write. */
#define PL_LOGINFO_CODE_ENCL_MGMT_NOT_SUPPORTED_ON_ENCL             (0x00200040)  /* Enclosure management services are not available for this WWID. */
#define PL_LOGINFO_CODE_ENCL_MGMT_ADDR_MODE_NOT_SUPPORTED           (0x00200050)  /* Address mode is not supported. */
#define PL_LOGINFO_CODE_ENCL_MGMT_BAD_SLOT_NUM                      (0x00200060)  /* SEP message contains an invalid slot number. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SGPIO_NOT_PRESENT                 (0x00200070)  /* SGPIO not present or enabled. */
#define PL_LOGINFO_CODE_ENCL_MGMT_GPIO_NOT_CONFIGURED               (0x00200080)  /* Unconfigured GPIO. */
#define PL_LOGINFO_CODE_ENCL_MGMT_GPIO_FRAME_ERROR                  (0x00200090)  /* GPIO cannot allocate a frame. */
#define PL_LOGINFO_CODE_ENCL_MGMT_GPIO_CONFIG_PAGE_ERROR            (0x002000A0)  /* GPIO failed configuration page request. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SES_FRAME_ALLOC_ERROR             (0x002000B0)  /* Cannot get a frame for the SES command. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SES_IO_ERROR                      (0x002000C0)  /* I/O execution error. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SES_RETRIES_EXHAUSTED             (0x002000D0)  /* SEP I/O retries exhausted. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SMP_FRAME_ALLOC_ERROR             (0x002000E0)  /* Cannot get a frame for the SMP command. */
#define PL_LOGINFO_CODE_ENCL_MGMT_UNAUTHORIZED_DEVICE               (0x002000F0)  /* Slot has an unauthorized device. */
#define PL_LOGINFO_CODE_ENCL_MGMT_SES_PAGE_A_MAP_ERROR              (0x002000F1)  /* SEP Request with Invalid DevHandle for Mapping SES Page 0x0A */

#define PL_LOGINFO_DA_SEP_NOT_PRESENT                               (0x00200100)  /* SEP not present when the message was received. */
#define PL_LOGINFO_DA_SEP_SINGLE_THREAD_ERROR                       (0x00200101)  /* Can only accept one message at a time. */
#define PL_LOGINFO_DA_SEP_ISTWI_INTR_IN_IDLE_STATE                  (0x00200102)  /* ISTWI interrupt received when idle. */
//#define PL_LOGINFO_DA_SEP_RECEIVED_NACK_FROM_SLAVE                (0x00200103)  /* FIRMWARE received NAK from SEP.  SEP is busy */
#define PL_LOGINFO_DA_SEP_DID_NOT_RECEIVE_ACK                       (0x00200104)  /* SEP did not receive ACK (last received bit is 1). */
#define PL_LOGINFO_DA_SEP_BAD_STATUS_HDR_CHKSUM                     (0x00200105)  /* SEP stopped or sent a bad checksum in the header. */
#define PL_LOGINFO_DA_SEP_STOP_ON_DATA                              (0x00200106)  /* SEP stopped when transferring data. */
#define PL_LOGINFO_DA_SEP_STOP_ON_SENSE_DATA                        (0x00200107)  /* SEP stopped when transferring sense data. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_SCSI_STATUS_1                 (0x00200108)  /* SEP returned an unknown SCSI status. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_SCSI_STATUS_2                 (0x00200109)  /* SEP returned an unknown SCSI status. */
#define PL_LOGINFO_DA_SEP_CHKSUM_ERROR_AFTER_STOP                   (0x0020010A)  /* SEP returned a bad checksum after a stop. */
#define PL_LOGINFO_DA_SEP_CHKSUM_ERROR_AFTER_STOP_GETDATA           (0x0020010B)  /* SEP returned a bad checksum after a stop while getting data. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_COMMAND                       (0x0020010C)  /* SEP does not support CDB opcode firmware location 1. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_COMMAND_2                     (0x0020010D)  /* SEP does not support CDB opcode firmware location 2. */
#define PL_LOGINFO_DA_SEP_UNSUPPORTED_COMMAND_3                     (0x0020010E)  /* SEP does not support CDB opcode firmware location 3. */

/* RESERVED
 * PL_LOGINFOs from 0x00210001 to 0x0021000A was used by SAS Port Virtualization
 * and should not be re-used.
 */

#define PL_LOGINFO_CODE_IO_REDUCED_FUNCTIONALITY_RETRY              (0x00220000)   /* I/O aborted for the device behind an expander in the reduced functionality state. */
#define PL_LOGINFO_CODE_IO_QUIESCED_BT1680_RETRY                    (0x00220001)   /* I/O aborted for the device in the BT1680 quiesce I/O state. */

#define PL_LOGINFO_CODE_SATL_SPI_SSPL_READ_FAILED                   (0x00220010)   /* Memory buffer read failed. */
#define PL_LOGINFO_CODE_SATL_SPI_SSPL_LISTLENGTH_TOO_BIG            (0x00220011)   /* List length returned by the device exceeds the maximum list length. */
#define PL_LOGINFO_CODE_SATL_SPI_SSPL_WRITE_FAILED                  (0x00220012)   /* Memory buffer write failed. */

#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY                        (0x00230000)    /* Base host-based discovery error. See the following table for specific failures. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_OPERATION_INVALID      (0x00230001)    /* An MPI HBD action message failed because the operation field contained an invalid value. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEVH_OUT_OF_RANGE      (0x00230002)    /* An MPI HBD action message failed because the device handle specified was not within the allowable range and is, therefore, invalid. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEVH_IN_USE            (0x00230003)    /* An MPI HBD action message failed because of an attempt to allocate an already established device handle. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEVH_NOT_ALLOC         (0x00230004)    /* An MPI HBD action message failed because of an attempt to update or remove a non-existent device handle. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_PARENT_INVALID         (0x00230005)    /* An MPI HBD action message failed because of an attempt to add or update a device handle whose parent does not exist. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_DEV_INFO_INVALID       (0x00230006)    /* An MPI HBD action message failed because of an attempt to add a device with invalid HBDDeviceInfo parameters. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_PORT_INVALID           (0x00230007)    /* An MPI HBD action message failed because of an attempt to add a device to a non-existent port. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_MAX_RATE_INVALID       (0x00230008)    /* An MPI HBD action message failed because of an attempt to add or update a device with a MaxRate value not equal to 0x8, 0x9, or 0xA. */
#define PL_LOGINFO_CODE_HOST_BASED_DISCOVERY_IO_TM_OUTSTANDING      (0x00230009)    /* An MPI HBD action message failed because of an attempt to remove a device with an outstanding IO or Task Management. */

#define PL_LOGINFO_FAST_PATH_ENGINE                                 (0x00240000)    /* FPE - Generic FastPath Engine LogInfo. */
#define PL_LOGINFO_FAST_PATH_ENGINE_IOEXCP_INVALID_DESCR_TYPE       (0x00240001)    /* FPE - Received I/O Exception from SAS-3.0 for MID with Invalid Descriptor Type. */
#define PL_LOGINFO_FAST_PATH_ENGINE_IOEXCP_INVALID_DEVICE_HANDLE    (0x00240002)    /* FPE - Received I/O Exception from SAS-3.0 for MID with Invalid Device Handle. */
#define PL_LOGINFO_FAST_PATH_ENGINE_IOEXCP_VF_MISMATCH              (0x00240003)    /* FPE - Received I/O Exception from SAS-3.0 for MID with VF Mismatch. */
#define PL_LOGINFO_FAST_PATH_ENGINE_REQUESTOR_TIMER_OVERFLOW_ERROR  (0x00240004)    /* FPE - The FPE Requestor has specified an Add Timer operation with an Add Time-out Value that corresponds to a timer bucket that is equal to 0xFFFF. */
#define PL_LOGINFO_FAST_PATH_ENGINE_COMPLETOR_TIMER_OVERFLOW_ERROR  (0x00240005)    /* FPE - The FPE Completor has specified a Remove Timer operation with a Remove Timer that corresponds to a timer bucket that is equal to 0x0000. */
#define PL_LOGINFO_FAST_PATH_ENGINE_REQUESTOR_TIMER_ERROR_MST_STALE (0x00240006)    /* FPE - The FPE Requestor encountered a TimerError (Timeout) for an FP IO started from an FP Request FIFO and was unable to update the MST. */

#define PL_LOGINFO_IOPI_MEMORY_MOVE_FAILURE                         (0x00250000)    /* iopiMemoryMove() did not return successful status. */

#define PL_LOGINFO_CODE_PCI_BAD_SGL_ERROR                           (0x00260000)    /* Generic PCIe device bad SGL error */
#define PL_LOGINFO_CODE_FXENGINE_BAD_NEXT_CHAIN_SEGMENT             (0x00260001)    /* Next Segment Format field in a chain SGE has an invalid value. */
#define PL_LOGINFO_CODE_PCI_SGL_BAD_ALIGNMENT                       (0x00260003)    /* IO failed due to an improperly aligned address in SGE. */

#define PL_LOGINFO_SUB_CODE_AHCI_PORT_CONNECT_CHANGE                (0x00260004)    /* AHCI port connect change occurred. */
#define PL_LOGINFO_SUB_CODE_AHCI_INVALID_SGE                        (0x00260006)    /* Unexpected SGE format - chain was expected */

#define PL_LOGINFO_CODE_PCI_BAD_SGLOFFSET                           (0x00260007)    /* IO failed due to a bad SGLOffset in the MPI request message. */
#define PL_LOGINFO_CODE_FXENGINE_BAD_DLAS_PLBNTA                    (0x00260008)    /* Data Location Address Space in an SGE for PCIe target contains PLB-NTA. */

#define PL_LOGINFO_CODE_NVME_FXENGINE_BAD_SGE_LENGTH                (0x00260009)    /* SG element length is zero. */
#define PL_LOGINFO_CODE_FXENGINE_DATALEN_GT_SGLLEN                  (0x0026000B)    /* Remaining DataLength is greater than SGL length. */
#define PL_LOGINFO_CODE_NVME_FXREQUESTOR_ERROR                      (0x0026000C)    /* NVMe FxRequestor Error */
#define PL_LOGINFO_CODE_NVME_FXCOMPLETOR_ERROR                      (0x0026000D)    /* NVMe FxCompletor Error */
#define PL_LOGINFO_CODE_PCI_REQUEST_ERROR                           (0x0026000E)    /* PCIe device request error */
#define PL_LOGINFO_CODE_PCI_COMPLETE_ERROR                          (0x0026000F)    /* PCIe device complete error */
#define PL_LOGINFO_CODE_PCI_AHCI_NO_TAG_AVAILABLE                   (0x00260010)    /* AHCI Device failed to allocate a tag */
#define PL_LOGINFO_CODE_PCI_SGE_DMA_FAILED                          (0x00260011)    /* PCIe Device failed to DMA an SGE */
#define PL_LOGINFO_CODE_PCI_SGE_INVALID_CHAIN_TYPE                  (0x00260012)    /* PCIe Device failed due to invalid chain type */
#define PL_LOGINFO_CODE_PCI_SGE_INVALID_BYTE_COUNT                  (0x00260013)    /* PCIe Device failed due to invalid byte count according to protocol rules */
#define PL_LOGINFO_CODE_PCI_SGE_TOO_MANY_SGES_FOR_SECTOR            (0x00260014)    /* Too many SGEs for one sector of data so does not fit in PRDT */

#define PL_LOGINFO_CODE_PCI_SGL_BIT_BUCKET                          (0x00260015)    /* IEEE bit bucket SGE was sent to a backend PCIe device. */
#define PL_LOGINFO_CODE_PCI_FXCORE_BAD_DEVHANDLE                    (0x00260016)    /* Invalid devhandle in a request to a backend PCIe device. */
#define PL_LOGINFO_CODE_SGL_TRANS_INVALID_PLB_ADDR                  (0x00260017)    /* Invalid PLB address in SGE for backend PCIe device */
#define PL_LOGINFO_CODE_PCI_PRD_INVALID_BYTE_COUNT                  (0x00260018)    /* PCIe Device failed due to invalid byte count in source PRD according to protocol rules */
#define PL_LOGINFO_CODE_PCI_PRD_BAD_ALIGNMENT                       (0x00260019)    /* I/O failed because of an improperly aligned address in the source PRD. */
#define PL_LOGINFO_CODE_AHCI_PRDT_TO_PRDT_NOT_SUPP                  (0x0026001A)    /* IO failed due IO with chain to PRDT that was not hardware automated */
#define PL_LOGINFO_CODE_PCI_DEVHANDLE_MISMATCH_COMPLETION           (0x0026001B)    /* Encountered PCI IO completion with IOReq/MST device handle mismatch */

#define PL_LOGINFO_CODE_SOP_PQI_FXENGINE_BAD_SGE_TYPE               (0x0026001C)    /* Unexpected SGE type (not chain and not data) */
#define PL_LOGINFO_CODE_AHCI_SGL_INVALID_INT_ADDRESS                (0x0026001D)    /* SGL translation for AHCI failed due to an internal address that did not map to PCI address */
#define PL_LOGINFO_CODE_SOP_PQI_TO_PQI_XLATE_NOT_SUPP               (0x0026001E)    /* SGL translation for SOP/PQI failed due to detecting PQI chain buffer */
#define PL_LOGINFO_CODE_SOP_PQI_SGL_INVALID_INT_ADDRESS             (0x0026001F)    /* SGL translation for SOP/PQI failed due to internal address that did not map to PCI address */

#define PL_LOGINFO_CODE_SOP_PQI_IO_COMPLETE_WITH_ERROR              (0x00260020)    /* Encapsulated IO Request to the SOP devie completed with error. */
#define PL_LOGINFO_CODE_SOP_PQI_FXENGINE_ENCAP_LENGTH_ZERO          (0x00260021)    /* Stang generated exception for encapsulated IU_Length Zero. */

#define PL_LOGINFO_CODE_PCI_CONFIG_MODIFIED_LINK_PHY_MAPPING        (0x00260100)    /* Link to phy mapping was found to be modified from the start of day configuration on write of PCIe IO Unit Page 1. The lowest 5 bits indicate the offending phy. */
#define PL_LOGINFO_CODE_PCI_CONFIG_LINK_SPEED_CHG_FAILED            (0x00260120)    /* An error occurred while changing PCIe link rate on write of PCIe IO Unit Page 1. The lowest 5 bits indicate the offending link. */
#define PL_LOGINFO_CODE_PCI_CONFIG_LINK_WIDTH_CHG_FAILED            (0x00260140)    /* An error occurred while changing PCIe link width on write of PCIe IO Unit Page 1. The lowest 5 bits indicate the offending link. */

/* NVMe firmware SGL translation errors */
#define PL_LOGINFO_CODE_NVME_SGE_NOT_DWORD_ALIGNED                  (0x00260400)    /* SGE contains an address that is not dWord aligned. */
#define PL_LOGINFO_CODE_NVME_SGE_ADDRESS_LEN_TOO_LARGE              (0x00260401)    /* SGE address plus length is greater than 64 bits. */
#define PL_LOGINFO_CODE_NVME_SGE_LENGTH_ZERO                        (0x00260402)    /* SGE length is zero. */
#define PL_LOGINFO_CODE_NVME_SGE_IEEE_BIT_BUCKET                    (0x00260403)    /* IEEE bit bucket SGEs are not supported for NVMe devices. */
#define PL_LOGINFO_CODE_NVME_SKIPCOUNT_EXCEEDS_SGL_LENGTH           (0x00260404)    /* SkipCount exceeds total SGL length. */
#define PL_LOGINFO_CODE_NVME_ILLEGAL_SGE_WITHIN_DEVSECTOR           (0x00260405)    /* Unaligned SGE boundary within a device sector. */
#define PL_LOGINFO_CODE_NVME_SGE_TRANSLATE_FETCH_FAILED             (0x00260406)    /* SGE translation code failed to fetch an SGE. */
#define PL_LOGINFO_CODE_NVME_ILLEGAL_SGE_WITHIN_DEVSECTOR_2         (0x00260408)    /* Unaligned SGE boundary within a device sector. */
#define PL_LOGINFO_CODE_NVME_SGL_TOO_SHORT                          (0x00260409)    /* Total SGL length is smaller than total transfer length. */
#define PL_LOGINFO_CODE_NVME_END_OF_SGL_MISALIGNED                  (0x0026040A)    /* The SGL does not end on a device sector boundary. */
#define PL_LOGINFO_CODE_NVME_NONSENSICAL_MIXED_IEEE_NATIVE_SGL      (0x0026040B)    /* Please generate an SGL that isn't so goofy. */
#define PL_LOGINFO_CODE_NVME_NATIVE_SKIPCOUNT_BADLY_ALIGNED         (0x0026040C)    /* SkipCount is not device sector size aligned. */
#define PL_LOGINFO_CODE_NVME_NATIVE_SKIPCOUNT_TOO_LARGE             (0x0026040D)    /* SkipCount exceeds total SGL length with a native SGL. */
#define PL_LOGINFO_CODE_NVME_ILLEGAL_TRANSITION_TO_NATIVE           (0x0026040E)    /* Mixed IEEE/native SGL does not follow the required format. */
#define PL_LOGINFO_CODE_NVME_SGL_INVALID_INT_ADDRESS                (0x0026040F)    /* SGL translation for NVMe failed due to an internal address that did not map to PCI address. */
#define PL_LOGINFO_CODE_NVME_SGL_TOO_MANY_PRP_ENTRIES               (0x00260410)    /* Translated SGL exceeds NVMe command-specific limit of 2 PRP entries. */
#define PL_LOGINFO_CODE_NVME_NONRW_SGL_TOO_LARGE                    (0x00260411)    /* Translated SGL exceeds the maximum PRP list buffer size. */
#define PL_LOGINFO_CODE_NVME_NONRW_SGL_BADLY_FORMED                 (0x00260412)    /* Incoming SGE for a non read write command has illegal alignment or size. */
#define PL_LOGINFO_CODE_NVME_ILLEGAL_SGE_WITHIN_DEVSECTOR_3         (0x00260413)    /* Unhandled SGE boundary within a device sector. */
#define PL_LOGINFO_CODE_NVME_ILLEGAL_SGE_WITHIN_DEVSECTOR_4         (0x00260414)    /* Unaligned SGE boundary within a device sector. */
#define PL_LOGINFO_CODE_NVME_SGL_UNEXPECTED_EOL                     (0x00260415)    /* The SGL ended before the transfer length was satisfied. */

#define PL_LOGINFO_CODE_NVME_NATIVE_SGL_VALIDATION_FAILED           (0x00260420)    /* Native format SGL validation failed. */

#define PL_LOGINFO_CODE_SATL                                        (0x00270000)    /* From SCSI to ATA Translation Layer (SATL). */
#define PL_LOGINFO_CODE_SATL_BAD_CDB_LENGTH                         (0x00270001)    /* CDB length is incorrect for the SCSI command sent. */
#define PL_LOGINFO_CODE_SATL_ATA_PT_BAD_PROTOCOL                    (0x00270002)    /* ATA Passthrough: Non-supported protocol. */
#define PL_LOGINFO_CODE_SATL_ATA_PT_PROTO_DIRECTION_MISMATCH        (0x00270003)    /* ATA Passthrough: Protocol/Direction mismatch, or non-data with t_length != 0 */
#define PL_LOGINFO_CODE_SATL_ATA_PT_MULTICNT_NONSUPPORTED_CMD       (0x00270004)    /* ATA Passthrough: Multiple count field set, but command is not a multiple command. */

/****************************************************************************/
/* IR LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IR            */
/****************************************************************************/

/* See iopiIocLogInfo.h */

/****************************************************************************/
/* Defines for convenience                                                  */
/****************************************************************************/
#define IOC_LOGINFO_PREFIX_IOP      ((MPI2_IOCLOGINFO_TYPE_SAS << MPI2_IOCLOGINFO_TYPE_SHIFT) | IOC_LOGINFO_ORIGINATOR_IOP)
#define IOC_LOGINFO_PREFIX_PL       ((MPI2_IOCLOGINFO_TYPE_SAS << MPI2_IOCLOGINFO_TYPE_SHIFT) | IOC_LOGINFO_ORIGINATOR_PL)
#define IOC_LOGINFO_PREFIX_IR       ((MPI2_IOCLOGINFO_TYPE_SAS << MPI2_IOCLOGINFO_TYPE_SHIFT) | IOC_LOGINFO_ORIGINATOR_IR)

#endif /* end of file */

